/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/result-to-error.ts > TAP > actual/expected > message 1`] = `
actual/expected
`

exports[`test/result-to-error.ts > TAP > actual/expected > properties 1`] = `
Object {
  "actual": 1,
  "expected": 2,
  "stack": "",
}
`

exports[`test/result-to-error.ts > TAP > error, no details > message 1`] = `
error, no details
`

exports[`test/result-to-error.ts > TAP > error, no details > properties 1`] = `
Object {
  "stack": "",
}
`

exports[`test/result-to-error.ts > TAP > found/wanted > message 1`] = `
found/wanted
`

exports[`test/result-to-error.ts > TAP > found/wanted > properties 1`] = `
Object {
  "actual": 1,
  "expected": 2,
  "stack": "",
}
`

exports[`test/result-to-error.ts > TAP > found/wanted diff > message 1`] = `
found/wanted diff
[48;2;172;62;163m[38;2;255;229;241m--- expected                                                               [39m[49m[m
[48;2;58;117;0m[38;2;242;255;229m+++ actual                                                                 [39m[49m[m
[48;2;34;34;34m[38;2;117;158;239m@@ -1,9 +1,7 @@                                                            [39m[49m[m
[48;2;51;51;51m[38;2;204;204;204m Object {                                                                  [39m[49m[m
[48;2;51;51;51m[38;2;204;204;204m   "some": Object {                                                        [39m[49m[m
[48;2;51;51;51m[38;2;204;204;204m     "object": Object {                                                    [39m[49m[m
[48;2;172;62;163m[38;2;255;229;241m-      "i": "wanted",                                                      [39m[49m[m
[48;2;58;117;0m[38;2;242;255;229m+      "you": Object {                                                     [39m[49m[m
[48;2;58;117;0m[38;2;242;255;229m+        "gave": "me",                                                     [39m[49m[m
[48;2;58;117;0m[38;2;242;255;229m+      },                                                                  [39m[49m[m
[48;2;51;51;51m[38;2;204;204;204m     },                                                                    [39m[49m[m
[48;2;51;51;51m[38;2;204;204;204m   },                                                                      [39m[49m[m
[48;2;51;51;51m[38;2;204;204;204m }                                                                         [39m[49m[m
| 
|     helpful snippet of source code
| -----------------------^
|     the better to find errors with!
`

exports[`test/result-to-error.ts > TAP > found/wanted diff > properties 1`] = `
Object {
  "stack": "",
}
`

exports[`test/result-to-error.ts > TAP > no name > message 1`] = `
unnamed test
`

exports[`test/result-to-error.ts > TAP > no name > message 2`] = `
unnamed test
`

exports[`test/result-to-error.ts > TAP > no name > message 3`] = `
unnamed test
`

exports[`test/result-to-error.ts > TAP > no name > properties 1`] = `
Object {
  "doesNotHave": Array [
    "name",
  ],
  "stack": "",
}
`

exports[`test/result-to-error.ts > TAP > no name > properties 2`] = `
Object {
  "stack": "    at /some/file.js:1:2",
}
`

exports[`test/result-to-error.ts > TAP > no name > properties 3`] = `
Object {
  "stack": "    at /some/file.js:1:2",
}
`

exports[`test/result-to-error.ts > TAP > result with actual/expected diff > message 1`] = `
result with actual/expected diff
[48;2;172;62;163m[38;2;255;229;241m--- expected                                                               [39m[49m[m
[48;2;58;117;0m[38;2;242;255;229m+++ actual                                                                 [39m[49m[m
[48;2;34;34;34m[38;2;117;158;239m@@ -1,9 +1,7 @@                                                            [39m[49m[m
[48;2;51;51;51m[38;2;204;204;204m Object {                                                                  [39m[49m[m
[48;2;51;51;51m[38;2;204;204;204m   "some": Object {                                                        [39m[49m[m
[48;2;51;51;51m[38;2;204;204;204m     "object": Object {                                                    [39m[49m[m
[48;2;172;62;163m[38;2;255;229;241m-      "i": "expected",                                                    [39m[49m[m
[48;2;58;117;0m[38;2;242;255;229m+      "you": Object {                                                     [39m[49m[m
[48;2;58;117;0m[38;2;242;255;229m+        "actually": "gave me",                                            [39m[49m[m
[48;2;58;117;0m[38;2;242;255;229m+      },                                                                  [39m[49m[m
[48;2;51;51;51m[38;2;204;204;204m     },                                                                    [39m[49m[m
[48;2;51;51;51m[38;2;204;204;204m   },                                                                      [39m[49m[m
[48;2;51;51;51m[38;2;204;204;204m }                                                                         [39m[49m[m
`

exports[`test/result-to-error.ts > TAP > result with actual/expected diff > properties 1`] = `
Object {
  "stack": "",
}
`

exports[`test/result-to-error.ts > TAP > this is fine > message 1`] = `
this is fine
`

exports[`test/result-to-error.ts > TAP > this is fine > properties 1`] = `
Object {
  "stack": "",
}
`
