html,
body,
div,
a {
  margin: 0;
  padding: 0;
}
* {
  box-sizing: border-box;
}
html {
  font-family: Open Sans, Arial, Helvetica, sans-serif;
  line-height: 1.15;
  background-color: #f8f8f8;
  font-size: 16px;
  /* make room for top menu */
  scroll-padding-top: 5rem;
}
@media screen and (max-width: 770px) {
  html {
    scroll-padding-top: 10rem;
  }
}
@media screen and (max-width: 650px) {
  html {
    scroll-padding-top: 17rem;
  }
}
@media screen and (max-width: 300px) {
  html {
    scroll-padding-top: 21rem;
  }
}

h1,
h2,
h3,
h4,
h5,
h6 {
  font-family: Trebuchet MS, Arial, Helvetica, sans-serif!important;
}
h1 a.selflink,
h2 a.selflink,
h3 a.selflink,
h4 a.selflink,
h5 a.selflink,
h6 a.selflink {
  color: #ccc;
  text-decoration: none;
  padding-left: 0.5rem;
}

li,
p {
  font-size: 16px;
  color: #333;
  line-height: 1.6;
}

blockquote {
  margin: 0;
  padding: 0.5rem 1.5rem;
  background: #fff;
  border: 1px solid #f90;
  border-radius: 4px;
  font-size: 1.1;
}

a {
  color: #204dff;
  font-weight: 600;
  text-decoration: none;
}
a:active,
a:focus,
a:hover {
  color: #09f;
  text-decoration: underline;
}

#top {
  box-sizing: border-box;
  display: block;
  background-color: rgb(255, 255, 255);
  box-shadow: rgb(184, 179, 179) 0 0 10px 0;
  -webkit-box-align: center;
  align-items: center;
  font-family: 'Trebuchet MS', Arial, Helvetica, sans-serif;
  z-index: 3;
  position: sticky;
  top: 0;
}
#top-inner {
  box-sizing: border-box;
  display: flex;
  width: 100%;
  margin-top: auto;
  -webkit-box-pack: justify;
  justify-content: space-between;
  -webkit-box-align: center;
  align-items: center;
  padding: 0 10px;
  background: url(/static/small-logo.png) left center no-repeat;
  background-size: 78px 72px;
}
#top h1 a {
  display: flex;
  align-items: bottom;
  gap: 10px;
  justify-content: start;
  font-size: 2rem;
  flex-wrap: wrap;
  white-space: nowrap;
  vertical-align: bottom;

  text-transform: uppercase;
  color: rgb(51, 51, 51);
  margin: 0;
  font-weight: 700;
  font-size: 26px;
  padding: 0 10px 0 70px;
  letter-spacing: 1px;
  text-shadow: rgba(136, 136, 136, 0.32) 1px -2px 1px;
  text-decoration: none;
}
#top h1 a:hover,
#top h1 a:active,
#top h1 a:focus {
  color: #09f;
  text-shadow: none;
}
#top h1 a small {
  text-transform: none;
  font-size: 13px;
  position: relative;
  padding-right: 12ex;
  padding-top: 8px;
}
#top h1 a small span {
  position: absolute;
  bottom: 4px;
  left: 0;
}

#top #site-nav a {
  padding: 5px 10px;
  padding-right: 10px;
  margin: 0;
  line-height: 1.5;
  font-weight: 600;
  text-decoration-line: none;
  font-size: 14px;
  letter-spacing: 1px;
  display: block;
  font-family: 'Trebuchet MS', Arial, Helvetica, sans-serif;
  color: rgb(51, 51, 51);
  white-space: nowrap;
}
#top #site-nav a:hover,
#top #site-nav a:focus,
#top #site-nav a:active {
  color: #09f;
}

#top nav ul,
#top nav li {
  display: block;
  margin: 0;
  padding: 0;
  list-style: none;
}
#top nav li.current-page a {
  background: #0ff;
  border-radius: 4px;
}
#top nav ul {
  align-items: flex-end;
  display: flex;
  flex-wrap: wrap;
  flex-direction: row;
  justify-content: end;
}

.hero__Container {
  background-color: rgb(233, 233, 233);
  padding: 20px 0;
}
.hero__Content {
  display: flex;
  position: relative;
  width: 800px;
  max-width: 100%;
  margin: -90px auto 0;
  padding: 0 0 0;
  min-height: fit-content(900px);
}
.hero__Circles {
  width: 100%;
  position: absolute;
  z-index: 0;
}
.hero__OuterCircle {
  background-color: rgb(241, 241, 241);
  position: relative;
  border-top-left-radius: 50%;
  border-top-right-radius: 50%;
  border-bottom-right-radius: 50%;
  border-bottom-left-radius: 50%;
  width: 100%;
  height: auto;
  padding-top: 100%;
  margin: 0 auto;
  top: 0;
}
.hero__InnerCircle {
  background-color: rgb(248, 248, 248);
  position: absolute;
  border-top-left-radius: 50%;
  border-top-right-radius: 50%;
  border-bottom-right-radius: 50%;
  border-bottom-left-radius: 50%;
  width: 80%;
  height: auto;
  padding-top: 80%;
  top: 0;
  top: 10%;
  left: 0;
  margin: auto;
  box-shadow: rgba(51, 51, 51, 0.05) 1px 1px 15px 0;
  right: 0;
}
.hero__InnerBits {
  position: relative;
  max-width: 600px;
  left: 0;
  right: 0;
  margin: 25% auto 50px;
  text-align: center;
}
@media screen and (min-width: 500px) {
  .hero__InnerBits {
    margin-bottom: 10%;
  }
}
.hero__Logo {
  box-sizing: border-box;
  margin: 0 auto;
  height: auto;
  max-width: 60%;
  text-align: center;
}
.hero__Title {
  font-size: 18px;
  text-align: center;
  margin-bottom: 30px;
}

.index .button-link {
  color: rgb(255, 255, 255);
  background-color: rgb(230, 131, 255);
  text-decoration: none;
  text-align: center;
  border-top-left-radius: 40px;
  border-top-right-radius: 40px;
  border-bottom-right-radius: 40px;
  border-bottom-left-radius: 40px;
  padding-top: 15px;
  font-size: 14px;
  display: block;
  width: 180px;
  margin: 20px auto;
  transition-duration: 0.5s;
  transition-timing-function: ease;
  transition-delay: 0s;
  transition-property: background-color;
  padding: 15px;
  font-weight: 600;
}
.index a.button-link:hover,
.index a.button-link:focus,
.index a.button-link:active {
  background-color: rgb(214, 48, 255);
}

.index .features {
  padding: 40px 20px;
  margin: -80px auto 0;
  background-color: rgb(255, 255, 255);
  box-shadow: rgba(51, 51, 51, 0.07) 0 0 11px 5px;
  position: relative;
  z-index: 2;
}
.index .features h2 {
  font-size: 20px;
  text-align: center;
}
.index .features h3 {
  font-size: 16px;
  text-align: center;
}
.index .features__Content {
  display: flex;
  max-width: 900px;
  margin: auto;
  flex-direction: column;
}
.index .features__Columns {
  display: flex;
  flex-direction: row;
  justify-content: space-between;
}
.index .features__Columns section {
  display: flex;
  flex-direction: column;
  padding: 0 30px;
  max-width: 500px;
  width: 50%;
}
.index .features__Columns section img {
  padding-bottom: 20px;
  max-width: 150px;
  max-height: 150px;
  margin: 0 auto;
  height: auto;
}
@media screen and (max-width: 768px) {
  .index .features__Columns {
    flex-direction: column;
    align-items: center;
    width: auto;
  }
  .index .features__Columns section {
    max-width: 100%;
    display: flex;
    flex-direction: row;
  }
  .index .features h3 {
    text-align: left;
  }
}
@media screen and (max-width: 500px) {
  .index .features__Columns section {
    flex-direction: column;
    width: auto;
  }
  .index .features h3 {
    text-align: center;
  }
}

.why-tap .why-tap__Content {
  max-width: 800px;
  margin: auto;
  padding: 40px 20px 1rem;
}

.EncircledImage__GreyCircle {
  width: 180px;
  height: 180px;
  background-color: rgb(241, 241, 241);
  border-top-left-radius: 50%;
  border-top-right-radius: 50%;
  border-bottom-right-radius: 50%;
  border-bottom-left-radius: 50%;
  margin: auto;
  position: relative;
}
.EncircledImage__WhiteCircle {
  width: 130px;
  height: 130px;
  background-color: rgb(255, 255, 255);
  border-top-left-radius: 50%;
  border-top-right-radius: 50%;
  border-bottom-right-radius: 50%;
  border-bottom-left-radius: 50%;
  margin-top: auto;
  top: 50%;
  margin-top: -65px;
  right: 0;
  left: 50%;
  margin-left: -65px;
  bottom: 0;
  position: absolute;
}
.EncircledImage__GreyCircle img {
  box-sizing: border-box;
  max-width: 100%;
  height: auto;
  width: 80px;
  position: absolute;
  top: 50%;
  margin-top: -48px;
  left: 50%;
  margin-left: -40px;
}

.index hr {
  display: block;
  height: 1px;
  margin: 1rem auto;
  width: 50%;
  background: 1px solid #666;
}

footer {
  box-shadow: inset 0 5px 5px #ddd;
  position: relative;
  background: #eee;
  padding: 1rem;
}
footer p {
  max-width: 800px;
  margin: 0.5rem auto;
  text-align: center;
  font-style: italic;
  font-size: 0.8em;
}

/* content stuff */
body.content main {
  display: flex;
  flex-direction: row;
  width: 100%;
  position: relative;
}
#content {
  overflow: auto;
  flex-basis: max-content;
  flex-grow: 1;
  min-width: 0;
  padding: 0 2rem 7rem;
}
#content pre {
  overflow-x: auto;
  overflow-y: hidden;
}
#content img,
#content pre {
  width: 100%;
}

#sidebar {
  min-height: calc(100vh - 4.5rem);
  min-width: 12rem;
  width: 12rem;
  padding-top: 1rem;
  padding-right: 1rem;
  padding-bottom: 0px;
  box-shadow: rgba(51, 51, 51, 0.19) 1px 0px 10px 1px;
  background-color: rgb(241, 241, 241);
}
#sidebar ul,
#sidebar li {
  padding: 0;
  margin: 0;
  list-style: none;
}
#sidebar a {
  border-radius: 4px;
  padding: 0.2rem 0 0.2rem 1rem;
  display: block;
  text-decoration: none;
  font-weight: normal;
  color: #333;
  overflow-x: hidden;
  font-size: 14px;
  font-family: 'Trebuchet MS', Arial, Helvetica, sans-serif;
}
#sidebar a:hover,
#sidebar a:focus,
#sidebar a:active {
  color: #09f;
}
#sidebar li.current-page > a {
  background: #ff9;
  color: #333;
}
#sidebar ul ul {
  padding-left: 0.5rem;
}
#sidebar nav.toc {
  margin-bottom: 1rem;
  padding-bottom: 1rem;
  border-bottom: 1px solid #ccc;
}

@media screen and (max-width: 600px) {
  body.content main {
    flex-direction: column;
  }
  #search {
    width: 100%;
    max-width: 100%;
  }
  #sidebar {
    min-height: 0;
    width: 100%;
    padding: 1rem;
    padding-top: 3rem;
  }
  #sidebar ul,
  #sidebar li {
    display: inline-block;
    padding: 0 !important;
  }
  #sidebar ul li a {
    display: inline-block;
    padding: 0 0.5rem 0 0;
  }
}

pre,
code,
kbd {
  font-family: Fira Code, Iosevka SS09 Web, Monoid, Jetbrains Mono,
    DejaVu Sans Code, Cascadia Code, Victor Mono, Lilex, monospace !important;
}
code {
  background: #fff;
  outline: 2px solid #eee;
  font-weight: normal;
  color: #2d2d2d;
  border-radius: 3px;
  padding: 0 2px;
  margin-right: 2px;
}
a code {
  color: inherit;
}
pre code {
  margin: 0;
  border: none;
  outline: none;
  background: none;
  color: inherit;
}
pre {
  border-radius: 3px;
  color: #ccc;
  background: #2d2d2d;
  padding: 1em;
  line-height: 1.3 !important;
}

body.not-found main {
  margin: 5rem auto;
  text-align: center;
  display: flex;
  flex-direction: column;
  width: min-content;
}
body.not-found main h1 {
  margin: 1rem 0 0;
}
.not-found main > div,
.not-found main h1,
.not-found main p {
  width: 100%;
}

#search {
  --pagefind-ui-scale: 0.6;
  --pagefind-ui-font: Open Sans, Arial, Helvetica, sans-serif;
  box-shadow: rgb(184, 179, 179) 0 0 10px 0;
  top: 100%;
  right: 0;
  position: absolute;
  background: #fff;
  max-width: 50%;
  border-radius: 0 0 0 6px;
  isolation: isolate;
  -webkit-mask-image: -webkit-radial-gradient(white, black);
  z-index: 2;
}
@media screen and (max-width: 600px) {
  #search {
    max-width: 100%;
  }
}
.pagefind-ui__message {
  color:#666;
}
.pagefind-ui__message, .pagefind-ui__result-title {
  font-family: Trebuchet MS, Arial, Helvetica, sans-serif!important;
}
.pagefind-ui__search-input {
  line-height: 1rem!important;
  padding: 0 45px!important;
  width: 100%;
  border-top: 0 !important;
  border-right: 0 !important;
  border-radius: 0 0 4px 4px!important;
}

.pagefind-ui__drawer {
  width: 100%;
  max-height: calc(100vh - 10rem);
  overflow: auto;
}
.pagefind-ui__drawer li:hover {
  background: #dff;
  cursor: pointer;
}
.pagefind-ui__drawer * {
  font-size: 16px !important;
}
.pagefind-ui__drawer .pagefind-ui__button {
  font-size: 14px !important;
  padding: 0.2rem;
}
.pagefind-ui__drawer .pagefind-ui__button:hover {
  background: #dff;
  border-color: #aaa!important;
}
@media screen and (max-width: 650px) {
  .pagefind-ui__drawer {
    max-height: calc(100vh - 8rem);
    padding-bottom: 1rem!important;
  }
}
@media screen and (max-width: 460px) {
  .pagefind-ui__drawer {
    max-height: calc(100vh - 10rem);
  }
}
@media screen and (max-width: 392px) {
  .pagefind-ui__drawer {
    max-height: calc(100vh - 12rem);
  }
}
@media screen and (max-width: 377px) {
  .pagefind-ui__drawer {
    max-height: calc(100vh - 14rem);
  }
}
@media screen and (max-width: 362px) {
  .pagefind-ui__drawer {
    max-height: calc(100vh - 16rem);
  }
}
@media screen and (max-width: 300px) {
  html {
    max-height: calc(100vh - 13rem);
  }
}

.pagefind-ui__results-area p, .pagefind-ui__results-area ul {
  padding: 0 1rem 1rem !important;
}
