/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/loc-from-callsite.ts > TAP > from CallSite 1`] = `
Object {
  "column": 33,
  "file": "{CWD}/test/loc-from-callsite.ts",
  "line": 5,
}
`

exports[`test/loc-from-callsite.ts > TAP > from CallSiteJSON 1`] = `
Object {
  "column": 33,
  "file": "{CWD}/test/loc-from-callsite.ts",
  "line": 6,
}
`

exports[`test/loc-from-callsite.ts > TAP > from null 1`] = `
Object {
  "column": undefined,
  "file": undefined,
  "line": undefined,
}
`

exports[`test/loc-from-callsite.ts > TAP > from undefined 1`] = `
Object {
  "column": undefined,
  "file": undefined,
  "line": undefined,
}
`
