/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/test-template.ts > TAP > generate some output > test output 1`] = `
TAP version 14
# Subtest: child test
    ok 1 - this is fine
    1..1
ok 1 - child test # time={TIME}

ok 2 - skipped test, do not run function # SKIP
ok 3 - todo test, do not run function # TODO
1..3

`

exports[`test/test-template.ts > TAP > make a test, do some stuff with it > extended test output 1`] = `
TAP version 14
ok 1 - this is fine
# Subtest: (unnamed test)
    1..0
ok 2 - (unnamed test) # time={TIME}

1..2

`

exports[`test/test-template.ts > TAP > make a test, do some stuff with it > pluginSignature 1`] = `

`
