/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/has-strict.ts > TAP > iterables match one another > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,4 +1,4 @@
-Array [
+And [
   1,
   2,
 ]

`

exports[`test/has-strict.ts > TAP > iterables match one another > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,4 +1,4 @@
-And [
+Array [
   1,
   2,
 ]

`

exports[`test/has-strict.ts > TAP > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "b": "2",
+  "b": 2,
 }

`

exports[`test/has-strict.ts > TAP > url object matches normal object (ctors not relevant) > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 URL {
-  "pathname": "/f",
+  "pathname": "/foo",
 }

`

exports[`test/has-strict.ts > TAP > url object matches normal object (ctors not relevant) > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "constructor": Function URL(classURL),
+  "constructor": Function Object(),
 }

`
