/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/same.tsx > TAP > another weird cycle case > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,5 +1,3 @@
 &ref_1 Object {
-  "o": &ref_1 Object {
-    "o": <*ref_1>,
-  },
+  "o": <*ref_1>,
 }

`

exports[`test/same.tsx > TAP > array is not the same as object > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,5 +1,5 @@
-Object {
-  "0": 1,
-  "1": 2,
-  "2": 3,
-}
+Array [
+  1,
+  2,
+  3,
+]

`

exports[`test/same.tsx > TAP > array-likes > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,5 +1,5 @@
-Ayyr [
+Array [
   1,
   2,
   3,
 ]

`

exports[`test/same.tsx > TAP > array-likes > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,6 +1,5 @@
-Object {
-  "0": 1,
-  "1": 2,
-  "2": 2,
-  "length": 3,
-}
+Array [
+  1,
+  2,
+  3,
+]

`

exports[`test/same.tsx > TAP > arrays extra and missing > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,7 +1,5 @@
 Array [
   1,
   2,
   3,
-  4,
-  5,
 ]

`

exports[`test/same.tsx > TAP > arrays extra and missing > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,5 +1,7 @@
 Array [
   1,
   2,
   3,
+  4,
+  5,
 ]

`

exports[`test/same.tsx > TAP > basic > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
--Infinity
+Infinity

`

exports[`test/same.tsx > TAP > basic > must match snapshot 10`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-undefined
+null

`

exports[`test/same.tsx > TAP > basic > must match snapshot 11`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-null
+undefined

`

exports[`test/same.tsx > TAP > basic > must match snapshot 12`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-Buffer <6162 6364  abcd>
+Buffer <6162 63  abc>

`

exports[`test/same.tsx > TAP > basic > must match snapshot 13`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-Buffer <6162 64  abd>
+Buffer <6162 63  abc>

`

exports[`test/same.tsx > TAP > basic > must match snapshot 14`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-2001-01-12T00:00:00.000Z
+2001-01-11T00:00:00.000Z

`

exports[`test/same.tsx > TAP > basic > must match snapshot 15`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-/^(howdy|hello)$/
+/^h[oe][wl][dl][oy]$/

`

exports[`test/same.tsx > TAP > basic > must match snapshot 16`] = `
--- expected
+++ actual
@@ -1,5 +1,5 @@
-Array [
+Arguments [
   1,
   2,
   3,
 ]

`

exports[`test/same.tsx > TAP > basic > must match snapshot 17`] = `
--- expected
+++ actual
@@ -1,5 +1,5 @@
-Arguments [
+Array [
   1,
   2,
   3,
 ]

`

exports[`test/same.tsx > TAP > basic > must match snapshot 18`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-Object {}
+Array []

`

exports[`test/same.tsx > TAP > basic > must match snapshot 19`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-Null Object {}
+Object {}

`

exports[`test/same.tsx > TAP > basic > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-"1"
+1

`

exports[`test/same.tsx > TAP > basic > must match snapshot 20`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "b": "b",
+  "a": "a",
 }

`

exports[`test/same.tsx > TAP > basic > must match snapshot 21`] = `
--- expected
+++ actual
@@ -1,3 +1,2 @@
 Object {
-  "c": undefined,
 }

`

exports[`test/same.tsx > TAP > basic > must match snapshot 22`] = `
--- expected
+++ actual
@@ -1,6 +1,5 @@
 EventEmitter {
   "_events": Null Object {
-    "data": Function ondata(data),
   },
-  "_eventsCount": 2,
+  "_eventsCount": 1,
 }

`

exports[`test/same.tsx > TAP > basic > must match snapshot 23`] = `
--- expected
+++ actual
@@ -1,18 +1,18 @@
 &ref_1 Object {
   "granular": Object {
     "stuff": Array [
       0,
       1,
-      3,
+      2,
     ],
     "self": Object {
       "granular": Object {
         "stuff": Array [
           0,
           1,
-          2,
+          3,
         ],
-<*ref_1>      },
+<*ref_undefined>      },
     },
   },
 }

`

exports[`test/same.tsx > TAP > basic > must match snapshot 3`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-"nok"
+"ok"

`

exports[`test/same.tsx > TAP > basic > must match snapshot 4`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-"0"
+0

`

exports[`test/same.tsx > TAP > basic > must match snapshot 5`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-null
+undefined

`

exports[`test/same.tsx > TAP > basic > must match snapshot 6`] = `
--- expected
+++ actual
@@ -1,2 +1,1 @@
 Function nop()
-/* object identities differ */

`

exports[`test/same.tsx > TAP > basic > must match snapshot 7`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-Object {}
+undefined

`

exports[`test/same.tsx > TAP > basic > must match snapshot 8`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-null
+Object {}

`

exports[`test/same.tsx > TAP > basic > must match snapshot 9`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-Object {}
+null

`

exports[`test/same.tsx > TAP > collections missing all entries > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,3 +1,2 @@
 Map {
-  1 => 1,
 }

`

exports[`test/same.tsx > TAP > collections missing all entries > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,3 +1,2 @@
 Object {
-  "a": 1,
 }

`

exports[`test/same.tsx > TAP > collections missing all entries > must match snapshot 3`] = `
--- expected
+++ actual
@@ -1,3 +1,1 @@
-Set {
-  1,
-}
+Set {}

`

exports[`test/same.tsx > TAP > collections missing all entries > must match snapshot 4`] = `
--- expected
+++ actual
@@ -1,3 +1,1 @@
-Array [
-  1,
-]
+Array []

`

exports[`test/same.tsx > TAP > ctor usage > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-false
+true

`

exports[`test/same.tsx > TAP > customize diff context > must match snapshot 1`] = `
--- expected
+++ actual
@@ -6,3 +6,3 @@
           "b": Object {
-            "c": 2,
+            "c": 1,
           },

`

exports[`test/same.tsx > TAP > different arrays don't match > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,6 +1,5 @@
 Array [
   1,
   2,
   3,
-  4,
 ]

`

exports[`test/same.tsx > TAP > different arrays don't match > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,5 +1,5 @@
 Array [
   1,
   2,
-  4,
+  3,
 ]

`

exports[`test/same.tsx > TAP > different numbers don't match > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-1
+0

`

exports[`test/same.tsx > TAP > different numbers don't match > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
--1
+1

`

exports[`test/same.tsx > TAP > different numbers don't match > must match snapshot 3`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-2.72
+3.14

`

exports[`test/same.tsx > TAP > diffing strings > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,28 +1,28 @@
 String(
-  str2asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  str1asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  foofoofoofoofoo
-  foofoofoofoofoo
-  foofoofoofoofoo
-  foofoofoofoofoo
-  foofoofoofoofoo
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
   
 )

`

exports[`test/same.tsx > TAP > diffing strings > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,28 +1,1 @@
-String(
-  str1asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
-  
-)
+""

`

exports[`test/same.tsx > TAP > diffing strings > must match snapshot 3`] = `
--- expected
+++ actual
@@ -1,1 +1,28 @@
-""
+String(
+  str1asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  asdfasdfasdfasdfasdfasdfasdfasdfasdfasdf
+  
+)

`

exports[`test/same.tsx > TAP > diffs of errors with \\n in the message > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,4 +1,4 @@
 Error: foo
 bar {
-  "foo": "two",
+  "foo": "bar",
 }

`

exports[`test/same.tsx > TAP > diffs of errors with \\n in the message > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,6 +1,6 @@
 Object {
   "er": Error: foo
 bar {
-    "foo": "two",
+    "foo": "bar",
   },
 }

`

exports[`test/same.tsx > TAP > errors > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-Error: oof
+Error: foo

`

exports[`test/same.tsx > TAP > errors > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,1 +1,4 @@
-Error: foo
+Object {
+  "name": "Error",
+  "message": "foo",
+}

`

exports[`test/same.tsx > TAP > errors > must match snapshot 3`] = `
--- expected
+++ actual
@@ -1,5 +1,3 @@
-drr: i have none {
-  "name": "drr",
-  "message": "i have none",
+Error: foo {
   "foo": "bar",
 }

`

exports[`test/same.tsx > TAP > errors > must match snapshot 4`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 drr: i have none {
-  "foo": "baz",
+  "foo": "bar",
 }

`

exports[`test/same.tsx > TAP > hidden props and getters > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Hidden {
-  "baseValue": 1,
+  "baseValue": 0,
 }

`

exports[`test/same.tsx > TAP > iterables match one another > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,4 +1,4 @@
-Array [
+And [
   1,
   2,
 ]

`

exports[`test/same.tsx > TAP > iterables match one another > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,4 +1,4 @@
-And [
+Array [
   1,
   2,
 ]

`

exports[`test/same.tsx > TAP > iterator that doesnt play nice > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,3 +1,5 @@
-Object {
-  [Symbol.iterator]: GeneratorFunction [Symbol.iterator](),
-}
+Array [
+  1,
+  2,
+  3,
+]

`

exports[`test/same.tsx > TAP > map > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,5 +1,5 @@
 Map {
   5 => Object {
-    "a": "1",
+    "a": 1,
   },
 }

`

exports[`test/same.tsx > TAP > map > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,5 +1,5 @@
 Map {
   5 => Object {
-    "a": "1",
+    "a": 1,
   },
 }

`

exports[`test/same.tsx > TAP > map > must match snapshot 3`] = `
--- expected
+++ actual
@@ -1,5 +1,5 @@
 Map {
   5 => Object {
-    "a": "1",
+    "a": 1,
   },
 }

`

exports[`test/same.tsx > TAP > map > must match snapshot 4`] = `
--- expected
+++ actual
@@ -1,22 +1,10 @@
-Array [
-  Array [
-    1,
-    2,
-  ],
-  Array [
-    3,
-    4,
-  ],
-  Array [
-    5,
-    Object {
-      "a": 1,
-    },
-  ],
-  Array [
-    Object {
-      "a": 1,
-    },
-    6,
-  ],
-]
+Map {
+  1 => 2,
+  3 => 4,
+  5 => Object {
+    "a": 1,
+  },
+  Object {
+    "a": 1,
+  } => 6,
+}

`

exports[`test/same.tsx > TAP > map > must match snapshot 5`] = `
--- expected
+++ actual
@@ -1,6 +1,5 @@
 Map {
   Object {
     "a": 1,
-    "b": 2,
   } => 6,
 }

`

exports[`test/same.tsx > TAP > map > must match snapshot 6`] = `
--- expected
+++ actual
@@ -1,9 +1,8 @@
 Map {
   5 => Object {
-    "a": 1,
+    "a": "1",
   },
   Object {
     "a": 1,
-    "b": 2,
   } => 6,
 }

`

exports[`test/same.tsx > TAP > map > must match snapshot 7`] = `
--- expected
+++ actual
@@ -1,22 +1,10 @@
-Array [
-  Array [
-    1,
-    2,
-  ],
-  Array [
-    3,
-    4,
-  ],
-  Array [
-    5,
-    Object {
-      "a": 1,
-    },
-  ],
-  Array [
-    Object {
-      "a": 1,
-    },
-    6,
-  ],
-]
+Map {
+  1 => 2,
+  3 => 4,
+  5 => Object {
+    "a": 1,
+  },
+  Object {
+    "a": 1,
+  } => 6,
+}

`

exports[`test/same.tsx > TAP > map > must match snapshot 8`] = `
--- expected
+++ actual
@@ -1,6 +1,5 @@
 Map {
   Object {
     "a": 1,
-    "b": 2,
   } => 6,
 }

`

exports[`test/same.tsx > TAP > map > must match snapshot 9`] = `
--- expected
+++ actual
@@ -1,6 +1,5 @@
 Map {
   Object {
     "a": 1,
-    "b": 2,
   } => 6,
 }

`

exports[`test/same.tsx > TAP > maps extra and missing > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,3 +1,2 @@
 Map {
-  "b" => 2,
 }

`

exports[`test/same.tsx > TAP > maps extra and missing > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,2 +1,3 @@
 Map {
+  "b" => 2,
 }

`

exports[`test/same.tsx > TAP > obj cycles, expect does not > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,8 +1,9 @@
 &ref_1 Object {
   "o": Object {
     "o": Object {
       "o": Object {
+        "a": <*ref_1>,
       },
     },
   },
 }

`

exports[`test/same.tsx > TAP > pojos extra and missing > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,3 +1,2 @@
 Object {
-  "b": 2,
 }

`

exports[`test/same.tsx > TAP > pojos extra and missing > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,2 +1,3 @@
 Object {
+  "b": 2,
 }

`

exports[`test/same.tsx > TAP > properly handle circular data structures > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,3 +1,9 @@
 &ref_1 Object {
-<*ref_1>  "other": <*ref_1>,
+<*ref_undefined>  "other": &ref_2 Object {
+    "z": 4,
+    "y": Object {
+      "x": <*ref_2>,
+    },
+    "other": <*ref_2>,
+  },
 }

`

exports[`test/same.tsx > TAP > properly handle circular data structures > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,5 +1,5 @@
 &ref_1 Object {
   "x": &ref_2 Object {
-    "a": <*ref_2>,
+    "a": <*ref_1>,
   },
 }

`

exports[`test/same.tsx > TAP > properly handle circular data structures > must match snapshot 3`] = `
--- expected
+++ actual
@@ -1,6 +1,5 @@
 Object {
   "x": &ref_1 Object {
-    "TWO": 2,
-    "a": <*ref_1>,
+    "happy": true,
   },
 }

`

exports[`test/same.tsx > TAP > react > array nesting relevant when not using react strings > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,23 +1,15 @@
 {
   "props": {
     "children": {
-      "0": [
-        [
-          [
-            {
-              "$$typeof": Symbol.for("react.element"),
-              "type": "img",
-              "key": null,
-              "ref": null,
-              "props": {
-                "src": "foo",
-              },
-              "_owner": null,
-              "_store": {},
-            },
-          ],
-        ],
-      ],
+      "$$typeof": Symbol.for("react.element"),
+      "type": "img",
+      "key": null,
+      "ref": null,
+      "props": {
+        "src": "foo",
+      },
+      "_owner": null,
+      "_store": {},
     },
   },
 }

`

exports[`test/same.tsx > TAP > react > array nesting relevant when not using react strings > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,23 +1,15 @@
 Object {
   "props": Object {
     "children": Object {
-      "0": Array [
-        Array [
-          Array [
-            Object {
-              "$$typeof": Symbol.for(react.element),
-              "type": "img",
-              "key": null,
-              "ref": null,
-              "props": Object {
-                "src": "foo",
-              },
-              "_owner": null,
-              "_store": Object {},
-            },
-          ],
-        ],
-      ],
+      "$$typeof": Symbol.for(react.element),
+      "type": "img",
+      "key": null,
+      "ref": null,
+      "props": Object {
+        "src": "foo",
+      },
+      "_owner": null,
+      "_store": Object {},
     },
   },
 }

`

exports[`test/same.tsx > TAP > react > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
-<div>
+<div className="divvy">
   <img src="foo" />
 </div>

`

exports[`test/same.tsx > TAP > react > must match snapshot 2`] = `

`

exports[`test/same.tsx > TAP > set > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,4 +1,3 @@
 Set {
   1,
-  2,
 }

`

exports[`test/same.tsx > TAP > set > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,5 +1,5 @@
 Set {
   1,
-  6,
-  2,
+  3,
+  5,
 }

`

exports[`test/same.tsx > TAP > set > must match snapshot 3`] = `
--- expected
+++ actual
@@ -1,9 +1,9 @@
-Array [
+Set {
   1,
   2,
   3,
   4,
   Object {
     "a": 1,
   },
-]
+}

`

exports[`test/same.tsx > TAP > set > must match snapshot 4`] = `
--- expected
+++ actual
@@ -1,4 +1,3 @@
 Set {
   1,
-  2,
 }

`

exports[`test/same.tsx > TAP > set > must match snapshot 5`] = `
--- expected
+++ actual
@@ -1,5 +1,5 @@
 Set {
   1,
-  6,
-  2,
+  3,
+  5,
 }

`

exports[`test/same.tsx > TAP > set > must match snapshot 6`] = `
--- expected
+++ actual
@@ -1,9 +1,9 @@
-Array [
+Set {
   1,
   2,
   3,
   4,
   Object {
     "a": 1,
   },
-]
+}

`

exports[`test/same.tsx > TAP > sets extra and missing > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,10 +1,6 @@
 Set {
   Array [
-    "b",
-    2,
-  ],
-  Array [
     "a",
     1,
   ],
 }

`

exports[`test/same.tsx > TAP > sets extra and missing > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,6 +1,10 @@
 Set {
   Array [
+    "b",
+    2,
+  ],
+  Array [
     "a",
     1,
   ],
 }

`

exports[`test/same.tsx > TAP > should handle bigint > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-2n
+1n

`

exports[`test/same.tsx > TAP > should handle dates > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-null
+1972-08-01T00:00:00.000Z

`

exports[`test/same.tsx > TAP > should handle dates > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-undefined
+1972-08-01T00:00:00.000Z

`

exports[`test/same.tsx > TAP > should handle functions > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,3 +1,4 @@
 function a() {
-        return a;
+        //@ts-expect-error
+        return b;
     }

`

exports[`test/same.tsx > TAP > should handle functions > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-Function fnB(a)
+Function fnA(a)

`

exports[`test/same.tsx > TAP > should handle RegExps > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-/[a]/
+/[b]/

`

exports[`test/same.tsx > TAP > should handle RegExps > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-/[a]/g
+/[a]/i

`

exports[`test/same.tsx > TAP > should notice different Buffers > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,1 +1,1 @@
-Buffer <0001 17  ...>
+Buffer <0001 02  ...>

`

exports[`test/same.tsx > TAP > should notice different Buffers > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,18 +1,7 @@
 Object {
   "x": Object {
     "y": Object {
-      "z": Buffer <
-        0000: 0000 0100 0200 0300 0400 0500 0600 0700 0800 0900 0a00 0b00 0c00 0d00 0e00 0f00  ................................
-        0020: 1000 1100 1200 1300 1400 1500 1600 1700 1800 1900 1a00 1b00 1c00 1d00 1e00 1f00  ................................
-        0040: 2000 2100 2200 2300 2400 2500 2600 2700 2800 2900 2a00 2b00 2c00 2d00 2e00 2f00  ..!.".#.$.%.&.'.(.).*.+.,.-.../.
-        0060: 3000 3100 3200 3300 3400 3500 3600 3700 3800 3900 3a00 3b00 3c00 3d00 3e00 3f00  0.1.2.3.4.5.6.7.8.9.:.;.<.=.>.?.
-        0080: 4000 4100 4200 4300 4400 4500 4600 4700 4800 4900 4a00 4b00 4c00 4d00 4e00 4f00  @.A.B.C.D.E.F.G.H.I.J.K.L.M.N.O.
-        00a0: 5000 5100 5200 5300 5400 5500 5600 5700 5800 5900 5a00 5b00 5c00 5d00 5e00 5f00  P.Q.R.S.T.U.V.W.X.Y.Z.[.\\.].^._.
-        00c0: 6000 6100 6200 6300 6400 6500 6600 6700 6800 6900 6a00 6b00 6c00 6d00 6e00 6f00  \`.a.b.c.d.e.f.g.h.i.j.k.l.m.n.o.
-        00e0: 7000 7100 7200 7300 7400 7500 7600 7700 7800 7900 7a00 7b00 7c00 7d00 7e00 7f00  p.q.r.s.t.u.v.w.x.y.z.{.|.}.~...
-        0100: 8000 8100 8200 8300 8400 8500 8600 8700 8800 8900 8a00 8b00 8c00 8d00 8e00 8f00  ................................
-        0120: 9000 9100 9200 9300 9400 9500 9600 9700 9800 9900 9a00 9b00 9c00 9d00 9e00 9f00  ................................
-      >,
+      "z": Buffer <0001  ..>,
     },
   },
 }

`

exports[`test/same.tsx > TAP > should notice objects with different keys > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "c": 2,
+  "b": 2,
 }

`

exports[`test/same.tsx > TAP > should notice objects with different shapes > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,3 +1,2 @@
 Object {
-  "b": undefined,
 }

`

exports[`test/same.tsx > TAP > simple diff nested in object > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,9 +1,9 @@
 Object {
   "a": Object {
     "b": Array [
       Object {
-        "x": false,
+        "x": true,
       },
     ],
   },
 }

`

exports[`test/same.tsx > TAP > symbology > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,4 +1,3 @@
 Object {
   "a": Symbol(a),
 }
-/* object identities differ */

`

exports[`test/same.tsx > TAP > symbology > must match snapshot 10`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "a": Symbol(a),
+  "a": "Symbol(a)",
 }

`

exports[`test/same.tsx > TAP > symbology > must match snapshot 11`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "a": Symbol.for(a),
+  "a": "Symbol(a)",
 }

`

exports[`test/same.tsx > TAP > symbology > must match snapshot 12`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "a": Function Symbol(),
+  "a": "Symbol(a)",
 }

`

exports[`test/same.tsx > TAP > symbology > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "a": Symbol.for(a),
+  "a": Symbol(a),
 }

`

exports[`test/same.tsx > TAP > symbology > must match snapshot 3`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "a": Function Symbol(),
+  "a": Symbol(a),
 }

`

exports[`test/same.tsx > TAP > symbology > must match snapshot 4`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "a": "Symbol(a)",
+  "a": Symbol(a),
 }

`

exports[`test/same.tsx > TAP > symbology > must match snapshot 5`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "a": Symbol(a),
+  "a": "Symbol(a)",
 }

`

exports[`test/same.tsx > TAP > symbology > must match snapshot 6`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "a": Symbol.for(a),
+  "a": "Symbol(a)",
 }

`

exports[`test/same.tsx > TAP > symbology > must match snapshot 7`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "a": Function Symbol(),
+  "a": "Symbol(a)",
 }

`

exports[`test/same.tsx > TAP > symbology > must match snapshot 8`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "a": Function Symbol(),
+  "a": Symbol(a),
 }

`

exports[`test/same.tsx > TAP > symbology > must match snapshot 9`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "a": "Symbol(a)",
+  "a": Symbol(a),
 }

`

exports[`test/same.tsx > TAP > tricky mismatched nesting > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,5 +1,3 @@
 &ref_1 Object {
-  "o": &ref_2 Object {
-    "o": <*ref_2>,
-  },
+  "o": <*ref_1>,
 }

`

exports[`test/same.tsx > TAP > tricky mismatched nesting > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,11 +1,3 @@
 &ref_1 Object {
-  "o": Object {
-    "o": &ref_2 Object {
-      "o": Object {
-        "o": Object {
-          "o": <*ref_2>,
-        },
-      },
-    },
-  },
+  "o": <*ref_1>,
 }

`

exports[`test/same.tsx > TAP > tricky mismatched nesting > must match snapshot 3`] = `
--- expected
+++ actual
@@ -1,11 +1,5 @@
 Object {
   "o": &ref_1 Object {
-    "o": &ref_2 Object {
-      "o": Object {
-        "o": Object {
-          "o": <*ref_2>,
-        },
-      },
-    },
+    "o": <*ref_1>,
   },
 }

`

exports[`test/same.tsx > TAP > undefined, null, missing > must match snapshot 1`] = `
--- expected
+++ actual
@@ -1,2 +1,3 @@
 Object {
+  "a": undefined,
 }

`

exports[`test/same.tsx > TAP > undefined, null, missing > must match snapshot 10`] = `
--- expected
+++ actual
@@ -1,2 +1,3 @@
 Object {
+  "a": null,
 }

`

exports[`test/same.tsx > TAP > undefined, null, missing > must match snapshot 11`] = `
--- expected
+++ actual
@@ -1,3 +1,2 @@
 Object {
-  "a": null,
 }

`

exports[`test/same.tsx > TAP > undefined, null, missing > must match snapshot 12`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "b": null,
+  "a": null,
 }

`

exports[`test/same.tsx > TAP > undefined, null, missing > must match snapshot 2`] = `
--- expected
+++ actual
@@ -1,3 +1,2 @@
 Object {
-  "a": undefined,
 }

`

exports[`test/same.tsx > TAP > undefined, null, missing > must match snapshot 3`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "b": undefined,
+  "a": undefined,
 }

`

exports[`test/same.tsx > TAP > undefined, null, missing > must match snapshot 4`] = `
--- expected
+++ actual
@@ -1,2 +1,3 @@
 Object {
+  "a": null,
 }

`

exports[`test/same.tsx > TAP > undefined, null, missing > must match snapshot 5`] = `
--- expected
+++ actual
@@ -1,3 +1,2 @@
 Object {
-  "a": null,
 }

`

exports[`test/same.tsx > TAP > undefined, null, missing > must match snapshot 6`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "b": null,
+  "a": null,
 }

`

exports[`test/same.tsx > TAP > undefined, null, missing > must match snapshot 7`] = `
--- expected
+++ actual
@@ -1,3 +1,3 @@
 Object {
-  "a": undefined,
+  "a": null,
 }

`

exports[`test/same.tsx > TAP > undefined, null, missing > must match snapshot 8`] = `
--- expected
+++ actual
@@ -1,3 +1,2 @@
 Object {
-  "a": undefined,
 }

`

exports[`test/same.tsx > TAP > undefined, null, missing > must match snapshot 9`] = `
--- expected
+++ actual
@@ -1,3 +1,2 @@
 Object {
-  "b": undefined,
 }

`
