/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/hooks.ts > TAP > must match snapshot 1`] = `
Array [
  "root before",
  "root beforeEach",
  "suite before",
  "root beforeEach",
  "suite beforeEach",
  "has_a_name",
  "suite afterEach",
  "root afterEach",
  "root beforeEach",
  "suite beforeEach",
  "is_async",
  "suite afterEach",
  "root afterEach",
  "root beforeEach",
  "suite beforeEach",
  "comes after async",
  "suite afterEach",
  "root afterEach",
  "root beforeEach",
  "suite beforeEach",
  "comes last",
  "suite afterEach",
  "root afterEach",
  "suite after",
  "root afterEach",
  "root beforeEach",
]
`
