/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/test-badge.tsx > TAP > bailed out for no raisin > must match snapshot 1`] = `
[1m[41m[38;2;255;255;255m FAIL [39m[49m[22m
`

exports[`test/test-badge.tsx > TAP > bailed out with reason > must match snapshot 1`] = `
[1m[41m[38;2;255;255;255m FAIL [39m[49m[22m
`

exports[`test/test-badge.tsx > TAP > fail counts.fail > must match snapshot 1`] = `
[1m[41m[38;2;255;255;255m FAIL [39m[49m[22m
`

exports[`test/test-badge.tsx > TAP > fail ok=false > must match snapshot 1`] = `
[1m[41m[38;2;255;255;255m FAIL [39m[49m[22m
`

exports[`test/test-badge.tsx > TAP > ok, but exited with signal > must match snapshot 1`] = `
[1m[41m[38;2;255;255;255m FAIL [39m[49m[22m
`

exports[`test/test-badge.tsx > TAP > pass > must match snapshot 1`] = `
[1m[48;2;0;170;0m[38;2;0;0;0m PASS [39m[49m[22m
`

exports[`test/test-badge.tsx > TAP > skip counts.skip > must match snapshot 1`] = `
[1m[44m[38;2;255;255;255m SKIP [39m[49m[22m
`

exports[`test/test-badge.tsx > TAP > skip plan.skipAll > must match snapshot 1`] = `
[1m[44m[38;2;255;255;255m SKIP [39m[49m[22m
`

exports[`test/test-badge.tsx > TAP > skip results.skip > must match snapshot 1`] = `
[1m[44m[38;2;255;255;255m SKIP [39m[49m[22m
`

exports[`test/test-badge.tsx > TAP > test in progress > must match snapshot 1`] = `
[1m[43m[38;2;0;0;0m RUNS [39m[49m[22m
`

exports[`test/test-badge.tsx > TAP > todo > must match snapshot 1`] = `
[1m[48;2;136;0;136m[38;2;255;255;255m TODO [39m[49m[22m
`
