import { Extra, MessageExtra } from './index.js';
/**
 * Normalize the {@link @tapjs/core!index.MessageExtra} arguments to test
 * assertion methods
 */
export declare const normalizeMessageExtra: (defaultMessage: string, [message, extra]: MessageExtra) => [string, Extra];
//# sourceMappingURL=normalize-message-extra.d.ts.map