/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/blockless-todo.ts > TAP > must match snapshot 1`] = `
TAP version 14
# Subtest: without a function
    ok 1 - is a todo test # TODO
    1..1
ok 1 - without a function # time={TIME}

1..1

`
