/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/hanging-indent.tsx > TAP > 3 space hang 1`] = `
WHEN I WROTE the following pages, or rather the bulk of them, I lived alone, in the woods, a mile
   from any neighbor, in a house which I had built myself, on the shore of Walden Pond, in Concord,
   Massachusetts, and earned my living by the labor of my hands only. I lived there two years and two
   months. At present I am a sojourner in civilized life again.
`

exports[`test/hanging-indent.tsx > TAP > default settings 1`] = `
WHEN I WROTE the following pages, or rather the bulk of them, I lived alone, in the woods, a mile
    from any neighbor, in a house which I had built myself, on the shore of Walden Pond, in Concord,
    Massachusetts, and earned my living by the labor of my hands only. I lived there two years and two
    months. At present I am a sojourner in civilized life again.
`
