/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/index.ts > TAP > format 1`] = `
Object {
  "a": 1,
}
`

exports[`test/index.ts > TAP > Format 1`] = `
Object {
  "a": 1,
}
`

exports[`test/index.ts > TAP > Format 2`] = `
{
  "a": 1,
}
`

exports[`test/index.ts > TAP > format opt 1`] = `
{
  "a": 1,
}
`

exports[`test/index.ts > TAP > has 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > Has 1`] = `

`

exports[`test/index.ts > TAP > Has 2`] = `

`

exports[`test/index.ts > TAP > has opt 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > hasStrict 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > HasStrict 1`] = `

`

exports[`test/index.ts > TAP > HasStrict 2`] = `

`

exports[`test/index.ts > TAP > hasStrict opt 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > match 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > Match 1`] = `

`

exports[`test/index.ts > TAP > Match 2`] = `

`

exports[`test/index.ts > TAP > match opt 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > matchOnly 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > MatchOnly 1`] = `

`

exports[`test/index.ts > TAP > MatchOnly 2`] = `

`

exports[`test/index.ts > TAP > matchOnly opt 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > matchOnlyStrict 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > MatchOnlyStrict 1`] = `

`

exports[`test/index.ts > TAP > MatchOnlyStrict 2`] = `

`

exports[`test/index.ts > TAP > matchOnlyStrict opt 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > matchStrict 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > MatchStrict 1`] = `

`

exports[`test/index.ts > TAP > MatchStrict 2`] = `

`

exports[`test/index.ts > TAP > matchStrict opt 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > same 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > Same 1`] = `

`

exports[`test/index.ts > TAP > Same 2`] = `

`

exports[`test/index.ts > TAP > same opt 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > strict 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > Strict 1`] = `

`

exports[`test/index.ts > TAP > Strict 2`] = `

`

exports[`test/index.ts > TAP > strict opt 1`] = `
Object {
  "diff": "",
  "match": true,
}
`

exports[`test/index.ts > TAP > styles object 1`] = `
Object {
  "js": Object {
    "arrayEmpty": Function arrayEmpty(_),
    "arrayEntrySep": Function arrayEntrySep(),
    "arrayHead": Function arrayHead(_),
    "arrayTail": Function arrayTail(indent),
    "bufferBody": Function bufferBody(buf),
    "bufferChunkSize": 32,
    "bufferEmpty": Function bufferEmpty(),
    "bufferEnd": Function bufferEnd(buf),
    "bufferHead": Function bufferHead(),
    "bufferKey": Function bufferKey(),
    "bufferKeySep": Function bufferKeySep(),
    "bufferLine": Function bufferLine(buf, chunkSize),
    "bufferLineSep": Function bufferLineSep(),
    "bufferStart": Function bufferStart(),
    "bufferTail": Function bufferTail(indent),
    "circular": Function circular(node),
    "diffable": true,
    "errorEmpty": Function errorEmpty(er, cls),
    "errorHead": Function errorHead(er, cls),
    "errorTail": Function errorTail(indent),
    "fn": Function fn(fn, _),
    "mapEmpty": Function mapEmpty(cls),
    "mapEntrySep": Function mapEntrySep(),
    "mapHead": Function mapHead(cls),
    "mapKeyStart": Function mapKeyStart(),
    "mapKeyValSep": Function mapKeyValSep(),
    "mapTail": Function mapTail(indent),
    "nodeId": Function nodeId(id),
    "pojoEmpty": Function pojoEmpty(_),
    "pojoEntrySep": Function pojoEntrySep(),
    "pojoHead": Function pojoHead(_),
    "pojoKeyValSep": Function pojoKeyValSep(),
    "pojoTail": Function pojoTail(indent),
    "setEmpty": Function setEmpty(cls),
    "setEntrySep": Function setEntrySep(),
    "setHead": Function setHead(cls),
    "setTail": Function setTail(indent),
    "start": Function start(indent, key, sep),
    "stringEmpty": Function stringEmpty(),
    "stringHead": Function stringHead(),
    "stringLine": Function stringLine(str),
    "stringLineSep": Function stringLineSep(),
    "stringOneLine": Function stringOneLine(str),
    "stringTail": Function stringTail(indent),
    "symbol": Function symbol(method, key),
  },
  "pretty": Object {
    "arrayEmpty": Function arrayEmpty(cls),
    "arrayEntrySep": Function arrayEntrySep(),
    "arrayHead": Function arrayHead(cls),
    "arrayTail": Function arrayTail(indent),
    "bufferBody": Function bufferBody(buf),
    "bufferChunkSize": 32,
    "bufferEmpty": Function bufferEmpty(),
    "bufferEnd": Function bufferEnd(buf),
    "bufferHead": Function bufferHead(),
    "bufferKey": Function bufferKey(i),
    "bufferKeySep": Function bufferKeySep(),
    "bufferLine": Function bufferLine(buf, chunkSize),
    "bufferLineSep": Function bufferLineSep(),
    "bufferStart": Function bufferStart(),
    "bufferTail": Function bufferTail(indent),
    "circular": Function circular(node),
    "diffable": true,
    "errorEmpty": Function errorEmpty(er),
    "errorHead": Function errorHead(er, cls),
    "errorTail": Function errorTail(indent),
    "fn": Function fn(fn, cls),
    "mapEmpty": Function mapEmpty(cls),
    "mapEntrySep": Function mapEntrySep(),
    "mapHead": Function mapHead(cls),
    "mapKeyStart": Function mapKeyStart(),
    "mapKeyValSep": Function mapKeyValSep(),
    "mapTail": Function mapTail(indent),
    "nodeId": Function nodeId(id),
    "pojoEmpty": Function pojoEmpty(cls),
    "pojoEntrySep": Function pojoEntrySep(),
    "pojoHead": Function pojoHead(cls),
    "pojoKeyValSep": Function pojoKeyValSep(),
    "pojoTail": Function pojoTail(indent),
    "reactElement": Function reactElement(el),
    "setEmpty": Function setEmpty(cls),
    "setEntrySep": Function setEntrySep(),
    "setHead": Function setHead(cls),
    "setTail": Function setTail(indent),
    "start": Function start(indent, key, sep),
    "stringEmpty": Function stringEmpty(),
    "stringHead": Function stringHead(),
    "stringLine": Function stringLine(str),
    "stringLineSep": Function stringLineSep(),
    "stringOneLine": Function stringOneLine(str),
    "stringTail": Function stringTail(indent),
    "symbol": Function symbol(method, key),
  },
  "tight": Object {
    "arrayEmpty": Function arrayEmpty(_),
    "arrayEntrySep": Function arrayEntrySep(),
    "arrayHead": Function arrayHead(_),
    "arrayTail": Function arrayTail(),
    "bufferBody": Function bufferBody(buf),
    "bufferChunkSize": Infinity,
    "bufferEmpty": Function bufferEmpty(),
    "bufferEnd": Function bufferEnd(),
    "bufferHead": Function bufferHead(),
    "bufferKey": Function bufferKey(),
    "bufferKeySep": Function bufferKeySep(),
    "bufferLine": Function bufferLine(buf, chunkSize),
    "bufferLineSep": Function bufferLineSep(),
    "bufferStart": Function bufferStart(),
    "bufferTail": Function bufferTail(indent),
    "circular": Function circular(node),
    "diffable": false,
    "errorEmpty": Function errorEmpty(er, cls),
    "errorHead": Function errorHead(er, cls),
    "errorTail": Function errorTail(_),
    "fn": Function fn(fn, _),
    "mapEmpty": Function mapEmpty(cls),
    "mapEntrySep": Function mapEntrySep(),
    "mapHead": Function mapHead(cls),
    "mapKeyStart": Function mapKeyStart(),
    "mapKeyValSep": Function mapKeyValSep(),
    "mapTail": Function mapTail(_),
    "nodeId": Function nodeId(id),
    "pojoEmpty": Function pojoEmpty(_),
    "pojoEntrySep": Function pojoEntrySep(),
    "pojoHead": Function pojoHead(_),
    "pojoKeyValSep": Function pojoKeyValSep(),
    "pojoTail": Function pojoTail(_),
    "setEmpty": Function setEmpty(cls),
    "setEntrySep": Function setEntrySep(),
    "setHead": Function setHead(cls),
    "setTail": Function setTail(_),
    "start": Function start(_indent, key, sep),
    "stringEmpty": Function stringEmpty(),
    "stringHead": Function stringHead(),
    "stringLine": Function stringLine(str),
    "stringLineSep": Function stringLineSep(),
    "stringOneLine": Function stringOneLine(str),
    "stringTail": Function stringTail(_),
    "symbol": Function symbol(method, key),
  },
}
`
