import type { BaseOpts } from './base.js';
import { Extra } from './index.js';
/**
 * Create an {@link @tapjs/core!index.Extra} object based on a thrown Error
 */
export declare const extraFromError: (er: any, extra?: Extra, options?: BaseOpts) => Extra;
//# sourceMappingURL=extra-from-error.d.ts.map