/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/reporter.tsx > TAP > reporter output 1`] = `
Array [
  String(
    just a dummy reporter
    test/reporter.tsx > dummy test
    /some/path
    {
      "asserts": {
        "total": 0,
        "pass": 0
      },
      "suites": {
        "total": 0,
        "pass": 0
      }
    }
  ),
  String(
    just a dummy reporter
    test/reporter.tsx > dummy test
    /some/path
    {
      "asserts": {
        "total": 1,
        "pass": 1
      },
      "suites": {
        "total": 1,
        "pass": 0
      }
    }
  ),
  String(
    just a dummy reporter
    test/reporter.tsx > dummy test
    /some/path
    {
      "asserts": {
        "total": 2,
        "pass": 1,
        "skip": 1
      },
      "suites": {
        "total": 1,
        "pass": 0
      }
    }
  ),
  String(
    just a dummy reporter
    test/reporter.tsx > dummy test
    /some/path
    {
      "asserts": {
        "total": 3,
        "pass": 1,
        "skip": 2
      },
      "suites": {
        "total": 1,
        "pass": 0
      }
    }
  ),
  String(
    just a dummy reporter
    test/reporter.tsx > dummy test
    /some/path
    {
      "asserts": {
        "total": 4,
        "pass": 1,
        "todo": 1,
        "skip": 2
      },
      "suites": {
        "total": 1,
        "pass": 0
      }
    }
  ),
  String(
    just a dummy reporter
    test/reporter.tsx > dummy test
    /some/path
    {
      "asserts": {
        "total": 4,
        "pass": 1,
        "todo": 1,
        "skip": 2
      },
      "suites": {
        "total": 2,
        "pass": 0
      }
    }
  ),
  String(
    just a dummy reporter
    test/reporter.tsx > dummy test
    /some/path
    {
      "asserts": {
        "total": 5,
        "pass": 1,
        "todo": 2,
        "skip": 2
      },
      "suites": {
        "total": 2,
        "pass": 0
      }
    }
  ),
  String(
    just a dummy reporter
    test/reporter.tsx > dummy test
    /some/path
    {
      "asserts": {
        "total": 6,
        "pass": 2,
        "todo": 2,
        "skip": 2
      },
      "suites": {
        "total": 2,
        "pass": 0
      }
    }
  ),
  String(
    just a dummy reporter
    test/reporter.tsx > dummy test
    /some/path
    {
      "asserts": {
        "total": 7,
        "pass": 2,
        "todo": 2,
        "skip": 3
      },
      "suites": {
        "total": 2,
        "pass": 0
      }
    }
  ),
  String(
    just a dummy reporter
    test/reporter.tsx > dummy test
    /some/path
    {
      "asserts": {
        "total": 8,
        "pass": 2,
        "todo": 2,
        "skip": 4
      },
      "suites": {
        "total": 2,
        "pass": 0
      }
    }
  ),
  String(
    just a dummy reporter
    test/reporter.tsx > dummy test
    /some/path
    {
      "asserts": {
        "total": 9,
        "pass": 2,
        "todo": 3,
        "skip": 4
      },
      "suites": {
        "total": 2,
        "pass": 0
      }
    }
  ),
  String(
    just a dummy reporter
    test/reporter.tsx > dummy test
    /some/path
    {
      "asserts": {
        "total": 9,
        "pass": 2,
        "todo": 3,
        "skip": 4
      },
      "suites": {
        "total": 2,
        "pass": 1,
        "complete": 1
      }
    }
  ),
  String(
    just a dummy reporter
    test/reporter.tsx > dummy test
    /some/path
    {
      "asserts": {
        "total": 10,
        "pass": 2,
        "todo": 4,
        "skip": 4
      },
      "suites": {
        "total": 2,
        "pass": 1,
        "complete": 1
      }
    }
  ),
  String(
    just a dummy reporter
    test/reporter.tsx > dummy test
    /some/path
    {
      "asserts": {
        "total": 10,
        "pass": 2,
        "todo": 4,
        "skip": 4
      },
      "suites": {
        "total": 2,
        "pass": 2,
        "complete": 2
      }
    }
  ),
]
`

exports[`test/reporter.tsx > TAP > test output 1`] = `
TAP version 14
# Subtest: child test
    ok 1 - ok
    ok 2 - skip # SKIP
    ok 3 - skip message # SKIP message
    ok 4 - todo # TODO
    ok 5 - todo message # TODO message
    1..5
ok 1 - child test # time={TIME}

# Subtest: next subtest
    ok 1 - ok
    ok 2 - skip # SKIP
    ok 3 - skip message # SKIP message
    ok 4 - todo # TODO
    ok 5 - todo message # TODO message
    1..5
ok 2 - next subtest # time={TIME}

1..2

`
