/**
 * List of {@link tap-parser!result.Result} objects corresponding to test
 * points encountered in a TAP stream.
 */
export class Lists {
    fail = [];
    todo = [];
    skip = [];
    pass = [];
}
//# sourceMappingURL=lists.js.map