/**
 * call wth the pluginDir and the path to @tapjs/core
 * and it'll make sure that the plugin dir has a package.json
 * that depends on the symlink we're about to create.
 */
export declare const setPluginPackageJson: (pluginDir: string, core: string) => void;
//# sourceMappingURL=set-plugin-package-json.d.ts.map