/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/test-results-list.tsx > TAP > bailout for no raisin > must match snapshot 1`] = `
[1m[31m✖[39m[22m nope [2mtest/test-results-list.tsx:114:6[22m
 [1m[31mBailout![39m[22m
`

exports[`test/test-results-list.tsx > TAP > bailout with raisin > must match snapshot 1`] = `
[1m[31m✖[39m[22m nope [2mtest/test-results-list.tsx:125:6[22m
 [1m[31mBailout![39m[22m i have my raisins
`

exports[`test/test-results-list.tsx > TAP > exit signal > must match snapshot 1`] = `
[1m[36m~ [39m[22mskippy[36m choosey moms choose gif[39m
[1m[35m☐ [39m[22mdo later[35m i promise i will[39m
 [1m[31m✖[39m[22m  SIGNOOOOOOPE
`

exports[`test/test-results-list.tsx > TAP > show passing test result > show passing test 1`] = `
[1m[32m✓ [39m[22mthis is fine
`

exports[`test/test-results-list.tsx > TAP > skipAll custom message > must match snapshot 1`] = `
 [1m[36m~[39m[22m skip it all
`

exports[`test/test-results-list.tsx > TAP > skipAll default no tests found message > must match snapshot 1`] = `
 [1m[36m~[39m[22m no tests found
`

exports[`test/test-results-list.tsx > TAP > skipAll no message > must match snapshot 1`] = `
 [1m[36m~[39m[22m no tests found
`

exports[`test/test-results-list.tsx > TAP > test with other types of points > must match snapshot 1`] = `
[1m[36m~ [39m[22mpass, skip
[1m[36m~ [39m[22mfail, skip message[36m dont worry about it[39m
[1m[35m☐ [39m[22mfail, but todo
[1m[35m☐ [39m[22mpass, todo message[35m do it some day[39m

[1m[31m✖[39m[22m not so fine
  SOURCE
  STACK
`
