"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultPlugins = void 0;
exports.defaultPlugins = [
    '@tapjs/after',
    '@tapjs/after-each',
    '@tapjs/asserts',
    '@tapjs/before',
    '@tapjs/before-each',
    '@tapjs/chdir',
    '@tapjs/filter',
    '@tapjs/fixture',
    '@tapjs/intercept',
    '@tapjs/mock',
    '@tapjs/node-serialize',
    '@tapjs/snapshot',
    '@tapjs/spawn',
    '@tapjs/stdin',
    '@tapjs/typescript',
    '@tapjs/worker',
];
//# sourceMappingURL=default-plugins.js.map