/* make room for top menu */
html {
  scroll-padding-top: 3rem;
}

/* targeted anchors in md files */
a.tsd-anchor {
  height: 0;
  position: static;
}

/* targeted anchors in the property/method lists */
section.tsd-panel.tsd-member a.tsd-anchor:target {
  position: absolute;
  display: block;
  z-index: -1;
  top: -10px;
  left: -10px;
  border: 10px solid #fff;
  background-color: #fff;
  width: 100%;
  height: 100%;
}
@media (prefers-color-scheme: dark) {
  section.tsd-panel.tsd-member a.tsd-anchor:target {
    background-color: #000;
    border-color: #000;
  }
}

/* do not make code tiny in headings */
h1 code,
h2 code,
h3 code,
h4 code,
h5 code,
h6 code {
  font-size: 90%;
}
