/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/synonyms.ts > TAP > must match snapshot 1`] = `
Object {
  "assert": "ok",
  "assert_not": "notOk",
  "assertnot": "notOk",
  "assertNot": "notOk",
  "contains": "has",
  "deep_equal": "same",
  "deep_equals": "same",
  "deep_inequal": "notSame",
  "deep_is": "strictSame",
  "deep_not": "strictNotSame",
  "deepequal": "same",
  "deepEqual": "same",
  "deepequals": "same",
  "deepEquals": "same",
  "deepinequal": "notSame",
  "deepInequal": "notSame",
  "deepis": "strictSame",
  "deepIs": "strictSame",
  "deepnot": "strictNotSame",
  "deepNot": "strictNotSame",
  "dissimilar": "notMatch",
  "does_not_equal": "not",
  "does_not_have": "notMatch",
  "does_not_throw": "doesNotThrow",
  "doesnotequal": "not",
  "doesNotEqual": "not",
  "doesnothave": "notMatch",
  "doesNotHave": "notMatch",
  "doesnotthrow": "doesNotThrow",
  "done": "end",
  "equals": "equal",
  "equivalent": "same",
  "false": "notOk",
  "has_fields": "has",
  "hasfields": "has",
  "hasFields": "has",
  "identical": "equal",
  "if_err": "error",
  "if_error": "error",
  "iferr": "error",
  "ifErr": "error",
  "iferror": "error",
  "ifError": "error",
  "include": "has",
  "includes": "has",
  "inequal": "not",
  "inequivalent": "notSame",
  "is": "equal",
  "is_a": "type",
  "is_deep": "strictSame",
  "is_deeply": "strictSame",
  "is_dissimilar": "notMatch",
  "is_equal": "equal",
  "is_equivalent": "same",
  "is_inequal": "not",
  "is_inequivalent": "notSame",
  "is_like": "match",
  "is_loose": "same",
  "is_not": "not",
  "is_not_deep_equal": "notSame",
  "is_not_deeply": "notSame",
  "is_not_equal": "not",
  "is_not_equivalent": "notSame",
  "is_not_like": "notMatch",
  "is_not_similar": "notMatch",
  "is_similar": "match",
  "is_strict": "equal",
  "is_strictly": "equal",
  "is_unlike": "notMatch",
  "isa": "type",
  "isA": "type",
  "isdeep": "strictSame",
  "isDeep": "strictSame",
  "isdeeply": "strictSame",
  "isDeeply": "strictSame",
  "isdissimilar": "notMatch",
  "isDissimilar": "notMatch",
  "isequal": "equal",
  "isEqual": "equal",
  "isequivalent": "same",
  "isEquivalent": "same",
  "isinequal": "not",
  "isInequal": "not",
  "isinequivalent": "notSame",
  "isInequivalent": "notSame",
  "islike": "match",
  "isLike": "match",
  "isloose": "same",
  "isLoose": "same",
  "isnot": "not",
  "isNot": "not",
  "isnotdeepequal": "notSame",
  "isNotDeepEqual": "notSame",
  "isnotdeeply": "notSame",
  "isNotDeeply": "notSame",
  "isnotequal": "not",
  "isNotEqual": "not",
  "isnotequivalent": "notSame",
  "isNotEquivalent": "notSame",
  "isnotlike": "notMatch",
  "isNotLike": "notMatch",
  "isnotsimilar": "notMatch",
  "isNotSimilar": "notMatch",
  "issimilar": "match",
  "isSimilar": "match",
  "isstrict": "equal",
  "isStrict": "equal",
  "isstrictly": "equal",
  "isStrictly": "equal",
  "isunlike": "notMatch",
  "isUnlike": "notMatch",
  "like": "match",
  "loose_equal": "same",
  "loose_equals": "same",
  "loose_inequal": "notSame",
  "loose_is": "same",
  "loose_not": "notSame",
  "looseequal": "same",
  "looseEqual": "same",
  "looseequals": "same",
  "looseEquals": "same",
  "looseinequal": "notSame",
  "looseInequal": "notSame",
  "looseis": "same",
  "looseIs": "same",
  "loosenot": "notSame",
  "looseNot": "notSame",
  "matches": "match",
  "not_deep": "notSame",
  "not_deep_equal": "notSame",
  "not_deeply": "strictNotSame",
  "not_equal": "not",
  "not_equals": "not",
  "not_equivalent": "notSame",
  "not_like": "notMatch",
  "not_loose": "notSame",
  "not_match": "notMatch",
  "not_ok": "notOk",
  "not_same": "notSame",
  "not_same_strict": "strictNotSame",
  "not_similar": "notMatch",
  "not_strict_equal": "not",
  "not_strict_equals": "not",
  "not_strict_same": "strictNotSame",
  "not_throw": "doesNotThrow",
  "notdeep": "notSame",
  "notDeep": "notSame",
  "notdeepequal": "notSame",
  "notDeepEqual": "notSame",
  "notdeeply": "strictNotSame",
  "notDeeply": "strictNotSame",
  "notequal": "not",
  "notEqual": "not",
  "notequals": "not",
  "notEquals": "not",
  "notequivalent": "notSame",
  "notEquivalent": "notSame",
  "notlike": "notMatch",
  "notLike": "notMatch",
  "notloose": "notSame",
  "notLoose": "notSame",
  "notmatch": "notMatch",
  "notok": "notOk",
  "notsame": "notSame",
  "notsamestrict": "strictNotSame",
  "notSameStrict": "strictNotSame",
  "notsimilar": "notMatch",
  "notSimilar": "notMatch",
  "notstrictequal": "not",
  "notStrictEqual": "not",
  "notstrictequals": "not",
  "notStrictEquals": "not",
  "notstrictsame": "strictNotSame",
  "notStrictSame": "strictNotSame",
  "notthrow": "doesNotThrow",
  "notThrow": "doesNotThrow",
  "same_strict": "strictSame",
  "samestrict": "strictSame",
  "sameStrict": "strictSame",
  "similar": "match",
  "strict_deep_equal": "strictSame",
  "strict_deep_equals": "strictSame",
  "strict_deep_inequal": "strictNotSame",
  "strict_deep_inequals": "strictNotSame",
  "strict_equal": "equal",
  "strict_equals": "equal",
  "strict_equivalent": "strictSame",
  "strict_inequivalent": "strictNotSame",
  "strict_is": "equal",
  "strict_not_same": "strictNotSame",
  "strict_same": "strictSame",
  "strictdeepequal": "strictSame",
  "strictDeepEqual": "strictSame",
  "strictdeepequals": "strictSame",
  "strictDeepEquals": "strictSame",
  "strictdeepinequal": "strictNotSame",
  "strictDeepInequal": "strictNotSame",
  "strictdeepinequals": "strictNotSame",
  "strictDeepInequals": "strictNotSame",
  "strictequal": "equal",
  "strictEqual": "equal",
  "strictequals": "equal",
  "strictEquals": "equal",
  "strictequivalent": "strictSame",
  "strictEquivalent": "strictSame",
  "strictinequivalent": "strictNotSame",
  "strictInequivalent": "strictNotSame",
  "strictis": "equal",
  "strictIs": "equal",
  "strictnotsame": "strictNotSame",
  "strictsame": "strictSame",
  "throw": "throws",
  "true": "ok",
  "unlike": "notMatch",
  "unsimilar": "notMatch",
}
`
