/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/index.ts > TAP > config 1`] = `
Object {
  "tsx-disable-cache": Object {
    "description": "Tell tsx not to use a cache",
    "type": "boolean",
  },
  "tsx-tsconfig-path": Object {
    "description": "Tell tsx where to find your tsconfig.json file.",
    "type": "string",
  },
}
`
