import { Extra } from './index.js';
/**
 * Print the YAML diagnostics based on the {@link @tapjs/core!index.Extra}
 * object received
 */
export declare const diags: (obj: Extra) => string;
//# sourceMappingURL=diags.d.ts.map