/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/cmd.ts > TAP > basic > bail > b w > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail > b w > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > b w > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > flat > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail > flat > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > flat > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > flat tap > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail > flat tap > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > flat tap > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > lines > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail > lines > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > lines > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > no args > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail > no args > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > no args > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > silent > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail > silent > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > silent > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > silent strict > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail > silent strict > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > silent strict > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > strict > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail > strict > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > strict > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > t > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail > t > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail > t > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > b w > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail_reason > b w > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > b w > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > flat > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail_reason > flat > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > flat > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > flat tap > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail_reason > flat tap > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > flat tap > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > lines > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail_reason > lines > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > lines > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > no args > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail_reason > no args > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > no args > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > silent > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail_reason > silent > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > silent > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > silent strict > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail_reason > silent strict > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > silent strict > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > strict > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail_reason > strict > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > strict > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > t > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > bail_reason > t > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > bail_reason > t > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > b w > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > fail > b w > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > b w > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > flat > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > fail > flat > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > flat > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > flat tap > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > fail > flat tap > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > flat tap > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > lines > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > fail > lines > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > lines > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > no args > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > fail > no args > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > no args > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > silent > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > fail > silent > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > silent > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > silent strict > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > fail > silent strict > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > silent strict > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > strict > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > fail > strict > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > strict > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > t > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > fail > t > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > fail > t > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > b w > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > b w > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > b w > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > flat > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > flat > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > flat > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > flat tap > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > flat tap > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > flat tap > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > lines > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > lines > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > lines > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > no args > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > no args > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > no args > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > silent > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > silent > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > silent > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > silent strict > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > silent strict > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > silent strict > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > strict > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > strict > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > strict > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > t > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > t > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > failing_child_with_broken_tap > t > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > b w > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > pass_mostly > b w > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > b w > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > flat > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > pass_mostly > flat > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > flat > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > flat tap > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > pass_mostly > flat tap > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > flat tap > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > lines > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > pass_mostly > lines > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > lines > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > no args > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > pass_mostly > no args > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > no args > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > silent > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > pass_mostly > silent > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > silent > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > silent strict > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > pass_mostly > silent strict > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > silent strict > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > strict > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > pass_mostly > strict > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > strict > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > t > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > pass_mostly > t > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > pass_mostly > t > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > b w > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > b w > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > b w > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > flat > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > flat > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > flat > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > flat tap > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > flat tap > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > flat tap > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > lines > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > lines > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > lines > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > no args > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > no args > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > no args > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > silent > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > silent > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > silent > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > silent strict > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > silent strict > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > silent strict > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > strict > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > strict > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > strict > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > t > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > t > output 1`] = `

`

exports[`test/cmd.ts > TAP > basic > passing_child_with_broken_tap > t > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > help > error 1`] = `
undefined
`

exports[`test/cmd.ts > TAP > help > output 1`] = `
Usage:
  tap-parser <options>

Parses TAP data from stdin, and outputs the parsed result
in the format specified by the options.  Default output
uses node's \`util.inspect()\` method.

Options:

  -j [<indent>] | --json[=indent]
    Output event data as JSON with the specified indentation (default=2)

  -t | --tap
    Output data as reconstituted TAP based on parsed results

  -l | --lines
    Output each parsed line as it is recognized by the parser

  -b | --bail
    Emit a \`Bail out!\` at the first failed test point encountered

  -B | --no-bail
    Do not bail out at the first failed test point encountered
    (Default)

  -f | --flat
    Flatten all assertions to the top level parser

  -F | --no-flat
    Do not flatten all assertions to the top level parser
    (Default)

  -w | --ignore-all-whitespace
    Skip over blank lines outside of YAML blocks

  -o | --omit-version
    Ignore the \`TAP version 13\` or \`TAP version 14\` line at the start of tests

  --strict
    Run the parser in strict mode

  --no-strict
    Do not run the parser in strict mode

  -s | --silent
    Do not print output, just exit success/failure based on TAP stream


`

exports[`test/cmd.ts > TAP > help > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > json output formatting > --json -f > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > json output formatting > --json -f > output 1`] = `

`

exports[`test/cmd.ts > TAP > json output formatting > --json -f > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > json output formatting > --json 1 > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > json output formatting > --json 1 > output 1`] = `

`

exports[`test/cmd.ts > TAP > json output formatting > --json 1 > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > json output formatting > --json=1 > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > json output formatting > --json=1 > output 1`] = `

`

exports[`test/cmd.ts > TAP > json output formatting > --json=1 > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > json output formatting > -j > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > json output formatting > -j > output 1`] = `

`

exports[`test/cmd.ts > TAP > json output formatting > -j > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > json output formatting > -j 1 > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > json output formatting > -j 1 > output 1`] = `

`

exports[`test/cmd.ts > TAP > json output formatting > -j 1 > stderr 1`] = `

`

exports[`test/cmd.ts > TAP > unrecognized arg > error 1`] = `
0
`

exports[`test/cmd.ts > TAP > unrecognized arg > output 1`] = `

`

exports[`test/cmd.ts > TAP > unrecognized arg > stderr 1`] = `

`
