/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/runs.tsx > TAP > must match snapshot 1`] = `

[1m[43m[38;2;0;0;0m RUNS [39m[49m[22m zro [1m0[22m [32mOK[39m
[1m[43m[38;2;0;0;0m RUNS [39m[49m[22m one [31m1 failed[39m [36m1 skip[39m of [1m3[22m
[1m[43m[38;2;0;0;0m RUNS [39m[49m[22m two [1m0[22m [32mOK[39m
[1m[43m[38;2;0;0;0m RUNS [39m[49m[22m tre [1m0[22m [32mOK[39m
[1m[43m[38;2;0;0;0m RUNS [39m[49m[22m fur [1m0[22m [32mOK[39m
`
