/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/call-site-like.ts > TAP > cjs > create from string error stack line > must match snapshot 1`] = `
{
  "jsStyle": "    at getError ({CWD}/test/fixtures/capture.ts:7:3)",
  "toString": "getError (test/fixtures/capture.ts:7:3)",
}
`

exports[`test/call-site-like.ts > TAP > cjs > turn pretty stack into tap stack, relative external path > must match snapshot 1`] = `
    at /a/b/c:1:2
`

exports[`test/call-site-like.ts > TAP > cjs > turn tap stack into js stack, absolute path > must match snapshot 1`] = `
    at /a/b/c:1:2
`

exports[`test/call-site-like.ts > TAP > mjs > create from string error stack line > must match snapshot 1`] = `
{
  "jsStyle": "    at getError ({CWD}/test/fixtures/capture.ts:7:3)",
  "toString": "getError (test/fixtures/capture.ts:7:3)",
}
`

exports[`test/call-site-like.ts > TAP > mjs > turn pretty stack into tap stack, relative external path > must match snapshot 1`] = `
    at /a/b/c:1:2
`

exports[`test/call-site-like.ts > TAP > mjs > turn tap stack into js stack, absolute path > must match snapshot 1`] = `
    at /a/b/c:1:2
`
