/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/index.ts > TAP > config 1`] = `
Object {
  "mocha-globals": Object {
    "default": true,
    "description": String(
      Automatically inject the BDD/TDD methods into the
                        global environment.
      
                        If this flag is disabled, then you may access these via
                        by importing them from \`@tapjs/mocha-globals\`
                        explicitly.
    ),
    "type": "boolean",
  },
}
`
