/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/index.ts > TAP > just verify the other exports > must match snapshot 1`] = `
Object {
  "dummy-flag": Object {
    "type": "boolean",
  },
  "dummy-flag-set": Object {
    "multiple": true,
    "type": "boolean",
  },
  "dummy-num": Object {
    "type": "number",
  },
  "dummy-num-set": Object {
    "multiple": true,
    "type": "number",
  },
  "dummy-opt": Object {
    "type": "string",
  },
  "dummy-opt-set": Object {
    "multiple": true,
    "type": "string",
  },
}
`
