/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/strict.ts > TAP > strictness is inherited > failures 1`] = `
Array [
  Object {
    "data": "flerggy blerg\\n",
    "tapError": "Non-TAP data encountered in strict mode",
  },
]
`

exports[`test/strict.ts > TAP > unstrict child does not make parent unstrict > failures 1`] = `
Array [
  Object {
    "data": "flerggy blerg\\n",
    "tapError": "Non-TAP data encountered in strict mode",
  },
]
`
