/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/constants.ts > TAP > must match snapshot 1`] = `
Null Object {
  "errorCtorNames": Set {
    "Error",
    "TypeError",
    "RangeError",
    "URIError",
    "SyntaxError",
    "ReferenceError",
    "EvalError",
  },
  "errors": Object {
    "Error": Function Error(),
    "EvalError": Function EvalError(),
    "RangeError": Function RangeError(),
    "ReferenceError": Function ReferenceError(),
    "SyntaxError": Function SyntaxError(),
    "TypeError": Function TypeError(),
    "URIError": Function URIError(),
  },
  "kCustomInspect": Symbol.for(nodejs.util.inspect.custom),
  "kCustomInspectedObject": 4,
  "kInspectedError": 2,
  "kInspectedSymbol": 3,
  "kSerializedError": 0,
  "kSerializedObject": 1,
  "kSerializedSizeHeader": 6,
  "kV8HeaderLength": 2,
}
`
