/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/bailed-out.tsx > TAP > bail out for no raisin > must match snapshot 1`] = `
[1m[31mBailout![39m[22m
`

exports[`test/bailed-out.tsx > TAP > bail out with message > must match snapshot 1`] = `
[1m[31mBailout! (some sort of raisin)[39m[22m
`
