/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/log.tsx > TAP > must match snapshot 1`] = `

XXX test summary zro XXX
XXX test summary one XXX

hello console
moar consule!

[36m[2m1>[22m[39m [2mzro[22m
zro stdout
zro moar stdout

[31m[2m2>[22m[39m [2mzro[22m
zro stderr
zro moar stderr

[31m[2m2>[22m[39m [2mone[22m
one stderr
one moar stderr

[36m[2m1>[22m[39m [2mone[22m
one stdout
one moar stdout

XXX test summary zro XXX
XXX test summary one XXX

hello console
moar consule!

[36m[2m1>[22m[39m [2mzro[22m
zro stdout
zro moar stdout

[31m[2m2>[22m[39m [2mzro[22m
zro stderr
zro moar stderr

[31m[2m2>[22m[39m [2mone[22m
one stderr
one moar stderr

[36m[2m1>[22m[39m [2mone[22m
one stdout
one moar stdout

`
