"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IMPLICIT = void 0;
/**
 * Sigil for implicit end() calls that should not
 * trigger an error if the user then calls t.end()
 */
exports.IMPLICIT = Symbol.for('implicit end');
//# sourceMappingURL=implicit-end-sigil.js.map