/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/test-point-results.ts > TAP > must match snapshot 1`] = `
Object {
  "column": 50,
  "details": Object {
    "duration_ms": 0,
  },
  "file": "{CWD}/test/test-point-results.ts",
  "line": 33,
  "name": "this is fine",
  "nesting": 2,
  "testNumber": 1,
}
`

exports[`test/test-point-results.ts > TAP > must match snapshot 10`] = `
Object {
  "column": 69,
  "details": Object {
    "duration_ms": 0,
    "error": Error: fail with diag {
      "cause": Error: fail with diag,
      "code": "ERR_TEST_FAILURE",
      "failureType": "testCodeFailure",
    },
  },
  "file": "{CWD}/some/file.js",
  "line": 420,
  "name": "fail with diag",
  "nesting": 2,
  "testNumber": 10,
}
`

exports[`test/test-point-results.ts > TAP > must match snapshot 11`] = `
Object {
  "column": 50,
  "details": Object {
    "duration_ms": 0,
    "error": Error: fail no diag {
      "cause": Error: fail no diag,
      "code": "ERR_TEST_FAILURE",
      "failureType": "testCodeFailure",
    },
  },
  "file": "{CWD}/test/test-point-results.ts",
  "line": 33,
  "name": "fail no diag",
  "nesting": 2,
  "testNumber": 11,
}
`

exports[`test/test-point-results.ts > TAP > must match snapshot 2`] = `
Object {
  "column": 50,
  "details": Object {
    "duration_ms": 0,
  },
  "file": "{CWD}/test/test-point-results.ts",
  "line": 33,
  "name": "skip bool",
  "nesting": 2,
  "skip": true,
  "testNumber": 2,
}
`

exports[`test/test-point-results.ts > TAP > must match snapshot 3`] = `
Object {
  "column": 50,
  "details": Object {
    "duration_ms": 0,
  },
  "file": "{CWD}/test/test-point-results.ts",
  "line": 33,
  "name": "skip msg",
  "nesting": 2,
  "skip": "msg",
  "testNumber": 3,
}
`

exports[`test/test-point-results.ts > TAP > must match snapshot 4`] = `
Object {
  "column": 50,
  "details": Object {
    "duration_ms": 0,
  },
  "file": "{CWD}/test/test-point-results.ts",
  "line": 33,
  "name": "todo bool",
  "nesting": 2,
  "testNumber": 4,
  "todo": true,
}
`

exports[`test/test-point-results.ts > TAP > must match snapshot 5`] = `
Object {
  "column": 50,
  "details": Object {
    "duration_ms": 0,
  },
  "file": "{CWD}/test/test-point-results.ts",
  "line": 33,
  "name": "todo msg",
  "nesting": 2,
  "testNumber": 5,
  "todo": "msg",
}
`

exports[`test/test-point-results.ts > TAP > must match snapshot 6`] = `
Object {
  "column": 50,
  "details": Object {
    "duration_ms": 0,
  },
  "file": "{CWD}/test/test-point-results.ts",
  "line": 33,
  "name": "fail skip bool",
  "nesting": 2,
  "skip": true,
  "testNumber": 6,
}
`

exports[`test/test-point-results.ts > TAP > must match snapshot 7`] = `
Object {
  "column": 50,
  "details": Object {
    "duration_ms": 0,
  },
  "file": "{CWD}/test/test-point-results.ts",
  "line": 33,
  "name": "fail skip msg",
  "nesting": 2,
  "skip": "msg",
  "testNumber": 7,
}
`

exports[`test/test-point-results.ts > TAP > must match snapshot 8`] = `
Object {
  "column": 50,
  "details": Object {
    "duration_ms": 0,
  },
  "file": "{CWD}/test/test-point-results.ts",
  "line": 33,
  "name": "fail todo bool",
  "nesting": 2,
  "testNumber": 8,
  "todo": true,
}
`

exports[`test/test-point-results.ts > TAP > must match snapshot 9`] = `
Object {
  "column": 50,
  "details": Object {
    "duration_ms": 0,
  },
  "file": "{CWD}/test/test-point-results.ts",
  "line": 33,
  "name": "fail todo msg",
  "nesting": 2,
  "testNumber": 9,
  "todo": "msg",
}
`
