/**
 * create a symlink from our @tapjs/core into the plugin dir's node_modules
 */
export declare const setPluginCoreLink: (pluginDir: string, core: string) => void;
//# sourceMappingURL=set-plugin-core-link.d.ts.map