/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/replay.ts > TAP > replay a saved test result > must match snapshot 1`] = `
TAP version 14
# Subtest: test.js
    ok 1 - this is fine
    1..1
ok 1 - test.js # time={TIME}

1..1

`
