"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Lists = void 0;
/**
 * List of {@link tap-parser!result.Result} objects corresponding to test
 * points encountered in a TAP stream.
 */
class Lists {
    fail = [];
    todo = [];
    skip = [];
    pass = [];
}
exports.Lists = Lists;
//# sourceMappingURL=lists.js.map