export const synonyms = {
  true: 'ok' as const,
  assert: 'ok' as const,
  notok: 'notOk' as const,
  not_ok: 'notOk' as const,
  false: 'notOk' as const,
  assertNot: 'notOk' as const,
  assertnot: 'notOk' as const,
  assert_not: 'notOk' as const,
  ifError: 'error' as const,
  iferror: 'error' as const,
  if_error: 'error' as const,
  ifErr: 'error' as const,
  iferr: 'error' as const,
  if_err: 'error' as const,
  throw: 'throws' as const,
  doesnotthrow: 'doesNotThrow' as const,
  does_not_throw: 'doesNotThrow' as const,
  notThrow: 'doesNotThrow' as const,
  notthrow: 'doesNotThrow' as const,
  not_throw: 'doesNotThrow' as const,
  equals: 'equal' as const,
  isEqual: 'equal' as const,
  isequal: 'equal' as const,
  is_equal: 'equal' as const,
  is: 'equal' as const,
  strictEqual: 'equal' as const,
  strictequal: 'equal' as const,
  strict_equal: 'equal' as const,
  strictEquals: 'equal' as const,
  strictequals: 'equal' as const,
  strict_equals: 'equal' as const,
  strictIs: 'equal' as const,
  strictis: 'equal' as const,
  strict_is: 'equal' as const,
  isStrict: 'equal' as const,
  isstrict: 'equal' as const,
  is_strict: 'equal' as const,
  isStrictly: 'equal' as const,
  isstrictly: 'equal' as const,
  is_strictly: 'equal' as const,
  identical: 'equal' as const,
  inequal: 'not' as const,
  notEqual: 'not' as const,
  notequal: 'not' as const,
  not_equal: 'not' as const,
  notEquals: 'not' as const,
  notequals: 'not' as const,
  not_equals: 'not' as const,
  notStrictEqual: 'not' as const,
  notstrictequal: 'not' as const,
  not_strict_equal: 'not' as const,
  notStrictEquals: 'not' as const,
  notstrictequals: 'not' as const,
  not_strict_equals: 'not' as const,
  isNotEqual: 'not' as const,
  isnotequal: 'not' as const,
  is_not_equal: 'not' as const,
  isNot: 'not' as const,
  isnot: 'not' as const,
  is_not: 'not' as const,
  doesNotEqual: 'not' as const,
  doesnotequal: 'not' as const,
  does_not_equal: 'not' as const,
  isInequal: 'not' as const,
  isinequal: 'not' as const,
  is_inequal: 'not' as const,
  equivalent: 'same' as const,
  looseEqual: 'same' as const,
  looseequal: 'same' as const,
  loose_equal: 'same' as const,
  looseEquals: 'same' as const,
  looseequals: 'same' as const,
  loose_equals: 'same' as const,
  deepEqual: 'same' as const,
  deepequal: 'same' as const,
  deep_equal: 'same' as const,
  deepEquals: 'same' as const,
  deepequals: 'same' as const,
  deep_equals: 'same' as const,
  isLoose: 'same' as const,
  isloose: 'same' as const,
  is_loose: 'same' as const,
  looseIs: 'same' as const,
  looseis: 'same' as const,
  loose_is: 'same' as const,
  isEquivalent: 'same' as const,
  isequivalent: 'same' as const,
  is_equivalent: 'same' as const,
  notsame: 'notSame' as const,
  not_same: 'notSame' as const,
  inequivalent: 'notSame' as const,
  looseInequal: 'notSame' as const,
  looseinequal: 'notSame' as const,
  loose_inequal: 'notSame' as const,
  notDeep: 'notSame' as const,
  notdeep: 'notSame' as const,
  not_deep: 'notSame' as const,
  deepInequal: 'notSame' as const,
  deepinequal: 'notSame' as const,
  deep_inequal: 'notSame' as const,
  notLoose: 'notSame' as const,
  notloose: 'notSame' as const,
  not_loose: 'notSame' as const,
  looseNot: 'notSame' as const,
  loosenot: 'notSame' as const,
  loose_not: 'notSame' as const,
  notEquivalent: 'notSame' as const,
  notequivalent: 'notSame' as const,
  not_equivalent: 'notSame' as const,
  isNotDeepEqual: 'notSame' as const,
  isnotdeepequal: 'notSame' as const,
  is_not_deep_equal: 'notSame' as const,
  isNotDeeply: 'notSame' as const,
  isnotdeeply: 'notSame' as const,
  is_not_deeply: 'notSame' as const,
  notDeepEqual: 'notSame' as const,
  notdeepequal: 'notSame' as const,
  not_deep_equal: 'notSame' as const,
  isInequivalent: 'notSame' as const,
  isinequivalent: 'notSame' as const,
  is_inequivalent: 'notSame' as const,
  isNotEquivalent: 'notSame' as const,
  isnotequivalent: 'notSame' as const,
  is_not_equivalent: 'notSame' as const,
  strictsame: 'strictSame' as const,
  strict_same: 'strictSame' as const,
  strictEquivalent: 'strictSame' as const,
  strictequivalent: 'strictSame' as const,
  strict_equivalent: 'strictSame' as const,
  strictDeepEqual: 'strictSame' as const,
  strictdeepequal: 'strictSame' as const,
  strict_deep_equal: 'strictSame' as const,
  sameStrict: 'strictSame' as const,
  samestrict: 'strictSame' as const,
  same_strict: 'strictSame' as const,
  deepIs: 'strictSame' as const,
  deepis: 'strictSame' as const,
  deep_is: 'strictSame' as const,
  isDeeply: 'strictSame' as const,
  isdeeply: 'strictSame' as const,
  is_deeply: 'strictSame' as const,
  isDeep: 'strictSame' as const,
  isdeep: 'strictSame' as const,
  is_deep: 'strictSame' as const,
  strictDeepEquals: 'strictSame' as const,
  strictdeepequals: 'strictSame' as const,
  strict_deep_equals: 'strictSame' as const,
  strictnotsame: 'strictNotSame' as const,
  strict_not_same: 'strictNotSame' as const,
  strictInequivalent: 'strictNotSame' as const,
  strictinequivalent: 'strictNotSame' as const,
  strict_inequivalent: 'strictNotSame' as const,
  strictDeepInequal: 'strictNotSame' as const,
  strictdeepinequal: 'strictNotSame' as const,
  strict_deep_inequal: 'strictNotSame' as const,
  notSameStrict: 'strictNotSame' as const,
  notsamestrict: 'strictNotSame' as const,
  not_same_strict: 'strictNotSame' as const,
  deepNot: 'strictNotSame' as const,
  deepnot: 'strictNotSame' as const,
  deep_not: 'strictNotSame' as const,
  notDeeply: 'strictNotSame' as const,
  notdeeply: 'strictNotSame' as const,
  not_deeply: 'strictNotSame' as const,
  strictDeepInequals: 'strictNotSame' as const,
  strictdeepinequals: 'strictNotSame' as const,
  strict_deep_inequals: 'strictNotSame' as const,
  notStrictSame: 'strictNotSame' as const,
  notstrictsame: 'strictNotSame' as const,
  not_strict_same: 'strictNotSame' as const,
  matches: 'match' as const,
  similar: 'match' as const,
  like: 'match' as const,
  isLike: 'match' as const,
  islike: 'match' as const,
  is_like: 'match' as const,
  isSimilar: 'match' as const,
  issimilar: 'match' as const,
  is_similar: 'match' as const,
  hasFields: 'has' as const,
  hasfields: 'has' as const,
  has_fields: 'has' as const,
  includes: 'has' as const,
  include: 'has' as const,
  contains: 'has' as const,
  notmatch: 'notMatch' as const,
  not_match: 'notMatch' as const,
  dissimilar: 'notMatch' as const,
  unsimilar: 'notMatch' as const,
  notSimilar: 'notMatch' as const,
  notsimilar: 'notMatch' as const,
  not_similar: 'notMatch' as const,
  unlike: 'notMatch' as const,
  isUnlike: 'notMatch' as const,
  isunlike: 'notMatch' as const,
  is_unlike: 'notMatch' as const,
  notLike: 'notMatch' as const,
  notlike: 'notMatch' as const,
  not_like: 'notMatch' as const,
  isNotLike: 'notMatch' as const,
  isnotlike: 'notMatch' as const,
  is_not_like: 'notMatch' as const,
  doesNotHave: 'notMatch' as const,
  doesnothave: 'notMatch' as const,
  does_not_have: 'notMatch' as const,
  isNotSimilar: 'notMatch' as const,
  isnotsimilar: 'notMatch' as const,
  is_not_similar: 'notMatch' as const,
  isDissimilar: 'notMatch' as const,
  isdissimilar: 'notMatch' as const,
  is_dissimilar: 'notMatch' as const,
  isA: 'type' as const,
  isa: 'type' as const,
  is_a: 'type' as const,
  done: 'end' as const,
}
