/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/get-install-set.ts > TAP > cannot resolve version > stderr 1`] = `
Array [
  Array [
    "\\u001b[31mCannot install x.\\u001b[39m",
  ],
  Array [
    "Version not found, or incompatible with the current tap version.",
  ],
]
`

exports[`test/get-install-set.ts > TAP > cannot resolve version > stdout 1`] = `
Array []
`

exports[`test/get-install-set.ts > TAP > plugin already built into tap > stderr 1`] = `
Array []
`

exports[`test/get-install-set.ts > TAP > plugin already built into tap > stdout 1`] = `
Array []
`

exports[`test/get-install-set.ts > TAP > plugin already installed > stderr 1`] = `
Array []
`

exports[`test/get-install-set.ts > TAP > plugin already installed > stdout 1`] = `
Array []
`

exports[`test/get-install-set.ts > TAP > plugin exists, but wrong version > stderr 1`] = `
Array []
`

exports[`test/get-install-set.ts > TAP > plugin exists, but wrong version > stdout 1`] = `
Array []
`

exports[`test/get-install-set.ts > TAP > plugin not installed > stderr 1`] = `
Array []
`

exports[`test/get-install-set.ts > TAP > plugin not installed > stdout 1`] = `
Array []
`

exports[`test/get-install-set.ts > TAP > re-adding disabled builtin > stderr 1`] = `
Array []
`

exports[`test/get-install-set.ts > TAP > re-adding disabled builtin > stdout 1`] = `
Array []
`
