/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/suite-summary.tsx > TAP > 1 fail of 1 > must match snapshot 1`] = `

[1mAsserts:[22m  [32m0 pass[39m  [1m[31m1 fail[39m[22m  [2m1 of 1 complete[22m
[1mSuites:[22m   [32m0 pass[39m  [1m[31m1 fail[39m[22m  [2m1 of 1 complete[22m

`

exports[`test/suite-summary.tsx > TAP > 1 passing of 1 > must match snapshot 1`] = `

[1mAsserts:[22m  [32m1 pass[39m  [31m[2m0 fail[22m[39m  [2m1 of 1 complete[22m
[1mSuites:[22m   [32m1 pass[39m  [31m[2m0 fail[22m[39m  [2m1 of 1 complete[22m

`

exports[`test/suite-summary.tsx > TAP > all the things > must match snapshot 1`] = `

[1mAsserts:[22m  [32m1 pass[39m  [1m[31m1 fail[39m[22m  [36m1 skip[39m  [35m1 todo[39m  [2m2 of 4 complete[22m
[1mSuites:[22m   [32m1 pass[39m  [1m[31m1 fail[39m[22m  [36m1 skip[39m          [2m2 of 3 complete[22m

`

exports[`test/suite-summary.tsx > TAP > bailed out for no raisin > must match snapshot 1`] = `

[1mAsserts:[22m  [32m1 pass[39m  [1m[31m2 fail[39m[22m  [36m14 skip[39m  [2m3 of 17 complete[22m
[1mSuites:[22m   [32m1 pass[39m  [1m[31m1 fail[39m[22m  [36m14 skip[39m  [2m2 of 16 complete[22m
[1m[31mBailout![39m[22m

`

exports[`test/suite-summary.tsx > TAP > bailed out for with raisin > must match snapshot 1`] = `

[1mAsserts:[22m  [32m1 pass[39m  [1m[31m2 fail[39m[22m  [36m14 skip[39m  [2m3 of 17 complete[22m
[1mSuites:[22m   [32m1 pass[39m  [1m[31m1 fail[39m[22m  [36m14 skip[39m  [2m2 of 16 complete[22m
[1m[31mBailout! (i have my raisins)[39m[22m

`

exports[`test/suite-summary.tsx > TAP > comments > must match snapshot 1`] = `

[1mAsserts:[22m  [32m0 pass[39m  [31m[2m0 fail[22m[39m  [2m0 of 1 complete[22m
[1mSuites:[22m   [32m1 pass[39m  [31m[2m0 fail[22m[39m  [2m1 of 1 complete[22m

[2mone[22m
[2mtwo[22m

`

exports[`test/suite-summary.tsx > TAP > skip only > must match snapshot 1`] = `

[1mAsserts:[22m  [32m0 pass[39m  [31m[2m0 fail[22m[39m  [36m1 skip[39m  [2m0 of 99 complete[22m
[1mSuites:[22m   [32m0 pass[39m  [31m[2m0 fail[22m[39m  [36m1 skip[39m  [2m0 of 10 complete[22m

`

exports[`test/suite-summary.tsx > TAP > timed out with signal > must match snapshot 1`] = `

[1mAsserts:[22m  [32m0 pass[39m  [31m[2m0 fail[22m[39m  [2m0 of 1 complete[22m
[1mSuites:[22m   [32m1 pass[39m  [31m[2m0 fail[22m[39m  [2m1 of 1 complete[22m
[1m[31mTimed out (SIGALRM)[39m[22m

`

exports[`test/suite-summary.tsx > TAP > timed out without signal > must match snapshot 1`] = `

[1mAsserts:[22m  [32m0 pass[39m  [31m[2m0 fail[22m[39m  [2m0 of 1 complete[22m
[1mSuites:[22m   [32m1 pass[39m  [31m[2m0 fail[22m[39m  [2m1 of 1 complete[22m
[1m[31mTimed out[39m[22m

`
