/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/parse-stringify.ts > TAP > tap-tests.tap > bail > parsed 1`] = `
Array [
  Array [
    "version",
    13,
  ],
  Array [
    "comment",
    "# buffer_compare.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# same buffers\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 1,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# not same buffers\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should not be equivalent",
      "id": 2,
      "name": "should not be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 2\\n",
  ],
  Array [
    "comment",
    "# pass  2\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/buffer_compare.js",
      "id": 3,
      "name": "test/buffer_compare.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# common.js\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "just setup, nothing relevant",
      "id": 4,
      "name": "just setup, nothing relevant",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/common.js",
      "id": 5,
      "name": "test/common.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# consumer.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# basic.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 6,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# indent.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 7,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# missing.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 8,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# skip-all.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 9,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# yamlish.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 10,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 5\\n",
  ],
  Array [
    "comment",
    "# pass  5\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/consumer.js",
      "id": 11,
      "name": "test/consumer.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# debug-test.js\\n",
  ],
  Array [
    "extra",
    "debug test\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "'Debugger listening on port 5858\\\\n'\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# debug test\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "Should output debugger message",
      "id": 12,
      "name": "Should output debugger message",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 1\\n",
  ],
  Array [
    "comment",
    "# pass  1\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/debug-test.js",
      "id": 13,
      "name": "test/debug-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deep-strict.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# strictDeepEquals shouldn't care about key order\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be strictly equal",
      "id": 14,
      "name": "should be strictly equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# strictDeepEquals shouldn't care about key order recursively\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be strictly equal",
      "id": 15,
      "name": "should be strictly equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# strictDeepEquals shoudn't care about key order (but still might)\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be strictly equal",
      "id": 16,
      "name": "should be strictly equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/deep-strict.js",
      "id": 17,
      "name": "test/deep-strict.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deep.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# deepEquals shouldn't care about key order and types\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 18,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deepEquals shouldn't care about key order recursively and types\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 19,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deepEquals shoudn't care about key order (but still might) and types\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 20,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/deep.js",
      "id": 21,
      "name": "test/deep.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# executed.sh\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "File with executable bit should be executed",
      "id": 22,
      "name": "File with executable bit should be executed",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/executed.sh",
      "id": 23,
      "name": "test/executed.sh",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# exit-code.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# exit code 1 when tap results show failure\\n",
  ],
  Array [
    "comment",
    "# test exits 0, has failures\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 24,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 25,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test exits 1, has failures\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 26,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 27,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test exits 1, has no failures\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 28,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 29,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# successes exit 0\\n",
  ],
  Array [
    "comment",
    "# test that does nothing, but exits 0\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 30,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 31,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test that succeeds, and exits 0\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 32,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 33,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 10\\n",
  ],
  Array [
    "comment",
    "# pass  10\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/exit-code.js",
      "id": 34,
      "name": "test/exit-code.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# expose-gc-test.js\\n",
  ],
  Array [
    "extra",
    "gc test\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "assert gc does not exist\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# gc test when the gc isn't there\\n",
  ],
  Array [
    "extra",
    "gc test\\n",
  ],
  Array [
    "extra",
    "t.plan=2\\n",
  ],
  Array [
    "extra",
    "gc test using --gc\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "assert gc exists\\n",
  ],
  Array [
    "extra",
    "gc test using --expose-gc\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "assert gc exists\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 35,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# gc test when the gc should be there\\n",
  ],
  Array [
    "comment",
    "# test for gc using --gc\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 36,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test for gc using --expose-gc\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 37,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# cleanup\\n",
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/expose-gc-test.js",
      "id": 38,
      "name": "test/expose-gc-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# global-harness-async.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# outer\\n",
  ],
  Array [
    "comment",
    "# inner 1\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "1-1",
      "id": 39,
      "name": "1-1",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# inner 2\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "2-1",
      "id": 40,
      "name": "2-1",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# inner 3\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "3-1",
      "id": 41,
      "name": "3-1",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/global-harness-async.js",
      "id": 42,
      "name": "test/global-harness-async.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# independent-timeouts.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# finishes in time\\n",
  ],
  Array [
    "comment",
    "# finishes in time too\\n",
  ],
  Array [
    "comment",
    "# tests 0\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/independent-timeouts.js",
      "id": 43,
      "name": "test/independent-timeouts.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# isolated-conf-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# one\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 44,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 45,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# two\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 46,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 47,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 4\\n",
  ],
  Array [
    "comment",
    "# pass  4\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/isolated-conf-test.js",
      "id": 48,
      "name": "test/isolated-conf-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# meta-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# meta test\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "sanity check",
      "id": 49,
      "name": "sanity check",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "not ok",
      "id": 50,
      "name": "not ok",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "total test count",
      "id": 51,
      "name": "total test count",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "tests passed",
      "id": 52,
      "name": "tests passed",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "tests failed",
      "id": 53,
      "name": "tests failed",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ok is boolean",
      "id": 54,
      "name": "ok is boolean",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip is number",
      "id": 55,
      "name": "skip is number",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "results isa Results",
      "id": 56,
      "name": "results isa Results",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test isa Test",
      "id": 57,
      "name": "test isa Test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test isa Harness",
      "id": 58,
      "name": "test isa Harness",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 10\\n",
  ],
  Array [
    "comment",
    "# pass  10\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/meta-test.js",
      "id": 59,
      "name": "test/meta-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# nested-async.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# Harness async test support\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "sync child A",
      "id": 60,
      "name": "sync child A",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# sync child B\\n",
  ],
  Array [
    "comment",
    "# async grandchild A\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "(unnamed assert)",
      "id": 61,
      "name": "(unnamed assert)",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# async grandchild B\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "(unnamed assert)",
      "id": 62,
      "name": "(unnamed assert)",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# async child\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "sync grandchild in async child A",
      "id": 63,
      "name": "sync grandchild in async child A",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# sync grandchild in async child B\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "(unnamed assert)",
      "id": 64,
      "name": "(unnamed assert)",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 5\\n",
  ],
  Array [
    "comment",
    "# pass  5\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/nested-async.js",
      "id": 65,
      "name": "test/nested-async.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# nested-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# parent\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "p test",
      "id": 66,
      "name": "p test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# subtest\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ch test",
      "id": 67,
      "name": "ch test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# nested subtest\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "grch test",
      "id": 68,
      "name": "grch test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# another subtest\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ch test 2",
      "id": 69,
      "name": "ch test 2",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 4\\n",
  ],
  Array [
    "comment",
    "# pass  4\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/nested-test.js",
      "id": 70,
      "name": "test/nested-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# non-tap-output.js\\n",
  ],
  Array [
    "comment",
    "# everything is fine\\n",
  ],
  Array [
    "comment",
    "# there are no errors\\n",
  ],
  Array [
    "comment",
    "# this output is not haiku.\\n",
  ],
  Array [
    "comment",
    "# is 8 ok?\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": ", 8 can stay.",
      "id": 71,
      "name": ", 8 can stay.",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# but: nevertheless, here we are\\n",
  ],
  Array [
    "comment",
    "# this: is indented\\n",
  ],
  Array [
    "comment",
    "# and: it\\n",
  ],
  Array [
    "comment",
    "# might: ~\\n",
  ],
  Array [
    "comment",
    "# be: yaml?\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "might be confusing",
      "id": 72,
      "name": "might be confusing",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "done now, exiting",
      "id": 73,
      "name": "done now, exiting",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/non-tap-output.js",
      "id": 74,
      "name": "test/non-tap-output.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# not-executed.sh\\n",
  ],
  Array [
    "comment",
    "# output-childtest-description.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# nested tests, parent and child pass\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs parent description",
      "id": 75,
      "name": "outputs parent description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs child description",
      "id": 76,
      "name": "outputs child description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs parent description before parent result",
      "id": 77,
      "name": "outputs parent description before parent result",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs parent result before child description",
      "id": 78,
      "name": "outputs parent result before child description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs child description before child result",
      "id": 79,
      "name": "outputs child description before child result",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 5\\n",
  ],
  Array [
    "comment",
    "# pass  5\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/output-childtest-description.js",
      "id": 80,
      "name": "test/output-childtest-description.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# result-trap.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# trap result\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 81,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 82,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 2\\n",
  ],
  Array [
    "comment",
    "# pass  2\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/result-trap.js",
      "id": 83,
      "name": "test/result-trap.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# segv.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# setup\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "compiled seg faulter",
      "id": 84,
      "name": "compiled seg faulter",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# segv\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 85,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 86,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 87,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 88,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 89,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 90,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# cleanup\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "cleaned up",
      "id": 91,
      "name": "cleaned up",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 8\\n",
  ],
  Array [
    "comment",
    "# pass  8\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/segv.js",
      "id": 92,
      "name": "test/segv.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# simple-harness-test-with-plan.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# trivial success\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "it works",
      "id": 93,
      "name": "it works",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# two tests\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "math should work",
      "id": 94,
      "name": "math should work",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false should not be ok",
      "id": 95,
      "name": "false should not be ok",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/simple-harness-test-with-plan.js",
      "id": 96,
      "name": "test/simple-harness-test-with-plan.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# simple-harness-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# trivial success\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "it works",
      "id": 97,
      "name": "it works",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# two tests\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "math should work",
      "id": 98,
      "name": "math should work",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false should not be ok",
      "id": 99,
      "name": "false should not be ok",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/simple-harness-test.js",
      "id": 100,
      "name": "test/simple-harness-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-assert-todo-skip.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# not much\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "always passes",
      "id": 101,
      "name": "always passes",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": "skip it good",
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false",
      "id": 102,
      "name": "false",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": "always fails",
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "bonus",
      "id": 103,
      "name": "bonus",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": "remove todo directive",
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "expected",
      "id": 104,
      "name": "expected",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": "implement a thing",
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "always passes without explanation",
      "id": 105,
      "name": "always passes without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": true,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false without explanation",
      "id": 106,
      "name": "false without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": true,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "bonus without explanation",
      "id": 107,
      "name": "bonus without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": true,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "expected without explanation",
      "id": 108,
      "name": "expected without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": true,
    },
  ],
  Array [
    "comment",
    "# tests 8\\n",
  ],
  Array [
    "comment",
    "# skip  4\\n",
  ],
  Array [
    "comment",
    "# todo  4\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/test-assert-todo-skip.js",
      "id": 109,
      "name": "test/test-assert-todo-skip.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-descriptions.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# captures test descriptions\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "exit cleanly",
      "id": 110,
      "name": "exit cleanly",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures SKIP description",
      "id": 111,
      "name": "captures SKIP description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary is not from file",
      "id": 112,
      "name": "skip summary is not from file",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not from file",
      "id": 113,
      "name": "todo summary is not from file",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "column": 7,
        "file": "child_process.js",
        "line": 707,
        "stack": Array [
          "getCaller (/some/path/tap-assert.js:439:17)\\n",
          "Function.assert (/some/path/tap-assert.js:21:16)\\n",
          "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
          "/some/path/test/test-descriptions.js:32:7\\n",
          "ChildProcess.exithandler (child_process.js:707:7)\\n",
          "emitTwo (events.js:87:13)\\n",
          "ChildProcess.emit (events.js:169:7)\\n",
          "maybeClose (child_process.js:984:16)\\n",
          "Socket.<anonymous> (child_process.js:1153:11)\\n",
          "emitOne (events.js:77:13)\\n",
        ],
      },
      "fullname": "captures TODO description",
      "id": 114,
      "name": "captures TODO description",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "bailout",
    "captures TODO description",
  ],
  Array [
    "complete",
    FinalResults {
      "bailout": "captures TODO description",
      "count": 114,
      "fail": 1,
      "failures": Array [
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "column": 7,
            "file": "child_process.js",
            "line": 707,
            "stack": Array [
              "getCaller (/some/path/tap-assert.js:439:17)\\n",
              "Function.assert (/some/path/tap-assert.js:21:16)\\n",
              "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
              "/some/path/test/test-descriptions.js:32:7\\n",
              "ChildProcess.exithandler (child_process.js:707:7)\\n",
              "emitTwo (events.js:87:13)\\n",
              "ChildProcess.emit (events.js:169:7)\\n",
              "maybeClose (child_process.js:984:16)\\n",
              "Socket.<anonymous> (child_process.js:1153:11)\\n",
              "emitOne (events.js:77:13)\\n",
            ],
          },
          "fullname": "captures TODO description",
          "id": 114,
          "name": "captures TODO description",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
      ],
      "ok": false,
      "pass": 113,
      "passes": undefined,
      "plan": FinalPlan {
        "comment": "",
        "end": null,
        "skipAll": false,
        "skipReason": "",
        "start": null,
      },
      "skip": 4,
      "skips": Array [
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "always passes",
          "id": 101,
          "name": "always passes",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": "skip it good",
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "false",
          "id": 102,
          "name": "false",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": "always fails",
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "always passes without explanation",
          "id": 105,
          "name": "always passes without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": true,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "false without explanation",
          "id": 106,
          "name": "false without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": true,
          "tapError": null,
          "time": null,
          "todo": false,
        },
      ],
      "time": null,
      "todo": 4,
      "todos": Array [
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "bonus",
          "id": 103,
          "name": "bonus",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": "remove todo directive",
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "expected",
          "id": 104,
          "name": "expected",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": "implement a thing",
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "bonus without explanation",
          "id": 107,
          "name": "bonus without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": true,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "expected without explanation",
          "id": 108,
          "name": "expected without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": true,
        },
      ],
    },
  ],
  Array [
    "finish",
  ],
  Array [
    "close",
  ],
]
`

exports[`test/parse-stringify.ts > TAP > tap-tests.tap > bail > stringified 1`] = `
TAP version 13
# buffer_compare.js
# TAP version 13
# same buffers
ok 1 - should be equivalent
# not same buffers
ok 2 - should not be equivalent
# tests 2
# pass  2
# ok
ok 3 - test/buffer_compare.js
# common.js
ok 4 - just setup, nothing relevant
ok 5 - test/common.js
# consumer.js
# TAP version 13
# basic.tap
ok 6 - should be equivalent
# indent.tap
ok 7 - should be equivalent
# missing.tap
ok 8 - should be equivalent
# skip-all.tap
ok 9 - should be equivalent
# yamlish.tap
ok 10 - should be equivalent
# tests 5
# pass  5
# ok
ok 11 - test/consumer.js
# debug-test.js
debug test
t.plan=1
'Debugger listening on port 5858\\n'
# TAP version 13
# debug test
ok 12 - Should output debugger message
# tests 1
# pass  1
# ok
ok 13 - test/debug-test.js
# deep-strict.js
# TAP version 13
# strictDeepEquals shouldn't care about key order
ok 14 - should be strictly equal
# strictDeepEquals shouldn't care about key order recursively
ok 15 - should be strictly equal
# strictDeepEquals shoudn't care about key order (but still might)
ok 16 - should be strictly equal
# tests 3
# pass  3
# ok
ok 17 - test/deep-strict.js
# deep.js
# TAP version 13
# deepEquals shouldn't care about key order and types
ok 18 - should be equivalent
# deepEquals shouldn't care about key order recursively and types
ok 19 - should be equivalent
# deepEquals shoudn't care about key order (but still might) and types
ok 20 - should be equivalent
# tests 3
# pass  3
# ok
ok 21 - test/deep.js
# executed.sh
ok 22 - File with executable bit should be executed
ok 23 - test/executed.sh
# exit-code.js
# TAP version 13
# exit code 1 when tap results show failure
# test exits 0, has failures
ok 24 - should be equal
ok 25 - should be equal
# test exits 1, has failures
ok 26 - should be equal
ok 27 - should be equal
# test exits 1, has no failures
ok 28 - should be equal
ok 29 - should be equal
# successes exit 0
# test that does nothing, but exits 0
ok 30 - should be equal
ok 31 - should be equal
# test that succeeds, and exits 0
ok 32 - should be equal
ok 33 - should be equal
# tests 10
# pass  10
# ok
ok 34 - test/exit-code.js
# expose-gc-test.js
gc test
t.plan=1
assert gc does not exist
# TAP version 13
# gc test when the gc isn't there
gc test
t.plan=2
gc test using --gc
t.plan=1
assert gc exists
gc test using --expose-gc
t.plan=1
assert gc exists
ok 35 - should be equal
# gc test when the gc should be there
# test for gc using --gc
ok 36 - should be equal
# test for gc using --expose-gc
ok 37 - should be equal
# cleanup
# tests 3
# pass  3
# ok
ok 38 - test/expose-gc-test.js
# global-harness-async.js
# TAP version 13
# outer
# inner 1
ok 39 - 1-1
# inner 2
ok 40 - 2-1
# inner 3
ok 41 - 3-1
ok 42 - test/global-harness-async.js
# independent-timeouts.js
# TAP version 13
# finishes in time
# finishes in time too
# tests 0
# ok
ok 43 - test/independent-timeouts.js
# isolated-conf-test.js
# TAP version 13
# one
ok 44 - should be equal
ok 45 - should be equal
# two
ok 46 - should be equal
ok 47 - should be equal
# tests 4
# pass  4
# ok
ok 48 - test/isolated-conf-test.js
# meta-test.js
# TAP version 13
# meta test
ok 49 - sanity check
ok 50 - not ok
ok 51 - total test count
ok 52 - tests passed
ok 53 - tests failed
ok 54 - ok is boolean
ok 55 - skip is number
ok 56 - results isa Results
ok 57 - test isa Test
ok 58 - test isa Harness
# tests 10
# pass  10
# ok
ok 59 - test/meta-test.js
# nested-async.js
# TAP version 13
# Harness async test support
ok 60 - sync child A
# sync child B
# async grandchild A
ok 61 - (unnamed assert)
# async grandchild B
ok 62 - (unnamed assert)
# async child
ok 63 - sync grandchild in async child A
# sync grandchild in async child B
ok 64 - (unnamed assert)
# tests 5
# pass  5
# ok
ok 65 - test/nested-async.js
# nested-test.js
# TAP version 13
# parent
ok 66 - p test
# subtest
ok 67 - ch test
# nested subtest
ok 68 - grch test
# another subtest
ok 69 - ch test 2
# tests 4
# pass  4
# ok
ok 70 - test/nested-test.js
# non-tap-output.js
# everything is fine
# there are no errors
# this output is not haiku.
# is 8 ok?
ok 71 - , 8 can stay.
# but: nevertheless, here we are
# this: is indented
# and: it
# might: ~
# be: yaml?
ok 72 - might be confusing
ok 73 - done now, exiting
ok 74 - test/non-tap-output.js
# not-executed.sh
# output-childtest-description.js
# TAP version 13
# nested tests, parent and child pass
ok 75 - outputs parent description
ok 76 - outputs child description
ok 77 - outputs parent description before parent result
ok 78 - outputs parent result before child description
ok 79 - outputs child description before child result
# tests 5
# pass  5
# ok
ok 80 - test/output-childtest-description.js
# result-trap.js
# TAP version 13
# trap result
ok 81 - should be equal
ok 82 - should be equal
# tests 2
# pass  2
# ok
ok 83 - test/result-trap.js
# segv.js
# TAP version 13
# setup
ok 84 - compiled seg faulter
# segv
ok 85 - should be equivalent
ok 86 - should be equivalent
ok 87 - should be equivalent
ok 88 - should be equivalent
ok 89 - should be equivalent
ok 90 - should be equal
# cleanup
ok 91 - cleaned up
# tests 8
# pass  8
# ok
ok 92 - test/segv.js
# simple-harness-test-with-plan.js
# TAP version 13
# trivial success
ok 93 - it works
# two tests
ok 94 - math should work
ok 95 - false should not be ok
# tests 3
# pass  3
# ok
ok 96 - test/simple-harness-test-with-plan.js
# simple-harness-test.js
# TAP version 13
# trivial success
ok 97 - it works
# two tests
ok 98 - math should work
ok 99 - false should not be ok
# tests 3
# pass  3
# ok
ok 100 - test/simple-harness-test.js
# test-assert-todo-skip.js
# TAP version 13
# not much
ok 101 - always passes # SKIP skip it good
ok 102 - false # SKIP always fails
ok 103 - bonus # TODO remove todo directive
ok 104 - expected # TODO implement a thing
ok 105 - always passes without explanation # SKIP
ok 106 - false without explanation # SKIP
ok 107 - bonus without explanation # TODO
ok 108 - expected without explanation # TODO
# tests 8
# skip  4
# todo  4
ok 109 - test/test-assert-todo-skip.js
# test-descriptions.js
# TAP version 13
# captures test descriptions
ok 110 - exit cleanly
ok 111 - captures SKIP description
ok 112 - skip summary is not from file
ok 113 - todo summary is not from file
not ok 114 - captures TODO description
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:32:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Socket.<anonymous> (child_process.js:1153:11)
    - |
      emitOne (events.js:77:13)
  ...
Bail out! captures TODO description

`

exports[`test/parse-stringify.ts > TAP > tap-tests.tap > bail > stringified flat 1`] = `
TAP version 13
# buffer_compare.js
# TAP version 13
# same buffers
ok 1 - should be equivalent
# not same buffers
ok 2 - should not be equivalent
# tests 2
# pass  2
# ok
ok 3 - test/buffer_compare.js
# common.js
ok 4 - just setup, nothing relevant
ok 5 - test/common.js
# consumer.js
# TAP version 13
# basic.tap
ok 6 - should be equivalent
# indent.tap
ok 7 - should be equivalent
# missing.tap
ok 8 - should be equivalent
# skip-all.tap
ok 9 - should be equivalent
# yamlish.tap
ok 10 - should be equivalent
# tests 5
# pass  5
# ok
ok 11 - test/consumer.js
# debug-test.js
debug test
t.plan=1
'Debugger listening on port 5858\\n'
# TAP version 13
# debug test
ok 12 - Should output debugger message
# tests 1
# pass  1
# ok
ok 13 - test/debug-test.js
# deep-strict.js
# TAP version 13
# strictDeepEquals shouldn't care about key order
ok 14 - should be strictly equal
# strictDeepEquals shouldn't care about key order recursively
ok 15 - should be strictly equal
# strictDeepEquals shoudn't care about key order (but still might)
ok 16 - should be strictly equal
# tests 3
# pass  3
# ok
ok 17 - test/deep-strict.js
# deep.js
# TAP version 13
# deepEquals shouldn't care about key order and types
ok 18 - should be equivalent
# deepEquals shouldn't care about key order recursively and types
ok 19 - should be equivalent
# deepEquals shoudn't care about key order (but still might) and types
ok 20 - should be equivalent
# tests 3
# pass  3
# ok
ok 21 - test/deep.js
# executed.sh
ok 22 - File with executable bit should be executed
ok 23 - test/executed.sh
# exit-code.js
# TAP version 13
# exit code 1 when tap results show failure
# test exits 0, has failures
ok 24 - should be equal
ok 25 - should be equal
# test exits 1, has failures
ok 26 - should be equal
ok 27 - should be equal
# test exits 1, has no failures
ok 28 - should be equal
ok 29 - should be equal
# successes exit 0
# test that does nothing, but exits 0
ok 30 - should be equal
ok 31 - should be equal
# test that succeeds, and exits 0
ok 32 - should be equal
ok 33 - should be equal
# tests 10
# pass  10
# ok
ok 34 - test/exit-code.js
# expose-gc-test.js
gc test
t.plan=1
assert gc does not exist
# TAP version 13
# gc test when the gc isn't there
gc test
t.plan=2
gc test using --gc
t.plan=1
assert gc exists
gc test using --expose-gc
t.plan=1
assert gc exists
ok 35 - should be equal
# gc test when the gc should be there
# test for gc using --gc
ok 36 - should be equal
# test for gc using --expose-gc
ok 37 - should be equal
# cleanup
# tests 3
# pass  3
# ok
ok 38 - test/expose-gc-test.js
# global-harness-async.js
# TAP version 13
# outer
# inner 1
ok 39 - 1-1
# inner 2
ok 40 - 2-1
# inner 3
ok 41 - 3-1
ok 42 - test/global-harness-async.js
# independent-timeouts.js
# TAP version 13
# finishes in time
# finishes in time too
# tests 0
# ok
ok 43 - test/independent-timeouts.js
# isolated-conf-test.js
# TAP version 13
# one
ok 44 - should be equal
ok 45 - should be equal
# two
ok 46 - should be equal
ok 47 - should be equal
# tests 4
# pass  4
# ok
ok 48 - test/isolated-conf-test.js
# meta-test.js
# TAP version 13
# meta test
ok 49 - sanity check
ok 50 - not ok
ok 51 - total test count
ok 52 - tests passed
ok 53 - tests failed
ok 54 - ok is boolean
ok 55 - skip is number
ok 56 - results isa Results
ok 57 - test isa Test
ok 58 - test isa Harness
# tests 10
# pass  10
# ok
ok 59 - test/meta-test.js
# nested-async.js
# TAP version 13
# Harness async test support
ok 60 - sync child A
# sync child B
# async grandchild A
ok 61 - (unnamed assert)
# async grandchild B
ok 62 - (unnamed assert)
# async child
ok 63 - sync grandchild in async child A
# sync grandchild in async child B
ok 64 - (unnamed assert)
# tests 5
# pass  5
# ok
ok 65 - test/nested-async.js
# nested-test.js
# TAP version 13
# parent
ok 66 - p test
# subtest
ok 67 - ch test
# nested subtest
ok 68 - grch test
# another subtest
ok 69 - ch test 2
# tests 4
# pass  4
# ok
ok 70 - test/nested-test.js
# non-tap-output.js
# everything is fine
# there are no errors
# this output is not haiku.
# is 8 ok?
ok 71 - , 8 can stay.
# but: nevertheless, here we are
# this: is indented
# and: it
# might: ~
# be: yaml?
ok 72 - might be confusing
ok 73 - done now, exiting
ok 74 - test/non-tap-output.js
# not-executed.sh
# output-childtest-description.js
# TAP version 13
# nested tests, parent and child pass
ok 75 - outputs parent description
ok 76 - outputs child description
ok 77 - outputs parent description before parent result
ok 78 - outputs parent result before child description
ok 79 - outputs child description before child result
# tests 5
# pass  5
# ok
ok 80 - test/output-childtest-description.js
# result-trap.js
# TAP version 13
# trap result
ok 81 - should be equal
ok 82 - should be equal
# tests 2
# pass  2
# ok
ok 83 - test/result-trap.js
# segv.js
# TAP version 13
# setup
ok 84 - compiled seg faulter
# segv
ok 85 - should be equivalent
ok 86 - should be equivalent
ok 87 - should be equivalent
ok 88 - should be equivalent
ok 89 - should be equivalent
ok 90 - should be equal
# cleanup
ok 91 - cleaned up
# tests 8
# pass  8
# ok
ok 92 - test/segv.js
# simple-harness-test-with-plan.js
# TAP version 13
# trivial success
ok 93 - it works
# two tests
ok 94 - math should work
ok 95 - false should not be ok
# tests 3
# pass  3
# ok
ok 96 - test/simple-harness-test-with-plan.js
# simple-harness-test.js
# TAP version 13
# trivial success
ok 97 - it works
# two tests
ok 98 - math should work
ok 99 - false should not be ok
# tests 3
# pass  3
# ok
ok 100 - test/simple-harness-test.js
# test-assert-todo-skip.js
# TAP version 13
# not much
ok 101 - always passes # SKIP skip it good
ok 102 - false # SKIP always fails
ok 103 - bonus # TODO remove todo directive
ok 104 - expected # TODO implement a thing
ok 105 - always passes without explanation # SKIP
ok 106 - false without explanation # SKIP
ok 107 - bonus without explanation # TODO
ok 108 - expected without explanation # TODO
# tests 8
# skip  4
# todo  4
ok 109 - test/test-assert-todo-skip.js
# test-descriptions.js
# TAP version 13
# captures test descriptions
ok 110 - exit cleanly
ok 111 - captures SKIP description
ok 112 - skip summary is not from file
ok 113 - todo summary is not from file
not ok 114 - captures TODO description
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:32:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Socket.<anonymous> (child_process.js:1153:11)
    - |
      emitOne (events.js:77:13)
  ...
Bail out! captures TODO description

`

exports[`test/parse-stringify.ts > TAP > tap-tests.tap > default settings > parsed 1`] = `
Array [
  Array [
    "version",
    13,
  ],
  Array [
    "comment",
    "# buffer_compare.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# same buffers\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 1,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# not same buffers\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should not be equivalent",
      "id": 2,
      "name": "should not be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 2\\n",
  ],
  Array [
    "comment",
    "# pass  2\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/buffer_compare.js",
      "id": 3,
      "name": "test/buffer_compare.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# common.js\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "just setup, nothing relevant",
      "id": 4,
      "name": "just setup, nothing relevant",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/common.js",
      "id": 5,
      "name": "test/common.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# consumer.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# basic.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 6,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# indent.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 7,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# missing.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 8,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# skip-all.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 9,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# yamlish.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 10,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 5\\n",
  ],
  Array [
    "comment",
    "# pass  5\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/consumer.js",
      "id": 11,
      "name": "test/consumer.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# debug-test.js\\n",
  ],
  Array [
    "extra",
    "debug test\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "'Debugger listening on port 5858\\\\n'\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# debug test\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "Should output debugger message",
      "id": 12,
      "name": "Should output debugger message",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 1\\n",
  ],
  Array [
    "comment",
    "# pass  1\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/debug-test.js",
      "id": 13,
      "name": "test/debug-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deep-strict.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# strictDeepEquals shouldn't care about key order\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be strictly equal",
      "id": 14,
      "name": "should be strictly equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# strictDeepEquals shouldn't care about key order recursively\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be strictly equal",
      "id": 15,
      "name": "should be strictly equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# strictDeepEquals shoudn't care about key order (but still might)\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be strictly equal",
      "id": 16,
      "name": "should be strictly equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/deep-strict.js",
      "id": 17,
      "name": "test/deep-strict.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deep.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# deepEquals shouldn't care about key order and types\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 18,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deepEquals shouldn't care about key order recursively and types\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 19,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deepEquals shoudn't care about key order (but still might) and types\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 20,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/deep.js",
      "id": 21,
      "name": "test/deep.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# executed.sh\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "File with executable bit should be executed",
      "id": 22,
      "name": "File with executable bit should be executed",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/executed.sh",
      "id": 23,
      "name": "test/executed.sh",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# exit-code.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# exit code 1 when tap results show failure\\n",
  ],
  Array [
    "comment",
    "# test exits 0, has failures\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 24,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 25,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test exits 1, has failures\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 26,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 27,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test exits 1, has no failures\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 28,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 29,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# successes exit 0\\n",
  ],
  Array [
    "comment",
    "# test that does nothing, but exits 0\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 30,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 31,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test that succeeds, and exits 0\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 32,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 33,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 10\\n",
  ],
  Array [
    "comment",
    "# pass  10\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/exit-code.js",
      "id": 34,
      "name": "test/exit-code.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# expose-gc-test.js\\n",
  ],
  Array [
    "extra",
    "gc test\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "assert gc does not exist\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# gc test when the gc isn't there\\n",
  ],
  Array [
    "extra",
    "gc test\\n",
  ],
  Array [
    "extra",
    "t.plan=2\\n",
  ],
  Array [
    "extra",
    "gc test using --gc\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "assert gc exists\\n",
  ],
  Array [
    "extra",
    "gc test using --expose-gc\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "assert gc exists\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 35,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# gc test when the gc should be there\\n",
  ],
  Array [
    "comment",
    "# test for gc using --gc\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 36,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test for gc using --expose-gc\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 37,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# cleanup\\n",
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/expose-gc-test.js",
      "id": 38,
      "name": "test/expose-gc-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# global-harness-async.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# outer\\n",
  ],
  Array [
    "comment",
    "# inner 1\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "1-1",
      "id": 39,
      "name": "1-1",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# inner 2\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "2-1",
      "id": 40,
      "name": "2-1",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# inner 3\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "3-1",
      "id": 41,
      "name": "3-1",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/global-harness-async.js",
      "id": 42,
      "name": "test/global-harness-async.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# independent-timeouts.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# finishes in time\\n",
  ],
  Array [
    "comment",
    "# finishes in time too\\n",
  ],
  Array [
    "comment",
    "# tests 0\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/independent-timeouts.js",
      "id": 43,
      "name": "test/independent-timeouts.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# isolated-conf-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# one\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 44,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 45,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# two\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 46,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 47,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 4\\n",
  ],
  Array [
    "comment",
    "# pass  4\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/isolated-conf-test.js",
      "id": 48,
      "name": "test/isolated-conf-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# meta-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# meta test\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "sanity check",
      "id": 49,
      "name": "sanity check",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "not ok",
      "id": 50,
      "name": "not ok",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "total test count",
      "id": 51,
      "name": "total test count",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "tests passed",
      "id": 52,
      "name": "tests passed",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "tests failed",
      "id": 53,
      "name": "tests failed",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ok is boolean",
      "id": 54,
      "name": "ok is boolean",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip is number",
      "id": 55,
      "name": "skip is number",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "results isa Results",
      "id": 56,
      "name": "results isa Results",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test isa Test",
      "id": 57,
      "name": "test isa Test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test isa Harness",
      "id": 58,
      "name": "test isa Harness",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 10\\n",
  ],
  Array [
    "comment",
    "# pass  10\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/meta-test.js",
      "id": 59,
      "name": "test/meta-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# nested-async.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# Harness async test support\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "sync child A",
      "id": 60,
      "name": "sync child A",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# sync child B\\n",
  ],
  Array [
    "comment",
    "# async grandchild A\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "(unnamed assert)",
      "id": 61,
      "name": "(unnamed assert)",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# async grandchild B\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "(unnamed assert)",
      "id": 62,
      "name": "(unnamed assert)",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# async child\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "sync grandchild in async child A",
      "id": 63,
      "name": "sync grandchild in async child A",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# sync grandchild in async child B\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "(unnamed assert)",
      "id": 64,
      "name": "(unnamed assert)",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 5\\n",
  ],
  Array [
    "comment",
    "# pass  5\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/nested-async.js",
      "id": 65,
      "name": "test/nested-async.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# nested-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# parent\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "p test",
      "id": 66,
      "name": "p test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# subtest\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ch test",
      "id": 67,
      "name": "ch test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# nested subtest\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "grch test",
      "id": 68,
      "name": "grch test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# another subtest\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ch test 2",
      "id": 69,
      "name": "ch test 2",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 4\\n",
  ],
  Array [
    "comment",
    "# pass  4\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/nested-test.js",
      "id": 70,
      "name": "test/nested-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# non-tap-output.js\\n",
  ],
  Array [
    "comment",
    "# everything is fine\\n",
  ],
  Array [
    "comment",
    "# there are no errors\\n",
  ],
  Array [
    "comment",
    "# this output is not haiku.\\n",
  ],
  Array [
    "comment",
    "# is 8 ok?\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": ", 8 can stay.",
      "id": 71,
      "name": ", 8 can stay.",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# but: nevertheless, here we are\\n",
  ],
  Array [
    "comment",
    "# this: is indented\\n",
  ],
  Array [
    "comment",
    "# and: it\\n",
  ],
  Array [
    "comment",
    "# might: ~\\n",
  ],
  Array [
    "comment",
    "# be: yaml?\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "might be confusing",
      "id": 72,
      "name": "might be confusing",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "done now, exiting",
      "id": 73,
      "name": "done now, exiting",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/non-tap-output.js",
      "id": 74,
      "name": "test/non-tap-output.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# not-executed.sh\\n",
  ],
  Array [
    "comment",
    "# output-childtest-description.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# nested tests, parent and child pass\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs parent description",
      "id": 75,
      "name": "outputs parent description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs child description",
      "id": 76,
      "name": "outputs child description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs parent description before parent result",
      "id": 77,
      "name": "outputs parent description before parent result",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs parent result before child description",
      "id": 78,
      "name": "outputs parent result before child description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs child description before child result",
      "id": 79,
      "name": "outputs child description before child result",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 5\\n",
  ],
  Array [
    "comment",
    "# pass  5\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/output-childtest-description.js",
      "id": 80,
      "name": "test/output-childtest-description.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# result-trap.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# trap result\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 81,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 82,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 2\\n",
  ],
  Array [
    "comment",
    "# pass  2\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/result-trap.js",
      "id": 83,
      "name": "test/result-trap.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# segv.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# setup\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "compiled seg faulter",
      "id": 84,
      "name": "compiled seg faulter",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# segv\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 85,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 86,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 87,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 88,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 89,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 90,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# cleanup\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "cleaned up",
      "id": 91,
      "name": "cleaned up",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 8\\n",
  ],
  Array [
    "comment",
    "# pass  8\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/segv.js",
      "id": 92,
      "name": "test/segv.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# simple-harness-test-with-plan.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# trivial success\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "it works",
      "id": 93,
      "name": "it works",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# two tests\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "math should work",
      "id": 94,
      "name": "math should work",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false should not be ok",
      "id": 95,
      "name": "false should not be ok",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/simple-harness-test-with-plan.js",
      "id": 96,
      "name": "test/simple-harness-test-with-plan.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# simple-harness-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# trivial success\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "it works",
      "id": 97,
      "name": "it works",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# two tests\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "math should work",
      "id": 98,
      "name": "math should work",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false should not be ok",
      "id": 99,
      "name": "false should not be ok",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/simple-harness-test.js",
      "id": 100,
      "name": "test/simple-harness-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-assert-todo-skip.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# not much\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "always passes",
      "id": 101,
      "name": "always passes",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": "skip it good",
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false",
      "id": 102,
      "name": "false",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": "always fails",
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "bonus",
      "id": 103,
      "name": "bonus",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": "remove todo directive",
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "expected",
      "id": 104,
      "name": "expected",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": "implement a thing",
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "always passes without explanation",
      "id": 105,
      "name": "always passes without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": true,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false without explanation",
      "id": 106,
      "name": "false without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": true,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "bonus without explanation",
      "id": 107,
      "name": "bonus without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": true,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "expected without explanation",
      "id": 108,
      "name": "expected without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": true,
    },
  ],
  Array [
    "comment",
    "# tests 8\\n",
  ],
  Array [
    "comment",
    "# skip  4\\n",
  ],
  Array [
    "comment",
    "# todo  4\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/test-assert-todo-skip.js",
      "id": 109,
      "name": "test/test-assert-todo-skip.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-descriptions.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# captures test descriptions\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "exit cleanly",
      "id": 110,
      "name": "exit cleanly",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures SKIP description",
      "id": 111,
      "name": "captures SKIP description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary is not from file",
      "id": 112,
      "name": "skip summary is not from file",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not from file",
      "id": 113,
      "name": "todo summary is not from file",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "column": 7,
        "file": "child_process.js",
        "line": 707,
        "stack": Array [
          "getCaller (/some/path/tap-assert.js:439:17)\\n",
          "Function.assert (/some/path/tap-assert.js:21:16)\\n",
          "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
          "/some/path/test/test-descriptions.js:32:7\\n",
          "ChildProcess.exithandler (child_process.js:707:7)\\n",
          "emitTwo (events.js:87:13)\\n",
          "ChildProcess.emit (events.js:169:7)\\n",
          "maybeClose (child_process.js:984:16)\\n",
          "Socket.<anonymous> (child_process.js:1153:11)\\n",
          "emitOne (events.js:77:13)\\n",
        ],
      },
      "fullname": "captures TODO description",
      "id": 114,
      "name": "captures TODO description",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "exit cleanly",
      "id": 115,
      "name": "exit cleanly",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures SKIP description",
      "id": 116,
      "name": "captures SKIP description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary is not in TAP output",
      "id": 117,
      "name": "skip summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not in TAP output",
      "id": 118,
      "name": "todo summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "column": 7,
        "file": "child_process.js",
        "line": 707,
        "stack": Array [
          "getCaller (/some/path/tap-assert.js:439:17)\\n",
          "Function.assert (/some/path/tap-assert.js:21:16)\\n",
          "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
          "/some/path/test/test-descriptions.js:18:7\\n",
          "ChildProcess.exithandler (child_process.js:707:7)\\n",
          "emitTwo (events.js:87:13)\\n",
          "ChildProcess.emit (events.js:169:7)\\n",
          "maybeClose (child_process.js:984:16)\\n",
          "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
        ],
      },
      "fullname": "captures TODO description",
      "id": 119,
      "name": "captures TODO description",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "exit cleanly",
      "id": 120,
      "name": "exit cleanly",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "column": 7,
        "file": "child_process.js",
        "line": 707,
        "stack": Array [
          "getCaller (/some/path/tap-assert.js:439:17)\\n",
          "Function.assert (/some/path/tap-assert.js:21:16)\\n",
          "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
          "/some/path/test/test-descriptions.js:23:7\\n",
          "ChildProcess.exithandler (child_process.js:707:7)\\n",
          "emitTwo (events.js:87:13)\\n",
          "ChildProcess.emit (events.js:169:7)\\n",
          "maybeClose (child_process.js:984:16)\\n",
          "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
        ],
      },
      "fullname": "summarizes skipped count",
      "id": 121,
      "name": "summarizes skipped count",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "column": 7,
        "file": "child_process.js",
        "line": 707,
        "stack": Array [
          "getCaller (/some/path/tap-assert.js:439:17)\\n",
          "Function.assert (/some/path/tap-assert.js:21:16)\\n",
          "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
          "/some/path/test/test-descriptions.js:24:7\\n",
          "ChildProcess.exithandler (child_process.js:707:7)\\n",
          "emitTwo (events.js:87:13)\\n",
          "ChildProcess.emit (events.js:169:7)\\n",
          "maybeClose (child_process.js:984:16)\\n",
          "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
        ],
      },
      "fullname": "summarizes todo count",
      "id": 122,
      "name": "summarizes todo count",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 13\\n",
  ],
  Array [
    "comment",
    "# pass  9\\n",
  ],
  Array [
    "comment",
    "# fail  4\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "command": "/usr/local/bin/iojs test-descriptions.js",
        "exit": 1,
      },
      "fullname": "test/test-descriptions.js",
      "id": 123,
      "name": "test/test-descriptions.js",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-directives.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# captures test descriptions\\n",
  ],
  Array [
    "comment",
    "# raw TAP > TAP consumer > TAP producer\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "overall result is PASS",
      "id": 124,
      "name": "overall result is PASS",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures ok SKIP",
      "id": 125,
      "name": "captures ok SKIP",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures not ok SKIP",
      "id": 126,
      "name": "captures not ok SKIP",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary not in TAP output",
      "id": 127,
      "name": "skip summary not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures ok TODO",
      "id": 128,
      "name": "captures ok TODO",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures not ok TODO",
      "id": 129,
      "name": "captures not ok TODO",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not in TAP output",
      "id": 130,
      "name": "todo summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "no ugly \\"undefined\\" in output",
      "id": 131,
      "name": "no ugly \\"undefined\\" in output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# raw TAP > TAP consumer > summary\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "overall result is PASS",
      "id": 132,
      "name": "overall result is PASS",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "no SKIP in summary",
      "id": 133,
      "name": "no SKIP in summary",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary is not in TAP output",
      "id": 134,
      "name": "skip summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "no TODO in summary",
      "id": 135,
      "name": "no TODO in summary",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not in TAP output",
      "id": 136,
      "name": "todo summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "no ugly \\"undefined\\" in output",
      "id": 137,
      "name": "no ugly \\"undefined\\" in output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# TAP producer via require(\\"tap\\")\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "overall result is PASS",
      "id": 138,
      "name": "overall result is PASS",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures ok SKIP",
      "id": 139,
      "name": "captures ok SKIP",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures not ok SKIP",
      "id": 140,
      "name": "captures not ok SKIP",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary not in TAP output",
      "id": 141,
      "name": "skip summary not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures ok TODO",
      "id": 142,
      "name": "captures ok TODO",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures not ok TODO",
      "id": 143,
      "name": "captures not ok TODO",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not in TAP output",
      "id": 144,
      "name": "todo summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "no ugly \\"undefined\\" in output",
      "id": 145,
      "name": "no ugly \\"undefined\\" in output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 22\\n",
  ],
  Array [
    "comment",
    "# pass  22\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/test-directives.js",
      "id": 146,
      "name": "test/test-directives.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-skip.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "does not count as failure",
      "id": 147,
      "name": "does not count as failure",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": true,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 1\\n",
  ],
  Array [
    "comment",
    "# skip  1\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/test-skip.js",
      "id": 148,
      "name": "test/test-skip.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# testing the test object\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test object should be instanceof Test",
      "id": 149,
      "name": "test object should be instanceof Test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test object should be instanceof Harness",
      "id": 150,
      "name": "test object should be instanceof Harness",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test._Test should be the Test class",
      "id": 151,
      "name": "test._Test should be the Test class",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotDeepEqual method",
      "id": 152,
      "name": "should have isNotDeepEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotDeepEqual method should be a function",
      "id": 153,
      "name": "isNotDeepEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have equals method",
      "id": 154,
      "name": "should have equals method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "equals method should be a function",
      "id": 155,
      "name": "equals method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have inequivalent method",
      "id": 156,
      "name": "should have inequivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "inequivalent method should be a function",
      "id": 157,
      "name": "inequivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have threw method",
      "id": 158,
      "name": "should have threw method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "threw method should be a function",
      "id": 159,
      "name": "threw method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have strictEqual method",
      "id": 160,
      "name": "should have strictEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "strictEqual method should be a function",
      "id": 161,
      "name": "strictEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have emit method",
      "id": 162,
      "name": "should have emit method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "emit method should be a function",
      "id": 163,
      "name": "emit method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have fail method",
      "id": 164,
      "name": "should have fail method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "fail method should be a function",
      "id": 165,
      "name": "fail method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have strictEquals method",
      "id": 166,
      "name": "should have strictEquals method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "strictEquals method should be a function",
      "id": 167,
      "name": "strictEquals method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notLike method",
      "id": 168,
      "name": "should have notLike method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notLike method should be a function",
      "id": 169,
      "name": "notLike method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have dissimilar method",
      "id": 170,
      "name": "should have dissimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "dissimilar method should be a function",
      "id": 171,
      "name": "dissimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have true method",
      "id": 172,
      "name": "should have true method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "true method should be a function",
      "id": 173,
      "name": "true method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have assert method",
      "id": 174,
      "name": "should have assert method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "assert method should be a function",
      "id": 175,
      "name": "assert method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have is method",
      "id": 176,
      "name": "should have is method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "is method should be a function",
      "id": 177,
      "name": "is method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have ok method",
      "id": 178,
      "name": "should have ok method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ok method should be a function",
      "id": 179,
      "name": "ok method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isEqual method",
      "id": 180,
      "name": "should have isEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isEqual method should be a function",
      "id": 181,
      "name": "isEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isDeeply method",
      "id": 182,
      "name": "should have isDeeply method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isDeeply method should be a function",
      "id": 183,
      "name": "isDeeply method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have deepEqual method",
      "id": 184,
      "name": "should have deepEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "deepEqual method should be a function",
      "id": 185,
      "name": "deepEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have deepEquals method",
      "id": 186,
      "name": "should have deepEquals method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "deepEquals method should be a function",
      "id": 187,
      "name": "deepEquals method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have pass method",
      "id": 188,
      "name": "should have pass method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "pass method should be a function",
      "id": 189,
      "name": "pass method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have length method",
      "id": 190,
      "name": "should have length method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "length method should be a function",
      "id": 191,
      "name": "length method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have skip method",
      "id": 192,
      "name": "should have skip method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip method should be a function",
      "id": 193,
      "name": "skip method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotEqual method",
      "id": 194,
      "name": "should have isNotEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotEqual method should be a function",
      "id": 195,
      "name": "isNotEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have looseEquals method",
      "id": 196,
      "name": "should have looseEquals method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "looseEquals method should be a function",
      "id": 197,
      "name": "looseEquals method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have false method",
      "id": 198,
      "name": "should have false method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false method should be a function",
      "id": 199,
      "name": "false method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notDeeply method",
      "id": 200,
      "name": "should have notDeeply method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notDeeply method should be a function",
      "id": 201,
      "name": "notDeeply method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have ifErr method",
      "id": 202,
      "name": "should have ifErr method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ifErr method should be a function",
      "id": 203,
      "name": "ifErr method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have hasFields method",
      "id": 204,
      "name": "should have hasFields method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "hasFields method should be a function",
      "id": 205,
      "name": "hasFields method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotDeeply method",
      "id": 206,
      "name": "should have isNotDeeply method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotDeeply method should be a function",
      "id": 207,
      "name": "isNotDeeply method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have like method",
      "id": 208,
      "name": "should have like method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "like method should be a function",
      "id": 209,
      "name": "like method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have similar method",
      "id": 210,
      "name": "should have similar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "similar method should be a function",
      "id": 211,
      "name": "similar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notOk method",
      "id": 212,
      "name": "should have notOk method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notOk method should be a function",
      "id": 213,
      "name": "notOk method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isDissimilar method",
      "id": 214,
      "name": "should have isDissimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isDissimilar method should be a function",
      "id": 215,
      "name": "isDissimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isEquivalent method",
      "id": 216,
      "name": "should have isEquivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isEquivalent method should be a function",
      "id": 217,
      "name": "isEquivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have doesNotEqual method",
      "id": 218,
      "name": "should have doesNotEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "doesNotEqual method should be a function",
      "id": 219,
      "name": "doesNotEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isSimilar method",
      "id": 220,
      "name": "should have isSimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isSimilar method should be a function",
      "id": 221,
      "name": "isSimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notDeepEqual method",
      "id": 222,
      "name": "should have notDeepEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notDeepEqual method should be a function",
      "id": 223,
      "name": "notDeepEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have type method",
      "id": 224,
      "name": "should have type method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "type method should be a function",
      "id": 225,
      "name": "type method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notok method",
      "id": 226,
      "name": "should have notok method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notok method should be a function",
      "id": 227,
      "name": "notok method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isInequivalent method",
      "id": 228,
      "name": "should have isInequivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isInequivalent method should be a function",
      "id": 229,
      "name": "isInequivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNot method",
      "id": 230,
      "name": "should have isNot method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNot method should be a function",
      "id": 231,
      "name": "isNot method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have same method",
      "id": 232,
      "name": "should have same method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "same method should be a function",
      "id": 233,
      "name": "same method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isInequal method",
      "id": 234,
      "name": "should have isInequal method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isInequal method should be a function",
      "id": 235,
      "name": "isInequal method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have _endNice method",
      "id": 236,
      "name": "should have _endNice method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "_endNice method should be a function",
      "id": 237,
      "name": "_endNice method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have ifError method",
      "id": 238,
      "name": "should have ifError method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ifError method should be a function",
      "id": 239,
      "name": "ifError method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have iferror method",
      "id": 240,
      "name": "should have iferror method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "iferror method should be a function",
      "id": 241,
      "name": "iferror method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have clear method",
      "id": 242,
      "name": "should have clear method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "clear method should be a function",
      "id": 243,
      "name": "clear method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have has method",
      "id": 244,
      "name": "should have has method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "has method should be a function",
      "id": 245,
      "name": "has method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have not method",
      "id": 246,
      "name": "should have not method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "not method should be a function",
      "id": 247,
      "name": "not method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have timeout method",
      "id": 248,
      "name": "should have timeout method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "timeout method should be a function",
      "id": 249,
      "name": "timeout method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notSimilar method",
      "id": 250,
      "name": "should have notSimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notSimilar method should be a function",
      "id": 251,
      "name": "notSimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isUnlike method",
      "id": 252,
      "name": "should have isUnlike method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isUnlike method should be a function",
      "id": 253,
      "name": "isUnlike method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notEquals method",
      "id": 254,
      "name": "should have notEquals method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notEquals method should be a function",
      "id": 255,
      "name": "notEquals method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have unsimilar method",
      "id": 256,
      "name": "should have unsimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "unsimilar method should be a function",
      "id": 257,
      "name": "unsimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have result method",
      "id": 258,
      "name": "should have result method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "result method should be a function",
      "id": 259,
      "name": "result method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have doesNotThrow method",
      "id": 260,
      "name": "should have doesNotThrow method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "doesNotThrow method should be a function",
      "id": 261,
      "name": "doesNotThrow method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have error method",
      "id": 262,
      "name": "should have error method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "error method should be a function",
      "id": 263,
      "name": "error method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have constructor method",
      "id": 264,
      "name": "should have constructor method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "constructor method should be a function",
      "id": 265,
      "name": "constructor method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notEqual method",
      "id": 266,
      "name": "should have notEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notEqual method should be a function",
      "id": 267,
      "name": "notEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have throws method",
      "id": 268,
      "name": "should have throws method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "throws method should be a function",
      "id": 269,
      "name": "throws method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isLike method",
      "id": 270,
      "name": "should have isLike method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isLike method should be a function",
      "id": 271,
      "name": "isLike method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotSimilar method",
      "id": 272,
      "name": "should have isNotSimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotSimilar method should be a function",
      "id": 273,
      "name": "isNotSimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotEquivalent method",
      "id": 274,
      "name": "should have isNotEquivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotEquivalent method should be a function",
      "id": 275,
      "name": "isNotEquivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have inequal method",
      "id": 276,
      "name": "should have inequal method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "inequal method should be a function",
      "id": 277,
      "name": "inequal method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notEquivalent method",
      "id": 278,
      "name": "should have notEquivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notEquivalent method should be a function",
      "id": 279,
      "name": "notEquivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotLike method",
      "id": 280,
      "name": "should have isNotLike method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotLike method should be a function",
      "id": 281,
      "name": "isNotLike method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have equivalent method",
      "id": 282,
      "name": "should have equivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "equivalent method should be a function",
      "id": 283,
      "name": "equivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have looseEqual method",
      "id": 284,
      "name": "should have looseEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "looseEqual method should be a function",
      "id": 285,
      "name": "looseEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have equal method",
      "id": 286,
      "name": "should have equal method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "equal method should be a function",
      "id": 287,
      "name": "equal method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have unlike method",
      "id": 288,
      "name": "should have unlike method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "unlike method should be a function",
      "id": 289,
      "name": "unlike method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have doesNotHave method",
      "id": 290,
      "name": "should have doesNotHave method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "doesNotHave method should be a function",
      "id": 291,
      "name": "doesNotHave method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have comment method",
      "id": 292,
      "name": "should have comment method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "comment method should be a function",
      "id": 293,
      "name": "comment method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isa method",
      "id": 294,
      "name": "should have isa method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isa method should be a function",
      "id": 295,
      "name": "isa method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 147\\n",
  ],
  Array [
    "comment",
    "# pass  147\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/test-test.js",
      "id": 296,
      "name": "test/test-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# timeout.js\\n",
  ],
  Array [
    "extra",
    "timeout test\\n",
  ],
  Array [
    "extra",
    "t.plan=2\\n",
  ],
  Array [
    "extra",
    "a assert\\n",
  ],
  Array [
    "extra",
    "b assert\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# timeout test with plan only\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "a",
      "id": 297,
      "name": "a",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "extra",
    "timeout test\\n",
  ],
  Array [
    "extra",
    "t.plan=2\\n",
  ],
  Array [
    "extra",
    "a assert\\n",
  ],
  Array [
    "extra",
    "b assert\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "b",
      "id": 298,
      "name": "b",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# timeout test with plan and end\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "a",
      "id": 299,
      "name": "a",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "b",
      "id": 300,
      "name": "b",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 4\\n",
  ],
  Array [
    "comment",
    "# pass  4\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/timeout.js",
      "id": 301,
      "name": "test/timeout.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# trivial-success.js\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/trivial-success.js",
      "id": 302,
      "name": "test/trivial-success.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# undefined_indented.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# consume yaml\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 303,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 1\\n",
  ],
  Array [
    "comment",
    "# pass  1\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/undefined_indented.js",
      "id": 304,
      "name": "test/undefined_indented.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# valid-command.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# valid command\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 305,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 306,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 307,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 308,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 309,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 310,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 6\\n",
  ],
  Array [
    "comment",
    "# pass  6\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/valid-command.js",
      "id": 311,
      "name": "test/valid-command.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "plan",
    Plan {
      "comment": "",
      "end": 311,
      "start": 1,
    },
  ],
  Array [
    "comment",
    "# tests 311\\n",
  ],
  Array [
    "comment",
    "# pass  297\\n",
  ],
  Array [
    "comment",
    "# fail  5\\n",
  ],
  Array [
    "comment",
    "# skip  5\\n",
  ],
  Array [
    "comment",
    "# todo  4\\n",
  ],
  Array [
    "complete",
    FinalResults {
      "bailout": false,
      "count": 311,
      "fail": 5,
      "failures": Array [
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "column": 7,
            "file": "child_process.js",
            "line": 707,
            "stack": Array [
              "getCaller (/some/path/tap-assert.js:439:17)\\n",
              "Function.assert (/some/path/tap-assert.js:21:16)\\n",
              "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
              "/some/path/test/test-descriptions.js:32:7\\n",
              "ChildProcess.exithandler (child_process.js:707:7)\\n",
              "emitTwo (events.js:87:13)\\n",
              "ChildProcess.emit (events.js:169:7)\\n",
              "maybeClose (child_process.js:984:16)\\n",
              "Socket.<anonymous> (child_process.js:1153:11)\\n",
              "emitOne (events.js:77:13)\\n",
            ],
          },
          "fullname": "captures TODO description",
          "id": 114,
          "name": "captures TODO description",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "column": 7,
            "file": "child_process.js",
            "line": 707,
            "stack": Array [
              "getCaller (/some/path/tap-assert.js:439:17)\\n",
              "Function.assert (/some/path/tap-assert.js:21:16)\\n",
              "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
              "/some/path/test/test-descriptions.js:18:7\\n",
              "ChildProcess.exithandler (child_process.js:707:7)\\n",
              "emitTwo (events.js:87:13)\\n",
              "ChildProcess.emit (events.js:169:7)\\n",
              "maybeClose (child_process.js:984:16)\\n",
              "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
            ],
          },
          "fullname": "captures TODO description",
          "id": 119,
          "name": "captures TODO description",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "column": 7,
            "file": "child_process.js",
            "line": 707,
            "stack": Array [
              "getCaller (/some/path/tap-assert.js:439:17)\\n",
              "Function.assert (/some/path/tap-assert.js:21:16)\\n",
              "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
              "/some/path/test/test-descriptions.js:23:7\\n",
              "ChildProcess.exithandler (child_process.js:707:7)\\n",
              "emitTwo (events.js:87:13)\\n",
              "ChildProcess.emit (events.js:169:7)\\n",
              "maybeClose (child_process.js:984:16)\\n",
              "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
            ],
          },
          "fullname": "summarizes skipped count",
          "id": 121,
          "name": "summarizes skipped count",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "column": 7,
            "file": "child_process.js",
            "line": 707,
            "stack": Array [
              "getCaller (/some/path/tap-assert.js:439:17)\\n",
              "Function.assert (/some/path/tap-assert.js:21:16)\\n",
              "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
              "/some/path/test/test-descriptions.js:24:7\\n",
              "ChildProcess.exithandler (child_process.js:707:7)\\n",
              "emitTwo (events.js:87:13)\\n",
              "ChildProcess.emit (events.js:169:7)\\n",
              "maybeClose (child_process.js:984:16)\\n",
              "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
            ],
          },
          "fullname": "summarizes todo count",
          "id": 122,
          "name": "summarizes todo count",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "command": "/usr/local/bin/iojs test-descriptions.js",
            "exit": 1,
          },
          "fullname": "test/test-descriptions.js",
          "id": 123,
          "name": "test/test-descriptions.js",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
      ],
      "ok": false,
      "pass": 306,
      "passes": undefined,
      "plan": FinalPlan {
        "comment": "",
        "end": 311,
        "skipAll": false,
        "skipReason": "",
        "start": 1,
      },
      "skip": 5,
      "skips": Array [
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "always passes",
          "id": 101,
          "name": "always passes",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": "skip it good",
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "false",
          "id": 102,
          "name": "false",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": "always fails",
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "always passes without explanation",
          "id": 105,
          "name": "always passes without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": true,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "false without explanation",
          "id": 106,
          "name": "false without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": true,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "does not count as failure",
          "id": 147,
          "name": "does not count as failure",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": true,
          "tapError": null,
          "time": null,
          "todo": false,
        },
      ],
      "time": null,
      "todo": 4,
      "todos": Array [
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "bonus",
          "id": 103,
          "name": "bonus",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": "remove todo directive",
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "expected",
          "id": 104,
          "name": "expected",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": "implement a thing",
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "bonus without explanation",
          "id": 107,
          "name": "bonus without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": true,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "expected without explanation",
          "id": 108,
          "name": "expected without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": true,
        },
      ],
    },
  ],
  Array [
    "finish",
  ],
  Array [
    "close",
  ],
]
`

exports[`test/parse-stringify.ts > TAP > tap-tests.tap > default settings > parsed flat 1`] = `
Array [
  Array [
    "version",
    13,
  ],
  Array [
    "comment",
    "# buffer_compare.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# same buffers\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 1,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# not same buffers\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should not be equivalent",
      "id": 2,
      "name": "should not be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 2\\n",
  ],
  Array [
    "comment",
    "# pass  2\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/buffer_compare.js",
      "id": 3,
      "name": "test/buffer_compare.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# common.js\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "just setup, nothing relevant",
      "id": 4,
      "name": "just setup, nothing relevant",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/common.js",
      "id": 5,
      "name": "test/common.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# consumer.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# basic.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 6,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# indent.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 7,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# missing.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 8,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# skip-all.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 9,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# yamlish.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 10,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 5\\n",
  ],
  Array [
    "comment",
    "# pass  5\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/consumer.js",
      "id": 11,
      "name": "test/consumer.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# debug-test.js\\n",
  ],
  Array [
    "extra",
    "debug test\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "'Debugger listening on port 5858\\\\n'\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# debug test\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "Should output debugger message",
      "id": 12,
      "name": "Should output debugger message",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 1\\n",
  ],
  Array [
    "comment",
    "# pass  1\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/debug-test.js",
      "id": 13,
      "name": "test/debug-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deep-strict.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# strictDeepEquals shouldn't care about key order\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be strictly equal",
      "id": 14,
      "name": "should be strictly equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# strictDeepEquals shouldn't care about key order recursively\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be strictly equal",
      "id": 15,
      "name": "should be strictly equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# strictDeepEquals shoudn't care about key order (but still might)\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be strictly equal",
      "id": 16,
      "name": "should be strictly equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/deep-strict.js",
      "id": 17,
      "name": "test/deep-strict.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deep.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# deepEquals shouldn't care about key order and types\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 18,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deepEquals shouldn't care about key order recursively and types\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 19,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deepEquals shoudn't care about key order (but still might) and types\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 20,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/deep.js",
      "id": 21,
      "name": "test/deep.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# executed.sh\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "File with executable bit should be executed",
      "id": 22,
      "name": "File with executable bit should be executed",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/executed.sh",
      "id": 23,
      "name": "test/executed.sh",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# exit-code.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# exit code 1 when tap results show failure\\n",
  ],
  Array [
    "comment",
    "# test exits 0, has failures\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 24,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 25,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test exits 1, has failures\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 26,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 27,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test exits 1, has no failures\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 28,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 29,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# successes exit 0\\n",
  ],
  Array [
    "comment",
    "# test that does nothing, but exits 0\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 30,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 31,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test that succeeds, and exits 0\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 32,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 33,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 10\\n",
  ],
  Array [
    "comment",
    "# pass  10\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/exit-code.js",
      "id": 34,
      "name": "test/exit-code.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# expose-gc-test.js\\n",
  ],
  Array [
    "extra",
    "gc test\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "assert gc does not exist\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# gc test when the gc isn't there\\n",
  ],
  Array [
    "extra",
    "gc test\\n",
  ],
  Array [
    "extra",
    "t.plan=2\\n",
  ],
  Array [
    "extra",
    "gc test using --gc\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "assert gc exists\\n",
  ],
  Array [
    "extra",
    "gc test using --expose-gc\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "assert gc exists\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 35,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# gc test when the gc should be there\\n",
  ],
  Array [
    "comment",
    "# test for gc using --gc\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 36,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test for gc using --expose-gc\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 37,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# cleanup\\n",
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/expose-gc-test.js",
      "id": 38,
      "name": "test/expose-gc-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# global-harness-async.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# outer\\n",
  ],
  Array [
    "comment",
    "# inner 1\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "1-1",
      "id": 39,
      "name": "1-1",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# inner 2\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "2-1",
      "id": 40,
      "name": "2-1",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# inner 3\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "3-1",
      "id": 41,
      "name": "3-1",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/global-harness-async.js",
      "id": 42,
      "name": "test/global-harness-async.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# independent-timeouts.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# finishes in time\\n",
  ],
  Array [
    "comment",
    "# finishes in time too\\n",
  ],
  Array [
    "comment",
    "# tests 0\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/independent-timeouts.js",
      "id": 43,
      "name": "test/independent-timeouts.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# isolated-conf-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# one\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 44,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 45,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# two\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 46,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 47,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 4\\n",
  ],
  Array [
    "comment",
    "# pass  4\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/isolated-conf-test.js",
      "id": 48,
      "name": "test/isolated-conf-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# meta-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# meta test\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "sanity check",
      "id": 49,
      "name": "sanity check",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "not ok",
      "id": 50,
      "name": "not ok",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "total test count",
      "id": 51,
      "name": "total test count",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "tests passed",
      "id": 52,
      "name": "tests passed",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "tests failed",
      "id": 53,
      "name": "tests failed",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ok is boolean",
      "id": 54,
      "name": "ok is boolean",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip is number",
      "id": 55,
      "name": "skip is number",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "results isa Results",
      "id": 56,
      "name": "results isa Results",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test isa Test",
      "id": 57,
      "name": "test isa Test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test isa Harness",
      "id": 58,
      "name": "test isa Harness",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 10\\n",
  ],
  Array [
    "comment",
    "# pass  10\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/meta-test.js",
      "id": 59,
      "name": "test/meta-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# nested-async.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# Harness async test support\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "sync child A",
      "id": 60,
      "name": "sync child A",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# sync child B\\n",
  ],
  Array [
    "comment",
    "# async grandchild A\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "(unnamed assert)",
      "id": 61,
      "name": "(unnamed assert)",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# async grandchild B\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "(unnamed assert)",
      "id": 62,
      "name": "(unnamed assert)",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# async child\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "sync grandchild in async child A",
      "id": 63,
      "name": "sync grandchild in async child A",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# sync grandchild in async child B\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "(unnamed assert)",
      "id": 64,
      "name": "(unnamed assert)",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 5\\n",
  ],
  Array [
    "comment",
    "# pass  5\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/nested-async.js",
      "id": 65,
      "name": "test/nested-async.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# nested-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# parent\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "p test",
      "id": 66,
      "name": "p test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# subtest\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ch test",
      "id": 67,
      "name": "ch test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# nested subtest\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "grch test",
      "id": 68,
      "name": "grch test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# another subtest\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ch test 2",
      "id": 69,
      "name": "ch test 2",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 4\\n",
  ],
  Array [
    "comment",
    "# pass  4\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/nested-test.js",
      "id": 70,
      "name": "test/nested-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# non-tap-output.js\\n",
  ],
  Array [
    "comment",
    "# everything is fine\\n",
  ],
  Array [
    "comment",
    "# there are no errors\\n",
  ],
  Array [
    "comment",
    "# this output is not haiku.\\n",
  ],
  Array [
    "comment",
    "# is 8 ok?\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": ", 8 can stay.",
      "id": 71,
      "name": ", 8 can stay.",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# but: nevertheless, here we are\\n",
  ],
  Array [
    "comment",
    "# this: is indented\\n",
  ],
  Array [
    "comment",
    "# and: it\\n",
  ],
  Array [
    "comment",
    "# might: ~\\n",
  ],
  Array [
    "comment",
    "# be: yaml?\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "might be confusing",
      "id": 72,
      "name": "might be confusing",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "done now, exiting",
      "id": 73,
      "name": "done now, exiting",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/non-tap-output.js",
      "id": 74,
      "name": "test/non-tap-output.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# not-executed.sh\\n",
  ],
  Array [
    "comment",
    "# output-childtest-description.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# nested tests, parent and child pass\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs parent description",
      "id": 75,
      "name": "outputs parent description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs child description",
      "id": 76,
      "name": "outputs child description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs parent description before parent result",
      "id": 77,
      "name": "outputs parent description before parent result",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs parent result before child description",
      "id": 78,
      "name": "outputs parent result before child description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs child description before child result",
      "id": 79,
      "name": "outputs child description before child result",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 5\\n",
  ],
  Array [
    "comment",
    "# pass  5\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/output-childtest-description.js",
      "id": 80,
      "name": "test/output-childtest-description.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# result-trap.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# trap result\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 81,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 82,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 2\\n",
  ],
  Array [
    "comment",
    "# pass  2\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/result-trap.js",
      "id": 83,
      "name": "test/result-trap.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# segv.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# setup\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "compiled seg faulter",
      "id": 84,
      "name": "compiled seg faulter",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# segv\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 85,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 86,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 87,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 88,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 89,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 90,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# cleanup\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "cleaned up",
      "id": 91,
      "name": "cleaned up",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 8\\n",
  ],
  Array [
    "comment",
    "# pass  8\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/segv.js",
      "id": 92,
      "name": "test/segv.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# simple-harness-test-with-plan.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# trivial success\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "it works",
      "id": 93,
      "name": "it works",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# two tests\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "math should work",
      "id": 94,
      "name": "math should work",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false should not be ok",
      "id": 95,
      "name": "false should not be ok",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/simple-harness-test-with-plan.js",
      "id": 96,
      "name": "test/simple-harness-test-with-plan.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# simple-harness-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# trivial success\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "it works",
      "id": 97,
      "name": "it works",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# two tests\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "math should work",
      "id": 98,
      "name": "math should work",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false should not be ok",
      "id": 99,
      "name": "false should not be ok",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/simple-harness-test.js",
      "id": 100,
      "name": "test/simple-harness-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-assert-todo-skip.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# not much\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "always passes",
      "id": 101,
      "name": "always passes",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": "skip it good",
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false",
      "id": 102,
      "name": "false",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": "always fails",
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "bonus",
      "id": 103,
      "name": "bonus",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": "remove todo directive",
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "expected",
      "id": 104,
      "name": "expected",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": "implement a thing",
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "always passes without explanation",
      "id": 105,
      "name": "always passes without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": true,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false without explanation",
      "id": 106,
      "name": "false without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": true,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "bonus without explanation",
      "id": 107,
      "name": "bonus without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": true,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "expected without explanation",
      "id": 108,
      "name": "expected without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": true,
    },
  ],
  Array [
    "comment",
    "# tests 8\\n",
  ],
  Array [
    "comment",
    "# skip  4\\n",
  ],
  Array [
    "comment",
    "# todo  4\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/test-assert-todo-skip.js",
      "id": 109,
      "name": "test/test-assert-todo-skip.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-descriptions.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# captures test descriptions\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "exit cleanly",
      "id": 110,
      "name": "exit cleanly",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures SKIP description",
      "id": 111,
      "name": "captures SKIP description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary is not from file",
      "id": 112,
      "name": "skip summary is not from file",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not from file",
      "id": 113,
      "name": "todo summary is not from file",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "column": 7,
        "file": "child_process.js",
        "line": 707,
        "stack": Array [
          "getCaller (/some/path/tap-assert.js:439:17)\\n",
          "Function.assert (/some/path/tap-assert.js:21:16)\\n",
          "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
          "/some/path/test/test-descriptions.js:32:7\\n",
          "ChildProcess.exithandler (child_process.js:707:7)\\n",
          "emitTwo (events.js:87:13)\\n",
          "ChildProcess.emit (events.js:169:7)\\n",
          "maybeClose (child_process.js:984:16)\\n",
          "Socket.<anonymous> (child_process.js:1153:11)\\n",
          "emitOne (events.js:77:13)\\n",
        ],
      },
      "fullname": "captures TODO description",
      "id": 114,
      "name": "captures TODO description",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "exit cleanly",
      "id": 115,
      "name": "exit cleanly",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures SKIP description",
      "id": 116,
      "name": "captures SKIP description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary is not in TAP output",
      "id": 117,
      "name": "skip summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not in TAP output",
      "id": 118,
      "name": "todo summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "column": 7,
        "file": "child_process.js",
        "line": 707,
        "stack": Array [
          "getCaller (/some/path/tap-assert.js:439:17)\\n",
          "Function.assert (/some/path/tap-assert.js:21:16)\\n",
          "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
          "/some/path/test/test-descriptions.js:18:7\\n",
          "ChildProcess.exithandler (child_process.js:707:7)\\n",
          "emitTwo (events.js:87:13)\\n",
          "ChildProcess.emit (events.js:169:7)\\n",
          "maybeClose (child_process.js:984:16)\\n",
          "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
        ],
      },
      "fullname": "captures TODO description",
      "id": 119,
      "name": "captures TODO description",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "exit cleanly",
      "id": 120,
      "name": "exit cleanly",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "column": 7,
        "file": "child_process.js",
        "line": 707,
        "stack": Array [
          "getCaller (/some/path/tap-assert.js:439:17)\\n",
          "Function.assert (/some/path/tap-assert.js:21:16)\\n",
          "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
          "/some/path/test/test-descriptions.js:23:7\\n",
          "ChildProcess.exithandler (child_process.js:707:7)\\n",
          "emitTwo (events.js:87:13)\\n",
          "ChildProcess.emit (events.js:169:7)\\n",
          "maybeClose (child_process.js:984:16)\\n",
          "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
        ],
      },
      "fullname": "summarizes skipped count",
      "id": 121,
      "name": "summarizes skipped count",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "column": 7,
        "file": "child_process.js",
        "line": 707,
        "stack": Array [
          "getCaller (/some/path/tap-assert.js:439:17)\\n",
          "Function.assert (/some/path/tap-assert.js:21:16)\\n",
          "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
          "/some/path/test/test-descriptions.js:24:7\\n",
          "ChildProcess.exithandler (child_process.js:707:7)\\n",
          "emitTwo (events.js:87:13)\\n",
          "ChildProcess.emit (events.js:169:7)\\n",
          "maybeClose (child_process.js:984:16)\\n",
          "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
        ],
      },
      "fullname": "summarizes todo count",
      "id": 122,
      "name": "summarizes todo count",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 13\\n",
  ],
  Array [
    "comment",
    "# pass  9\\n",
  ],
  Array [
    "comment",
    "# fail  4\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "command": "/usr/local/bin/iojs test-descriptions.js",
        "exit": 1,
      },
      "fullname": "test/test-descriptions.js",
      "id": 123,
      "name": "test/test-descriptions.js",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-directives.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# captures test descriptions\\n",
  ],
  Array [
    "comment",
    "# raw TAP > TAP consumer > TAP producer\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "overall result is PASS",
      "id": 124,
      "name": "overall result is PASS",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures ok SKIP",
      "id": 125,
      "name": "captures ok SKIP",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures not ok SKIP",
      "id": 126,
      "name": "captures not ok SKIP",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary not in TAP output",
      "id": 127,
      "name": "skip summary not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures ok TODO",
      "id": 128,
      "name": "captures ok TODO",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures not ok TODO",
      "id": 129,
      "name": "captures not ok TODO",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not in TAP output",
      "id": 130,
      "name": "todo summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "no ugly \\"undefined\\" in output",
      "id": 131,
      "name": "no ugly \\"undefined\\" in output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# raw TAP > TAP consumer > summary\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "overall result is PASS",
      "id": 132,
      "name": "overall result is PASS",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "no SKIP in summary",
      "id": 133,
      "name": "no SKIP in summary",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary is not in TAP output",
      "id": 134,
      "name": "skip summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "no TODO in summary",
      "id": 135,
      "name": "no TODO in summary",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not in TAP output",
      "id": 136,
      "name": "todo summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "no ugly \\"undefined\\" in output",
      "id": 137,
      "name": "no ugly \\"undefined\\" in output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# TAP producer via require(\\"tap\\")\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "overall result is PASS",
      "id": 138,
      "name": "overall result is PASS",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures ok SKIP",
      "id": 139,
      "name": "captures ok SKIP",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures not ok SKIP",
      "id": 140,
      "name": "captures not ok SKIP",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary not in TAP output",
      "id": 141,
      "name": "skip summary not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures ok TODO",
      "id": 142,
      "name": "captures ok TODO",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures not ok TODO",
      "id": 143,
      "name": "captures not ok TODO",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not in TAP output",
      "id": 144,
      "name": "todo summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "no ugly \\"undefined\\" in output",
      "id": 145,
      "name": "no ugly \\"undefined\\" in output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 22\\n",
  ],
  Array [
    "comment",
    "# pass  22\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/test-directives.js",
      "id": 146,
      "name": "test/test-directives.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-skip.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "does not count as failure",
      "id": 147,
      "name": "does not count as failure",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": true,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 1\\n",
  ],
  Array [
    "comment",
    "# skip  1\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/test-skip.js",
      "id": 148,
      "name": "test/test-skip.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# testing the test object\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test object should be instanceof Test",
      "id": 149,
      "name": "test object should be instanceof Test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test object should be instanceof Harness",
      "id": 150,
      "name": "test object should be instanceof Harness",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test._Test should be the Test class",
      "id": 151,
      "name": "test._Test should be the Test class",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotDeepEqual method",
      "id": 152,
      "name": "should have isNotDeepEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotDeepEqual method should be a function",
      "id": 153,
      "name": "isNotDeepEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have equals method",
      "id": 154,
      "name": "should have equals method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "equals method should be a function",
      "id": 155,
      "name": "equals method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have inequivalent method",
      "id": 156,
      "name": "should have inequivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "inequivalent method should be a function",
      "id": 157,
      "name": "inequivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have threw method",
      "id": 158,
      "name": "should have threw method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "threw method should be a function",
      "id": 159,
      "name": "threw method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have strictEqual method",
      "id": 160,
      "name": "should have strictEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "strictEqual method should be a function",
      "id": 161,
      "name": "strictEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have emit method",
      "id": 162,
      "name": "should have emit method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "emit method should be a function",
      "id": 163,
      "name": "emit method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have fail method",
      "id": 164,
      "name": "should have fail method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "fail method should be a function",
      "id": 165,
      "name": "fail method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have strictEquals method",
      "id": 166,
      "name": "should have strictEquals method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "strictEquals method should be a function",
      "id": 167,
      "name": "strictEquals method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notLike method",
      "id": 168,
      "name": "should have notLike method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notLike method should be a function",
      "id": 169,
      "name": "notLike method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have dissimilar method",
      "id": 170,
      "name": "should have dissimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "dissimilar method should be a function",
      "id": 171,
      "name": "dissimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have true method",
      "id": 172,
      "name": "should have true method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "true method should be a function",
      "id": 173,
      "name": "true method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have assert method",
      "id": 174,
      "name": "should have assert method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "assert method should be a function",
      "id": 175,
      "name": "assert method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have is method",
      "id": 176,
      "name": "should have is method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "is method should be a function",
      "id": 177,
      "name": "is method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have ok method",
      "id": 178,
      "name": "should have ok method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ok method should be a function",
      "id": 179,
      "name": "ok method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isEqual method",
      "id": 180,
      "name": "should have isEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isEqual method should be a function",
      "id": 181,
      "name": "isEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isDeeply method",
      "id": 182,
      "name": "should have isDeeply method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isDeeply method should be a function",
      "id": 183,
      "name": "isDeeply method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have deepEqual method",
      "id": 184,
      "name": "should have deepEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "deepEqual method should be a function",
      "id": 185,
      "name": "deepEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have deepEquals method",
      "id": 186,
      "name": "should have deepEquals method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "deepEquals method should be a function",
      "id": 187,
      "name": "deepEquals method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have pass method",
      "id": 188,
      "name": "should have pass method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "pass method should be a function",
      "id": 189,
      "name": "pass method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have length method",
      "id": 190,
      "name": "should have length method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "length method should be a function",
      "id": 191,
      "name": "length method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have skip method",
      "id": 192,
      "name": "should have skip method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip method should be a function",
      "id": 193,
      "name": "skip method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotEqual method",
      "id": 194,
      "name": "should have isNotEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotEqual method should be a function",
      "id": 195,
      "name": "isNotEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have looseEquals method",
      "id": 196,
      "name": "should have looseEquals method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "looseEquals method should be a function",
      "id": 197,
      "name": "looseEquals method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have false method",
      "id": 198,
      "name": "should have false method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false method should be a function",
      "id": 199,
      "name": "false method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notDeeply method",
      "id": 200,
      "name": "should have notDeeply method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notDeeply method should be a function",
      "id": 201,
      "name": "notDeeply method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have ifErr method",
      "id": 202,
      "name": "should have ifErr method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ifErr method should be a function",
      "id": 203,
      "name": "ifErr method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have hasFields method",
      "id": 204,
      "name": "should have hasFields method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "hasFields method should be a function",
      "id": 205,
      "name": "hasFields method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotDeeply method",
      "id": 206,
      "name": "should have isNotDeeply method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotDeeply method should be a function",
      "id": 207,
      "name": "isNotDeeply method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have like method",
      "id": 208,
      "name": "should have like method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "like method should be a function",
      "id": 209,
      "name": "like method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have similar method",
      "id": 210,
      "name": "should have similar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "similar method should be a function",
      "id": 211,
      "name": "similar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notOk method",
      "id": 212,
      "name": "should have notOk method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notOk method should be a function",
      "id": 213,
      "name": "notOk method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isDissimilar method",
      "id": 214,
      "name": "should have isDissimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isDissimilar method should be a function",
      "id": 215,
      "name": "isDissimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isEquivalent method",
      "id": 216,
      "name": "should have isEquivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isEquivalent method should be a function",
      "id": 217,
      "name": "isEquivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have doesNotEqual method",
      "id": 218,
      "name": "should have doesNotEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "doesNotEqual method should be a function",
      "id": 219,
      "name": "doesNotEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isSimilar method",
      "id": 220,
      "name": "should have isSimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isSimilar method should be a function",
      "id": 221,
      "name": "isSimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notDeepEqual method",
      "id": 222,
      "name": "should have notDeepEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notDeepEqual method should be a function",
      "id": 223,
      "name": "notDeepEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have type method",
      "id": 224,
      "name": "should have type method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "type method should be a function",
      "id": 225,
      "name": "type method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notok method",
      "id": 226,
      "name": "should have notok method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notok method should be a function",
      "id": 227,
      "name": "notok method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isInequivalent method",
      "id": 228,
      "name": "should have isInequivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isInequivalent method should be a function",
      "id": 229,
      "name": "isInequivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNot method",
      "id": 230,
      "name": "should have isNot method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNot method should be a function",
      "id": 231,
      "name": "isNot method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have same method",
      "id": 232,
      "name": "should have same method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "same method should be a function",
      "id": 233,
      "name": "same method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isInequal method",
      "id": 234,
      "name": "should have isInequal method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isInequal method should be a function",
      "id": 235,
      "name": "isInequal method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have _endNice method",
      "id": 236,
      "name": "should have _endNice method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "_endNice method should be a function",
      "id": 237,
      "name": "_endNice method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have ifError method",
      "id": 238,
      "name": "should have ifError method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ifError method should be a function",
      "id": 239,
      "name": "ifError method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have iferror method",
      "id": 240,
      "name": "should have iferror method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "iferror method should be a function",
      "id": 241,
      "name": "iferror method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have clear method",
      "id": 242,
      "name": "should have clear method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "clear method should be a function",
      "id": 243,
      "name": "clear method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have has method",
      "id": 244,
      "name": "should have has method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "has method should be a function",
      "id": 245,
      "name": "has method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have not method",
      "id": 246,
      "name": "should have not method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "not method should be a function",
      "id": 247,
      "name": "not method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have timeout method",
      "id": 248,
      "name": "should have timeout method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "timeout method should be a function",
      "id": 249,
      "name": "timeout method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notSimilar method",
      "id": 250,
      "name": "should have notSimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notSimilar method should be a function",
      "id": 251,
      "name": "notSimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isUnlike method",
      "id": 252,
      "name": "should have isUnlike method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isUnlike method should be a function",
      "id": 253,
      "name": "isUnlike method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notEquals method",
      "id": 254,
      "name": "should have notEquals method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notEquals method should be a function",
      "id": 255,
      "name": "notEquals method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have unsimilar method",
      "id": 256,
      "name": "should have unsimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "unsimilar method should be a function",
      "id": 257,
      "name": "unsimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have result method",
      "id": 258,
      "name": "should have result method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "result method should be a function",
      "id": 259,
      "name": "result method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have doesNotThrow method",
      "id": 260,
      "name": "should have doesNotThrow method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "doesNotThrow method should be a function",
      "id": 261,
      "name": "doesNotThrow method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have error method",
      "id": 262,
      "name": "should have error method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "error method should be a function",
      "id": 263,
      "name": "error method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have constructor method",
      "id": 264,
      "name": "should have constructor method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "constructor method should be a function",
      "id": 265,
      "name": "constructor method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notEqual method",
      "id": 266,
      "name": "should have notEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notEqual method should be a function",
      "id": 267,
      "name": "notEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have throws method",
      "id": 268,
      "name": "should have throws method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "throws method should be a function",
      "id": 269,
      "name": "throws method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isLike method",
      "id": 270,
      "name": "should have isLike method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isLike method should be a function",
      "id": 271,
      "name": "isLike method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotSimilar method",
      "id": 272,
      "name": "should have isNotSimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotSimilar method should be a function",
      "id": 273,
      "name": "isNotSimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotEquivalent method",
      "id": 274,
      "name": "should have isNotEquivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotEquivalent method should be a function",
      "id": 275,
      "name": "isNotEquivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have inequal method",
      "id": 276,
      "name": "should have inequal method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "inequal method should be a function",
      "id": 277,
      "name": "inequal method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notEquivalent method",
      "id": 278,
      "name": "should have notEquivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notEquivalent method should be a function",
      "id": 279,
      "name": "notEquivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotLike method",
      "id": 280,
      "name": "should have isNotLike method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotLike method should be a function",
      "id": 281,
      "name": "isNotLike method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have equivalent method",
      "id": 282,
      "name": "should have equivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "equivalent method should be a function",
      "id": 283,
      "name": "equivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have looseEqual method",
      "id": 284,
      "name": "should have looseEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "looseEqual method should be a function",
      "id": 285,
      "name": "looseEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have equal method",
      "id": 286,
      "name": "should have equal method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "equal method should be a function",
      "id": 287,
      "name": "equal method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have unlike method",
      "id": 288,
      "name": "should have unlike method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "unlike method should be a function",
      "id": 289,
      "name": "unlike method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have doesNotHave method",
      "id": 290,
      "name": "should have doesNotHave method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "doesNotHave method should be a function",
      "id": 291,
      "name": "doesNotHave method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have comment method",
      "id": 292,
      "name": "should have comment method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "comment method should be a function",
      "id": 293,
      "name": "comment method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isa method",
      "id": 294,
      "name": "should have isa method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isa method should be a function",
      "id": 295,
      "name": "isa method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 147\\n",
  ],
  Array [
    "comment",
    "# pass  147\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/test-test.js",
      "id": 296,
      "name": "test/test-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# timeout.js\\n",
  ],
  Array [
    "extra",
    "timeout test\\n",
  ],
  Array [
    "extra",
    "t.plan=2\\n",
  ],
  Array [
    "extra",
    "a assert\\n",
  ],
  Array [
    "extra",
    "b assert\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# timeout test with plan only\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "a",
      "id": 297,
      "name": "a",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "extra",
    "timeout test\\n",
  ],
  Array [
    "extra",
    "t.plan=2\\n",
  ],
  Array [
    "extra",
    "a assert\\n",
  ],
  Array [
    "extra",
    "b assert\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "b",
      "id": 298,
      "name": "b",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# timeout test with plan and end\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "a",
      "id": 299,
      "name": "a",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "b",
      "id": 300,
      "name": "b",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 4\\n",
  ],
  Array [
    "comment",
    "# pass  4\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/timeout.js",
      "id": 301,
      "name": "test/timeout.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# trivial-success.js\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/trivial-success.js",
      "id": 302,
      "name": "test/trivial-success.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# undefined_indented.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# consume yaml\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 303,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 1\\n",
  ],
  Array [
    "comment",
    "# pass  1\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/undefined_indented.js",
      "id": 304,
      "name": "test/undefined_indented.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# valid-command.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# valid command\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 305,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 306,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 307,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 308,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 309,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 310,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 6\\n",
  ],
  Array [
    "comment",
    "# pass  6\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/valid-command.js",
      "id": 311,
      "name": "test/valid-command.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 311\\n",
  ],
  Array [
    "comment",
    "# pass  297\\n",
  ],
  Array [
    "comment",
    "# fail  5\\n",
  ],
  Array [
    "comment",
    "# skip  5\\n",
  ],
  Array [
    "comment",
    "# todo  4\\n",
  ],
  Array [
    "plan",
    Object {
      "end": 311,
      "start": 1,
    },
  ],
  Array [
    "complete",
    FinalResults {
      "bailout": false,
      "count": 311,
      "fail": 5,
      "failures": Array [
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "column": 7,
            "file": "child_process.js",
            "line": 707,
            "stack": Array [
              "getCaller (/some/path/tap-assert.js:439:17)\\n",
              "Function.assert (/some/path/tap-assert.js:21:16)\\n",
              "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
              "/some/path/test/test-descriptions.js:32:7\\n",
              "ChildProcess.exithandler (child_process.js:707:7)\\n",
              "emitTwo (events.js:87:13)\\n",
              "ChildProcess.emit (events.js:169:7)\\n",
              "maybeClose (child_process.js:984:16)\\n",
              "Socket.<anonymous> (child_process.js:1153:11)\\n",
              "emitOne (events.js:77:13)\\n",
            ],
          },
          "fullname": "captures TODO description",
          "id": 114,
          "name": "captures TODO description",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "column": 7,
            "file": "child_process.js",
            "line": 707,
            "stack": Array [
              "getCaller (/some/path/tap-assert.js:439:17)\\n",
              "Function.assert (/some/path/tap-assert.js:21:16)\\n",
              "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
              "/some/path/test/test-descriptions.js:18:7\\n",
              "ChildProcess.exithandler (child_process.js:707:7)\\n",
              "emitTwo (events.js:87:13)\\n",
              "ChildProcess.emit (events.js:169:7)\\n",
              "maybeClose (child_process.js:984:16)\\n",
              "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
            ],
          },
          "fullname": "captures TODO description",
          "id": 119,
          "name": "captures TODO description",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "column": 7,
            "file": "child_process.js",
            "line": 707,
            "stack": Array [
              "getCaller (/some/path/tap-assert.js:439:17)\\n",
              "Function.assert (/some/path/tap-assert.js:21:16)\\n",
              "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
              "/some/path/test/test-descriptions.js:23:7\\n",
              "ChildProcess.exithandler (child_process.js:707:7)\\n",
              "emitTwo (events.js:87:13)\\n",
              "ChildProcess.emit (events.js:169:7)\\n",
              "maybeClose (child_process.js:984:16)\\n",
              "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
            ],
          },
          "fullname": "summarizes skipped count",
          "id": 121,
          "name": "summarizes skipped count",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "column": 7,
            "file": "child_process.js",
            "line": 707,
            "stack": Array [
              "getCaller (/some/path/tap-assert.js:439:17)\\n",
              "Function.assert (/some/path/tap-assert.js:21:16)\\n",
              "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
              "/some/path/test/test-descriptions.js:24:7\\n",
              "ChildProcess.exithandler (child_process.js:707:7)\\n",
              "emitTwo (events.js:87:13)\\n",
              "ChildProcess.emit (events.js:169:7)\\n",
              "maybeClose (child_process.js:984:16)\\n",
              "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
            ],
          },
          "fullname": "summarizes todo count",
          "id": 122,
          "name": "summarizes todo count",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "command": "/usr/local/bin/iojs test-descriptions.js",
            "exit": 1,
          },
          "fullname": "test/test-descriptions.js",
          "id": 123,
          "name": "test/test-descriptions.js",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
      ],
      "ok": false,
      "pass": 306,
      "passes": undefined,
      "plan": FinalPlan {
        "comment": "",
        "end": 311,
        "skipAll": false,
        "skipReason": "",
        "start": 1,
      },
      "skip": 5,
      "skips": Array [
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "always passes",
          "id": 101,
          "name": "always passes",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": "skip it good",
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "false",
          "id": 102,
          "name": "false",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": "always fails",
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "always passes without explanation",
          "id": 105,
          "name": "always passes without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": true,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "false without explanation",
          "id": 106,
          "name": "false without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": true,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "does not count as failure",
          "id": 147,
          "name": "does not count as failure",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": true,
          "tapError": null,
          "time": null,
          "todo": false,
        },
      ],
      "time": null,
      "todo": 4,
      "todos": Array [
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "bonus",
          "id": 103,
          "name": "bonus",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": "remove todo directive",
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "expected",
          "id": 104,
          "name": "expected",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": "implement a thing",
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "bonus without explanation",
          "id": 107,
          "name": "bonus without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": true,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "expected without explanation",
          "id": 108,
          "name": "expected without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": true,
        },
      ],
    },
  ],
  Array [
    "finish",
  ],
  Array [
    "close",
  ],
]
`

exports[`test/parse-stringify.ts > TAP > tap-tests.tap > default settings > stringified 1`] = `
TAP version 13
# buffer_compare.js
# TAP version 13
# same buffers
ok 1 - should be equivalent
# not same buffers
ok 2 - should not be equivalent
# tests 2
# pass  2
# ok
ok 3 - test/buffer_compare.js
# common.js
ok 4 - just setup, nothing relevant
ok 5 - test/common.js
# consumer.js
# TAP version 13
# basic.tap
ok 6 - should be equivalent
# indent.tap
ok 7 - should be equivalent
# missing.tap
ok 8 - should be equivalent
# skip-all.tap
ok 9 - should be equivalent
# yamlish.tap
ok 10 - should be equivalent
# tests 5
# pass  5
# ok
ok 11 - test/consumer.js
# debug-test.js
debug test
t.plan=1
'Debugger listening on port 5858\\n'
# TAP version 13
# debug test
ok 12 - Should output debugger message
# tests 1
# pass  1
# ok
ok 13 - test/debug-test.js
# deep-strict.js
# TAP version 13
# strictDeepEquals shouldn't care about key order
ok 14 - should be strictly equal
# strictDeepEquals shouldn't care about key order recursively
ok 15 - should be strictly equal
# strictDeepEquals shoudn't care about key order (but still might)
ok 16 - should be strictly equal
# tests 3
# pass  3
# ok
ok 17 - test/deep-strict.js
# deep.js
# TAP version 13
# deepEquals shouldn't care about key order and types
ok 18 - should be equivalent
# deepEquals shouldn't care about key order recursively and types
ok 19 - should be equivalent
# deepEquals shoudn't care about key order (but still might) and types
ok 20 - should be equivalent
# tests 3
# pass  3
# ok
ok 21 - test/deep.js
# executed.sh
ok 22 - File with executable bit should be executed
ok 23 - test/executed.sh
# exit-code.js
# TAP version 13
# exit code 1 when tap results show failure
# test exits 0, has failures
ok 24 - should be equal
ok 25 - should be equal
# test exits 1, has failures
ok 26 - should be equal
ok 27 - should be equal
# test exits 1, has no failures
ok 28 - should be equal
ok 29 - should be equal
# successes exit 0
# test that does nothing, but exits 0
ok 30 - should be equal
ok 31 - should be equal
# test that succeeds, and exits 0
ok 32 - should be equal
ok 33 - should be equal
# tests 10
# pass  10
# ok
ok 34 - test/exit-code.js
# expose-gc-test.js
gc test
t.plan=1
assert gc does not exist
# TAP version 13
# gc test when the gc isn't there
gc test
t.plan=2
gc test using --gc
t.plan=1
assert gc exists
gc test using --expose-gc
t.plan=1
assert gc exists
ok 35 - should be equal
# gc test when the gc should be there
# test for gc using --gc
ok 36 - should be equal
# test for gc using --expose-gc
ok 37 - should be equal
# cleanup
# tests 3
# pass  3
# ok
ok 38 - test/expose-gc-test.js
# global-harness-async.js
# TAP version 13
# outer
# inner 1
ok 39 - 1-1
# inner 2
ok 40 - 2-1
# inner 3
ok 41 - 3-1
ok 42 - test/global-harness-async.js
# independent-timeouts.js
# TAP version 13
# finishes in time
# finishes in time too
# tests 0
# ok
ok 43 - test/independent-timeouts.js
# isolated-conf-test.js
# TAP version 13
# one
ok 44 - should be equal
ok 45 - should be equal
# two
ok 46 - should be equal
ok 47 - should be equal
# tests 4
# pass  4
# ok
ok 48 - test/isolated-conf-test.js
# meta-test.js
# TAP version 13
# meta test
ok 49 - sanity check
ok 50 - not ok
ok 51 - total test count
ok 52 - tests passed
ok 53 - tests failed
ok 54 - ok is boolean
ok 55 - skip is number
ok 56 - results isa Results
ok 57 - test isa Test
ok 58 - test isa Harness
# tests 10
# pass  10
# ok
ok 59 - test/meta-test.js
# nested-async.js
# TAP version 13
# Harness async test support
ok 60 - sync child A
# sync child B
# async grandchild A
ok 61 - (unnamed assert)
# async grandchild B
ok 62 - (unnamed assert)
# async child
ok 63 - sync grandchild in async child A
# sync grandchild in async child B
ok 64 - (unnamed assert)
# tests 5
# pass  5
# ok
ok 65 - test/nested-async.js
# nested-test.js
# TAP version 13
# parent
ok 66 - p test
# subtest
ok 67 - ch test
# nested subtest
ok 68 - grch test
# another subtest
ok 69 - ch test 2
# tests 4
# pass  4
# ok
ok 70 - test/nested-test.js
# non-tap-output.js
# everything is fine
# there are no errors
# this output is not haiku.
# is 8 ok?
ok 71 - , 8 can stay.
# but: nevertheless, here we are
# this: is indented
# and: it
# might: ~
# be: yaml?
ok 72 - might be confusing
ok 73 - done now, exiting
ok 74 - test/non-tap-output.js
# not-executed.sh
# output-childtest-description.js
# TAP version 13
# nested tests, parent and child pass
ok 75 - outputs parent description
ok 76 - outputs child description
ok 77 - outputs parent description before parent result
ok 78 - outputs parent result before child description
ok 79 - outputs child description before child result
# tests 5
# pass  5
# ok
ok 80 - test/output-childtest-description.js
# result-trap.js
# TAP version 13
# trap result
ok 81 - should be equal
ok 82 - should be equal
# tests 2
# pass  2
# ok
ok 83 - test/result-trap.js
# segv.js
# TAP version 13
# setup
ok 84 - compiled seg faulter
# segv
ok 85 - should be equivalent
ok 86 - should be equivalent
ok 87 - should be equivalent
ok 88 - should be equivalent
ok 89 - should be equivalent
ok 90 - should be equal
# cleanup
ok 91 - cleaned up
# tests 8
# pass  8
# ok
ok 92 - test/segv.js
# simple-harness-test-with-plan.js
# TAP version 13
# trivial success
ok 93 - it works
# two tests
ok 94 - math should work
ok 95 - false should not be ok
# tests 3
# pass  3
# ok
ok 96 - test/simple-harness-test-with-plan.js
# simple-harness-test.js
# TAP version 13
# trivial success
ok 97 - it works
# two tests
ok 98 - math should work
ok 99 - false should not be ok
# tests 3
# pass  3
# ok
ok 100 - test/simple-harness-test.js
# test-assert-todo-skip.js
# TAP version 13
# not much
ok 101 - always passes # SKIP skip it good
ok 102 - false # SKIP always fails
ok 103 - bonus # TODO remove todo directive
ok 104 - expected # TODO implement a thing
ok 105 - always passes without explanation # SKIP
ok 106 - false without explanation # SKIP
ok 107 - bonus without explanation # TODO
ok 108 - expected without explanation # TODO
# tests 8
# skip  4
# todo  4
ok 109 - test/test-assert-todo-skip.js
# test-descriptions.js
# TAP version 13
# captures test descriptions
ok 110 - exit cleanly
ok 111 - captures SKIP description
ok 112 - skip summary is not from file
ok 113 - todo summary is not from file
not ok 114 - captures TODO description
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:32:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Socket.<anonymous> (child_process.js:1153:11)
    - |
      emitOne (events.js:77:13)
  ...
ok 115 - exit cleanly
ok 116 - captures SKIP description
ok 117 - skip summary is not in TAP output
ok 118 - todo summary is not in TAP output
not ok 119 - captures TODO description
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:18:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Process.ChildProcess._handle.onexit (child_process.js:1057:5)
  ...
ok 120 - exit cleanly
not ok 121 - summarizes skipped count
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:23:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Process.ChildProcess._handle.onexit (child_process.js:1057:5)
  ...
not ok 122 - summarizes todo count
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:24:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Process.ChildProcess._handle.onexit (child_process.js:1057:5)
  ...
# tests 13
# pass  9
# fail  4
not ok 123 - test/test-descriptions.js
  ---
  exit: 1
  command: /usr/local/bin/iojs test-descriptions.js
  ...
# test-directives.js
# TAP version 13
# captures test descriptions
# raw TAP > TAP consumer > TAP producer
ok 124 - overall result is PASS
ok 125 - captures ok SKIP
ok 126 - captures not ok SKIP
ok 127 - skip summary not in TAP output
ok 128 - captures ok TODO
ok 129 - captures not ok TODO
ok 130 - todo summary is not in TAP output
ok 131 - no ugly "undefined" in output
# raw TAP > TAP consumer > summary
ok 132 - overall result is PASS
ok 133 - no SKIP in summary
ok 134 - skip summary is not in TAP output
ok 135 - no TODO in summary
ok 136 - todo summary is not in TAP output
ok 137 - no ugly "undefined" in output
# TAP producer via require("tap")
ok 138 - overall result is PASS
ok 139 - captures ok SKIP
ok 140 - captures not ok SKIP
ok 141 - skip summary not in TAP output
ok 142 - captures ok TODO
ok 143 - captures not ok TODO
ok 144 - todo summary is not in TAP output
ok 145 - no ugly "undefined" in output
# tests 22
# pass  22
# ok
ok 146 - test/test-directives.js
# test-skip.js
# TAP version 13
ok 147 - does not count as failure # SKIP
# tests 1
# skip  1
ok 148 - test/test-skip.js
# test-test.js
# TAP version 13
# testing the test object
ok 149 - test object should be instanceof Test
ok 150 - test object should be instanceof Harness
ok 151 - test._Test should be the Test class
ok 152 - should have isNotDeepEqual method
ok 153 - isNotDeepEqual method should be a function
ok 154 - should have equals method
ok 155 - equals method should be a function
ok 156 - should have inequivalent method
ok 157 - inequivalent method should be a function
ok 158 - should have threw method
ok 159 - threw method should be a function
ok 160 - should have strictEqual method
ok 161 - strictEqual method should be a function
ok 162 - should have emit method
ok 163 - emit method should be a function
ok 164 - should have fail method
ok 165 - fail method should be a function
ok 166 - should have strictEquals method
ok 167 - strictEquals method should be a function
ok 168 - should have notLike method
ok 169 - notLike method should be a function
ok 170 - should have dissimilar method
ok 171 - dissimilar method should be a function
ok 172 - should have true method
ok 173 - true method should be a function
ok 174 - should have assert method
ok 175 - assert method should be a function
ok 176 - should have is method
ok 177 - is method should be a function
ok 178 - should have ok method
ok 179 - ok method should be a function
ok 180 - should have isEqual method
ok 181 - isEqual method should be a function
ok 182 - should have isDeeply method
ok 183 - isDeeply method should be a function
ok 184 - should have deepEqual method
ok 185 - deepEqual method should be a function
ok 186 - should have deepEquals method
ok 187 - deepEquals method should be a function
ok 188 - should have pass method
ok 189 - pass method should be a function
ok 190 - should have length method
ok 191 - length method should be a function
ok 192 - should have skip method
ok 193 - skip method should be a function
ok 194 - should have isNotEqual method
ok 195 - isNotEqual method should be a function
ok 196 - should have looseEquals method
ok 197 - looseEquals method should be a function
ok 198 - should have false method
ok 199 - false method should be a function
ok 200 - should have notDeeply method
ok 201 - notDeeply method should be a function
ok 202 - should have ifErr method
ok 203 - ifErr method should be a function
ok 204 - should have hasFields method
ok 205 - hasFields method should be a function
ok 206 - should have isNotDeeply method
ok 207 - isNotDeeply method should be a function
ok 208 - should have like method
ok 209 - like method should be a function
ok 210 - should have similar method
ok 211 - similar method should be a function
ok 212 - should have notOk method
ok 213 - notOk method should be a function
ok 214 - should have isDissimilar method
ok 215 - isDissimilar method should be a function
ok 216 - should have isEquivalent method
ok 217 - isEquivalent method should be a function
ok 218 - should have doesNotEqual method
ok 219 - doesNotEqual method should be a function
ok 220 - should have isSimilar method
ok 221 - isSimilar method should be a function
ok 222 - should have notDeepEqual method
ok 223 - notDeepEqual method should be a function
ok 224 - should have type method
ok 225 - type method should be a function
ok 226 - should have notok method
ok 227 - notok method should be a function
ok 228 - should have isInequivalent method
ok 229 - isInequivalent method should be a function
ok 230 - should have isNot method
ok 231 - isNot method should be a function
ok 232 - should have same method
ok 233 - same method should be a function
ok 234 - should have isInequal method
ok 235 - isInequal method should be a function
ok 236 - should have _endNice method
ok 237 - _endNice method should be a function
ok 238 - should have ifError method
ok 239 - ifError method should be a function
ok 240 - should have iferror method
ok 241 - iferror method should be a function
ok 242 - should have clear method
ok 243 - clear method should be a function
ok 244 - should have has method
ok 245 - has method should be a function
ok 246 - should have not method
ok 247 - not method should be a function
ok 248 - should have timeout method
ok 249 - timeout method should be a function
ok 250 - should have notSimilar method
ok 251 - notSimilar method should be a function
ok 252 - should have isUnlike method
ok 253 - isUnlike method should be a function
ok 254 - should have notEquals method
ok 255 - notEquals method should be a function
ok 256 - should have unsimilar method
ok 257 - unsimilar method should be a function
ok 258 - should have result method
ok 259 - result method should be a function
ok 260 - should have doesNotThrow method
ok 261 - doesNotThrow method should be a function
ok 262 - should have error method
ok 263 - error method should be a function
ok 264 - should have constructor method
ok 265 - constructor method should be a function
ok 266 - should have notEqual method
ok 267 - notEqual method should be a function
ok 268 - should have throws method
ok 269 - throws method should be a function
ok 270 - should have isLike method
ok 271 - isLike method should be a function
ok 272 - should have isNotSimilar method
ok 273 - isNotSimilar method should be a function
ok 274 - should have isNotEquivalent method
ok 275 - isNotEquivalent method should be a function
ok 276 - should have inequal method
ok 277 - inequal method should be a function
ok 278 - should have notEquivalent method
ok 279 - notEquivalent method should be a function
ok 280 - should have isNotLike method
ok 281 - isNotLike method should be a function
ok 282 - should have equivalent method
ok 283 - equivalent method should be a function
ok 284 - should have looseEqual method
ok 285 - looseEqual method should be a function
ok 286 - should have equal method
ok 287 - equal method should be a function
ok 288 - should have unlike method
ok 289 - unlike method should be a function
ok 290 - should have doesNotHave method
ok 291 - doesNotHave method should be a function
ok 292 - should have comment method
ok 293 - comment method should be a function
ok 294 - should have isa method
ok 295 - isa method should be a function
# tests 147
# pass  147
# ok
ok 296 - test/test-test.js
# timeout.js
timeout test
t.plan=2
a assert
b assert
# TAP version 13
# timeout test with plan only
ok 297 - a
timeout test
t.plan=2
a assert
b assert
ok 298 - b
# timeout test with plan and end
ok 299 - a
ok 300 - b
# tests 4
# pass  4
# ok
ok 301 - test/timeout.js
# trivial-success.js
ok 302 - test/trivial-success.js
# undefined_indented.js
# TAP version 13
# consume yaml
ok 303 - should be equivalent
# tests 1
# pass  1
# ok
ok 304 - test/undefined_indented.js
# valid-command.js
# TAP version 13
# valid command
ok 305 - should be equivalent
ok 306 - should be equivalent
ok 307 - should be equivalent
ok 308 - should be equivalent
ok 309 - should be equivalent
ok 310 - should be equal
# tests 6
# pass  6
# ok
ok 311 - test/valid-command.js
1..311
# tests 311
# pass  297
# fail  5
# skip  5
# todo  4

`

exports[`test/parse-stringify.ts > TAP > tap-tests.tap > default settings > stringified flat 1`] = `
TAP version 13
# buffer_compare.js
# TAP version 13
# same buffers
ok 1 - should be equivalent
# not same buffers
ok 2 - should not be equivalent
# tests 2
# pass  2
# ok
ok 3 - test/buffer_compare.js
# common.js
ok 4 - just setup, nothing relevant
ok 5 - test/common.js
# consumer.js
# TAP version 13
# basic.tap
ok 6 - should be equivalent
# indent.tap
ok 7 - should be equivalent
# missing.tap
ok 8 - should be equivalent
# skip-all.tap
ok 9 - should be equivalent
# yamlish.tap
ok 10 - should be equivalent
# tests 5
# pass  5
# ok
ok 11 - test/consumer.js
# debug-test.js
debug test
t.plan=1
'Debugger listening on port 5858\\n'
# TAP version 13
# debug test
ok 12 - Should output debugger message
# tests 1
# pass  1
# ok
ok 13 - test/debug-test.js
# deep-strict.js
# TAP version 13
# strictDeepEquals shouldn't care about key order
ok 14 - should be strictly equal
# strictDeepEquals shouldn't care about key order recursively
ok 15 - should be strictly equal
# strictDeepEquals shoudn't care about key order (but still might)
ok 16 - should be strictly equal
# tests 3
# pass  3
# ok
ok 17 - test/deep-strict.js
# deep.js
# TAP version 13
# deepEquals shouldn't care about key order and types
ok 18 - should be equivalent
# deepEquals shouldn't care about key order recursively and types
ok 19 - should be equivalent
# deepEquals shoudn't care about key order (but still might) and types
ok 20 - should be equivalent
# tests 3
# pass  3
# ok
ok 21 - test/deep.js
# executed.sh
ok 22 - File with executable bit should be executed
ok 23 - test/executed.sh
# exit-code.js
# TAP version 13
# exit code 1 when tap results show failure
# test exits 0, has failures
ok 24 - should be equal
ok 25 - should be equal
# test exits 1, has failures
ok 26 - should be equal
ok 27 - should be equal
# test exits 1, has no failures
ok 28 - should be equal
ok 29 - should be equal
# successes exit 0
# test that does nothing, but exits 0
ok 30 - should be equal
ok 31 - should be equal
# test that succeeds, and exits 0
ok 32 - should be equal
ok 33 - should be equal
# tests 10
# pass  10
# ok
ok 34 - test/exit-code.js
# expose-gc-test.js
gc test
t.plan=1
assert gc does not exist
# TAP version 13
# gc test when the gc isn't there
gc test
t.plan=2
gc test using --gc
t.plan=1
assert gc exists
gc test using --expose-gc
t.plan=1
assert gc exists
ok 35 - should be equal
# gc test when the gc should be there
# test for gc using --gc
ok 36 - should be equal
# test for gc using --expose-gc
ok 37 - should be equal
# cleanup
# tests 3
# pass  3
# ok
ok 38 - test/expose-gc-test.js
# global-harness-async.js
# TAP version 13
# outer
# inner 1
ok 39 - 1-1
# inner 2
ok 40 - 2-1
# inner 3
ok 41 - 3-1
ok 42 - test/global-harness-async.js
# independent-timeouts.js
# TAP version 13
# finishes in time
# finishes in time too
# tests 0
# ok
ok 43 - test/independent-timeouts.js
# isolated-conf-test.js
# TAP version 13
# one
ok 44 - should be equal
ok 45 - should be equal
# two
ok 46 - should be equal
ok 47 - should be equal
# tests 4
# pass  4
# ok
ok 48 - test/isolated-conf-test.js
# meta-test.js
# TAP version 13
# meta test
ok 49 - sanity check
ok 50 - not ok
ok 51 - total test count
ok 52 - tests passed
ok 53 - tests failed
ok 54 - ok is boolean
ok 55 - skip is number
ok 56 - results isa Results
ok 57 - test isa Test
ok 58 - test isa Harness
# tests 10
# pass  10
# ok
ok 59 - test/meta-test.js
# nested-async.js
# TAP version 13
# Harness async test support
ok 60 - sync child A
# sync child B
# async grandchild A
ok 61 - (unnamed assert)
# async grandchild B
ok 62 - (unnamed assert)
# async child
ok 63 - sync grandchild in async child A
# sync grandchild in async child B
ok 64 - (unnamed assert)
# tests 5
# pass  5
# ok
ok 65 - test/nested-async.js
# nested-test.js
# TAP version 13
# parent
ok 66 - p test
# subtest
ok 67 - ch test
# nested subtest
ok 68 - grch test
# another subtest
ok 69 - ch test 2
# tests 4
# pass  4
# ok
ok 70 - test/nested-test.js
# non-tap-output.js
# everything is fine
# there are no errors
# this output is not haiku.
# is 8 ok?
ok 71 - , 8 can stay.
# but: nevertheless, here we are
# this: is indented
# and: it
# might: ~
# be: yaml?
ok 72 - might be confusing
ok 73 - done now, exiting
ok 74 - test/non-tap-output.js
# not-executed.sh
# output-childtest-description.js
# TAP version 13
# nested tests, parent and child pass
ok 75 - outputs parent description
ok 76 - outputs child description
ok 77 - outputs parent description before parent result
ok 78 - outputs parent result before child description
ok 79 - outputs child description before child result
# tests 5
# pass  5
# ok
ok 80 - test/output-childtest-description.js
# result-trap.js
# TAP version 13
# trap result
ok 81 - should be equal
ok 82 - should be equal
# tests 2
# pass  2
# ok
ok 83 - test/result-trap.js
# segv.js
# TAP version 13
# setup
ok 84 - compiled seg faulter
# segv
ok 85 - should be equivalent
ok 86 - should be equivalent
ok 87 - should be equivalent
ok 88 - should be equivalent
ok 89 - should be equivalent
ok 90 - should be equal
# cleanup
ok 91 - cleaned up
# tests 8
# pass  8
# ok
ok 92 - test/segv.js
# simple-harness-test-with-plan.js
# TAP version 13
# trivial success
ok 93 - it works
# two tests
ok 94 - math should work
ok 95 - false should not be ok
# tests 3
# pass  3
# ok
ok 96 - test/simple-harness-test-with-plan.js
# simple-harness-test.js
# TAP version 13
# trivial success
ok 97 - it works
# two tests
ok 98 - math should work
ok 99 - false should not be ok
# tests 3
# pass  3
# ok
ok 100 - test/simple-harness-test.js
# test-assert-todo-skip.js
# TAP version 13
# not much
ok 101 - always passes # SKIP skip it good
ok 102 - false # SKIP always fails
ok 103 - bonus # TODO remove todo directive
ok 104 - expected # TODO implement a thing
ok 105 - always passes without explanation # SKIP
ok 106 - false without explanation # SKIP
ok 107 - bonus without explanation # TODO
ok 108 - expected without explanation # TODO
# tests 8
# skip  4
# todo  4
ok 109 - test/test-assert-todo-skip.js
# test-descriptions.js
# TAP version 13
# captures test descriptions
ok 110 - exit cleanly
ok 111 - captures SKIP description
ok 112 - skip summary is not from file
ok 113 - todo summary is not from file
not ok 114 - captures TODO description
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:32:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Socket.<anonymous> (child_process.js:1153:11)
    - |
      emitOne (events.js:77:13)
  ...
ok 115 - exit cleanly
ok 116 - captures SKIP description
ok 117 - skip summary is not in TAP output
ok 118 - todo summary is not in TAP output
not ok 119 - captures TODO description
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:18:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Process.ChildProcess._handle.onexit (child_process.js:1057:5)
  ...
ok 120 - exit cleanly
not ok 121 - summarizes skipped count
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:23:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Process.ChildProcess._handle.onexit (child_process.js:1057:5)
  ...
not ok 122 - summarizes todo count
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:24:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Process.ChildProcess._handle.onexit (child_process.js:1057:5)
  ...
# tests 13
# pass  9
# fail  4
not ok 123 - test/test-descriptions.js
  ---
  exit: 1
  command: /usr/local/bin/iojs test-descriptions.js
  ...
# test-directives.js
# TAP version 13
# captures test descriptions
# raw TAP > TAP consumer > TAP producer
ok 124 - overall result is PASS
ok 125 - captures ok SKIP
ok 126 - captures not ok SKIP
ok 127 - skip summary not in TAP output
ok 128 - captures ok TODO
ok 129 - captures not ok TODO
ok 130 - todo summary is not in TAP output
ok 131 - no ugly "undefined" in output
# raw TAP > TAP consumer > summary
ok 132 - overall result is PASS
ok 133 - no SKIP in summary
ok 134 - skip summary is not in TAP output
ok 135 - no TODO in summary
ok 136 - todo summary is not in TAP output
ok 137 - no ugly "undefined" in output
# TAP producer via require("tap")
ok 138 - overall result is PASS
ok 139 - captures ok SKIP
ok 140 - captures not ok SKIP
ok 141 - skip summary not in TAP output
ok 142 - captures ok TODO
ok 143 - captures not ok TODO
ok 144 - todo summary is not in TAP output
ok 145 - no ugly "undefined" in output
# tests 22
# pass  22
# ok
ok 146 - test/test-directives.js
# test-skip.js
# TAP version 13
ok 147 - does not count as failure # SKIP
# tests 1
# skip  1
ok 148 - test/test-skip.js
# test-test.js
# TAP version 13
# testing the test object
ok 149 - test object should be instanceof Test
ok 150 - test object should be instanceof Harness
ok 151 - test._Test should be the Test class
ok 152 - should have isNotDeepEqual method
ok 153 - isNotDeepEqual method should be a function
ok 154 - should have equals method
ok 155 - equals method should be a function
ok 156 - should have inequivalent method
ok 157 - inequivalent method should be a function
ok 158 - should have threw method
ok 159 - threw method should be a function
ok 160 - should have strictEqual method
ok 161 - strictEqual method should be a function
ok 162 - should have emit method
ok 163 - emit method should be a function
ok 164 - should have fail method
ok 165 - fail method should be a function
ok 166 - should have strictEquals method
ok 167 - strictEquals method should be a function
ok 168 - should have notLike method
ok 169 - notLike method should be a function
ok 170 - should have dissimilar method
ok 171 - dissimilar method should be a function
ok 172 - should have true method
ok 173 - true method should be a function
ok 174 - should have assert method
ok 175 - assert method should be a function
ok 176 - should have is method
ok 177 - is method should be a function
ok 178 - should have ok method
ok 179 - ok method should be a function
ok 180 - should have isEqual method
ok 181 - isEqual method should be a function
ok 182 - should have isDeeply method
ok 183 - isDeeply method should be a function
ok 184 - should have deepEqual method
ok 185 - deepEqual method should be a function
ok 186 - should have deepEquals method
ok 187 - deepEquals method should be a function
ok 188 - should have pass method
ok 189 - pass method should be a function
ok 190 - should have length method
ok 191 - length method should be a function
ok 192 - should have skip method
ok 193 - skip method should be a function
ok 194 - should have isNotEqual method
ok 195 - isNotEqual method should be a function
ok 196 - should have looseEquals method
ok 197 - looseEquals method should be a function
ok 198 - should have false method
ok 199 - false method should be a function
ok 200 - should have notDeeply method
ok 201 - notDeeply method should be a function
ok 202 - should have ifErr method
ok 203 - ifErr method should be a function
ok 204 - should have hasFields method
ok 205 - hasFields method should be a function
ok 206 - should have isNotDeeply method
ok 207 - isNotDeeply method should be a function
ok 208 - should have like method
ok 209 - like method should be a function
ok 210 - should have similar method
ok 211 - similar method should be a function
ok 212 - should have notOk method
ok 213 - notOk method should be a function
ok 214 - should have isDissimilar method
ok 215 - isDissimilar method should be a function
ok 216 - should have isEquivalent method
ok 217 - isEquivalent method should be a function
ok 218 - should have doesNotEqual method
ok 219 - doesNotEqual method should be a function
ok 220 - should have isSimilar method
ok 221 - isSimilar method should be a function
ok 222 - should have notDeepEqual method
ok 223 - notDeepEqual method should be a function
ok 224 - should have type method
ok 225 - type method should be a function
ok 226 - should have notok method
ok 227 - notok method should be a function
ok 228 - should have isInequivalent method
ok 229 - isInequivalent method should be a function
ok 230 - should have isNot method
ok 231 - isNot method should be a function
ok 232 - should have same method
ok 233 - same method should be a function
ok 234 - should have isInequal method
ok 235 - isInequal method should be a function
ok 236 - should have _endNice method
ok 237 - _endNice method should be a function
ok 238 - should have ifError method
ok 239 - ifError method should be a function
ok 240 - should have iferror method
ok 241 - iferror method should be a function
ok 242 - should have clear method
ok 243 - clear method should be a function
ok 244 - should have has method
ok 245 - has method should be a function
ok 246 - should have not method
ok 247 - not method should be a function
ok 248 - should have timeout method
ok 249 - timeout method should be a function
ok 250 - should have notSimilar method
ok 251 - notSimilar method should be a function
ok 252 - should have isUnlike method
ok 253 - isUnlike method should be a function
ok 254 - should have notEquals method
ok 255 - notEquals method should be a function
ok 256 - should have unsimilar method
ok 257 - unsimilar method should be a function
ok 258 - should have result method
ok 259 - result method should be a function
ok 260 - should have doesNotThrow method
ok 261 - doesNotThrow method should be a function
ok 262 - should have error method
ok 263 - error method should be a function
ok 264 - should have constructor method
ok 265 - constructor method should be a function
ok 266 - should have notEqual method
ok 267 - notEqual method should be a function
ok 268 - should have throws method
ok 269 - throws method should be a function
ok 270 - should have isLike method
ok 271 - isLike method should be a function
ok 272 - should have isNotSimilar method
ok 273 - isNotSimilar method should be a function
ok 274 - should have isNotEquivalent method
ok 275 - isNotEquivalent method should be a function
ok 276 - should have inequal method
ok 277 - inequal method should be a function
ok 278 - should have notEquivalent method
ok 279 - notEquivalent method should be a function
ok 280 - should have isNotLike method
ok 281 - isNotLike method should be a function
ok 282 - should have equivalent method
ok 283 - equivalent method should be a function
ok 284 - should have looseEqual method
ok 285 - looseEqual method should be a function
ok 286 - should have equal method
ok 287 - equal method should be a function
ok 288 - should have unlike method
ok 289 - unlike method should be a function
ok 290 - should have doesNotHave method
ok 291 - doesNotHave method should be a function
ok 292 - should have comment method
ok 293 - comment method should be a function
ok 294 - should have isa method
ok 295 - isa method should be a function
# tests 147
# pass  147
# ok
ok 296 - test/test-test.js
# timeout.js
timeout test
t.plan=2
a assert
b assert
# TAP version 13
# timeout test with plan only
ok 297 - a
timeout test
t.plan=2
a assert
b assert
ok 298 - b
# timeout test with plan and end
ok 299 - a
ok 300 - b
# tests 4
# pass  4
# ok
ok 301 - test/timeout.js
# trivial-success.js
ok 302 - test/trivial-success.js
# undefined_indented.js
# TAP version 13
# consume yaml
ok 303 - should be equivalent
# tests 1
# pass  1
# ok
ok 304 - test/undefined_indented.js
# valid-command.js
# TAP version 13
# valid command
ok 305 - should be equivalent
ok 306 - should be equivalent
ok 307 - should be equivalent
ok 308 - should be equivalent
ok 309 - should be equivalent
ok 310 - should be equal
# tests 6
# pass  6
# ok
ok 311 - test/valid-command.js
1..311
# tests 311
# pass  297
# fail  5
# skip  5
# todo  4

`

exports[`test/parse-stringify.ts > TAP > tap-tests.tap > strict > parsed 1`] = `
Array [
  Array [
    "version",
    13,
  ],
  Array [
    "comment",
    "# buffer_compare.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# same buffers\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 1,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# not same buffers\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should not be equivalent",
      "id": 2,
      "name": "should not be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 2\\n",
  ],
  Array [
    "comment",
    "# pass  2\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/buffer_compare.js",
      "id": 3,
      "name": "test/buffer_compare.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# common.js\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "just setup, nothing relevant",
      "id": 4,
      "name": "just setup, nothing relevant",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/common.js",
      "id": 5,
      "name": "test/common.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# consumer.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# basic.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 6,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# indent.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 7,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# missing.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 8,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# skip-all.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 9,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# yamlish.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 10,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 5\\n",
  ],
  Array [
    "comment",
    "# pass  5\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/consumer.js",
      "id": 11,
      "name": "test/consumer.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# debug-test.js\\n",
  ],
  Array [
    "extra",
    "debug test\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "'Debugger listening on port 5858\\\\n'\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# debug test\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "Should output debugger message",
      "id": 12,
      "name": "Should output debugger message",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 1\\n",
  ],
  Array [
    "comment",
    "# pass  1\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/debug-test.js",
      "id": 13,
      "name": "test/debug-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deep-strict.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# strictDeepEquals shouldn't care about key order\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be strictly equal",
      "id": 14,
      "name": "should be strictly equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# strictDeepEquals shouldn't care about key order recursively\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be strictly equal",
      "id": 15,
      "name": "should be strictly equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# strictDeepEquals shoudn't care about key order (but still might)\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be strictly equal",
      "id": 16,
      "name": "should be strictly equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/deep-strict.js",
      "id": 17,
      "name": "test/deep-strict.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deep.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# deepEquals shouldn't care about key order and types\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 18,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deepEquals shouldn't care about key order recursively and types\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 19,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deepEquals shoudn't care about key order (but still might) and types\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 20,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/deep.js",
      "id": 21,
      "name": "test/deep.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# executed.sh\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "File with executable bit should be executed",
      "id": 22,
      "name": "File with executable bit should be executed",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/executed.sh",
      "id": 23,
      "name": "test/executed.sh",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# exit-code.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# exit code 1 when tap results show failure\\n",
  ],
  Array [
    "comment",
    "# test exits 0, has failures\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 24,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 25,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test exits 1, has failures\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 26,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 27,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test exits 1, has no failures\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 28,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 29,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# successes exit 0\\n",
  ],
  Array [
    "comment",
    "# test that does nothing, but exits 0\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 30,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 31,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test that succeeds, and exits 0\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 32,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 33,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 10\\n",
  ],
  Array [
    "comment",
    "# pass  10\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/exit-code.js",
      "id": 34,
      "name": "test/exit-code.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# expose-gc-test.js\\n",
  ],
  Array [
    "extra",
    "gc test\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "assert gc does not exist\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# gc test when the gc isn't there\\n",
  ],
  Array [
    "extra",
    "gc test\\n",
  ],
  Array [
    "extra",
    "t.plan=2\\n",
  ],
  Array [
    "extra",
    "gc test using --gc\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "assert gc exists\\n",
  ],
  Array [
    "extra",
    "gc test using --expose-gc\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "assert gc exists\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 35,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# gc test when the gc should be there\\n",
  ],
  Array [
    "comment",
    "# test for gc using --gc\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 36,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test for gc using --expose-gc\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 37,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# cleanup\\n",
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/expose-gc-test.js",
      "id": 38,
      "name": "test/expose-gc-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# global-harness-async.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# outer\\n",
  ],
  Array [
    "comment",
    "# inner 1\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "1-1",
      "id": 39,
      "name": "1-1",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# inner 2\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "2-1",
      "id": 40,
      "name": "2-1",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# inner 3\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "3-1",
      "id": 41,
      "name": "3-1",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/global-harness-async.js",
      "id": 42,
      "name": "test/global-harness-async.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# independent-timeouts.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# finishes in time\\n",
  ],
  Array [
    "comment",
    "# finishes in time too\\n",
  ],
  Array [
    "comment",
    "# tests 0\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/independent-timeouts.js",
      "id": 43,
      "name": "test/independent-timeouts.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# isolated-conf-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# one\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 44,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 45,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# two\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 46,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 47,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 4\\n",
  ],
  Array [
    "comment",
    "# pass  4\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/isolated-conf-test.js",
      "id": 48,
      "name": "test/isolated-conf-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# meta-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# meta test\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "sanity check",
      "id": 49,
      "name": "sanity check",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "not ok",
      "id": 50,
      "name": "not ok",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "total test count",
      "id": 51,
      "name": "total test count",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "tests passed",
      "id": 52,
      "name": "tests passed",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "tests failed",
      "id": 53,
      "name": "tests failed",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ok is boolean",
      "id": 54,
      "name": "ok is boolean",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip is number",
      "id": 55,
      "name": "skip is number",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "results isa Results",
      "id": 56,
      "name": "results isa Results",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test isa Test",
      "id": 57,
      "name": "test isa Test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test isa Harness",
      "id": 58,
      "name": "test isa Harness",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 10\\n",
  ],
  Array [
    "comment",
    "# pass  10\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/meta-test.js",
      "id": 59,
      "name": "test/meta-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# nested-async.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# Harness async test support\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "sync child A",
      "id": 60,
      "name": "sync child A",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# sync child B\\n",
  ],
  Array [
    "comment",
    "# async grandchild A\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "(unnamed assert)",
      "id": 61,
      "name": "(unnamed assert)",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# async grandchild B\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "(unnamed assert)",
      "id": 62,
      "name": "(unnamed assert)",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# async child\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "sync grandchild in async child A",
      "id": 63,
      "name": "sync grandchild in async child A",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# sync grandchild in async child B\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "(unnamed assert)",
      "id": 64,
      "name": "(unnamed assert)",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 5\\n",
  ],
  Array [
    "comment",
    "# pass  5\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/nested-async.js",
      "id": 65,
      "name": "test/nested-async.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# nested-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# parent\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "p test",
      "id": 66,
      "name": "p test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# subtest\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ch test",
      "id": 67,
      "name": "ch test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# nested subtest\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "grch test",
      "id": 68,
      "name": "grch test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# another subtest\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ch test 2",
      "id": 69,
      "name": "ch test 2",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 4\\n",
  ],
  Array [
    "comment",
    "# pass  4\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/nested-test.js",
      "id": 70,
      "name": "test/nested-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# non-tap-output.js\\n",
  ],
  Array [
    "comment",
    "# everything is fine\\n",
  ],
  Array [
    "comment",
    "# there are no errors\\n",
  ],
  Array [
    "comment",
    "# this output is not haiku.\\n",
  ],
  Array [
    "comment",
    "# is 8 ok?\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": ", 8 can stay.",
      "id": 71,
      "name": ", 8 can stay.",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# but: nevertheless, here we are\\n",
  ],
  Array [
    "comment",
    "# this: is indented\\n",
  ],
  Array [
    "comment",
    "# and: it\\n",
  ],
  Array [
    "comment",
    "# might: ~\\n",
  ],
  Array [
    "comment",
    "# be: yaml?\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "might be confusing",
      "id": 72,
      "name": "might be confusing",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "done now, exiting",
      "id": 73,
      "name": "done now, exiting",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/non-tap-output.js",
      "id": 74,
      "name": "test/non-tap-output.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# not-executed.sh\\n",
  ],
  Array [
    "comment",
    "# output-childtest-description.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# nested tests, parent and child pass\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs parent description",
      "id": 75,
      "name": "outputs parent description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs child description",
      "id": 76,
      "name": "outputs child description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs parent description before parent result",
      "id": 77,
      "name": "outputs parent description before parent result",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs parent result before child description",
      "id": 78,
      "name": "outputs parent result before child description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs child description before child result",
      "id": 79,
      "name": "outputs child description before child result",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 5\\n",
  ],
  Array [
    "comment",
    "# pass  5\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/output-childtest-description.js",
      "id": 80,
      "name": "test/output-childtest-description.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# result-trap.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# trap result\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 81,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 82,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 2\\n",
  ],
  Array [
    "comment",
    "# pass  2\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/result-trap.js",
      "id": 83,
      "name": "test/result-trap.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# segv.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# setup\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "compiled seg faulter",
      "id": 84,
      "name": "compiled seg faulter",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# segv\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 85,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 86,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 87,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 88,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 89,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 90,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# cleanup\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "cleaned up",
      "id": 91,
      "name": "cleaned up",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 8\\n",
  ],
  Array [
    "comment",
    "# pass  8\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/segv.js",
      "id": 92,
      "name": "test/segv.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# simple-harness-test-with-plan.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# trivial success\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "it works",
      "id": 93,
      "name": "it works",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# two tests\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "math should work",
      "id": 94,
      "name": "math should work",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false should not be ok",
      "id": 95,
      "name": "false should not be ok",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/simple-harness-test-with-plan.js",
      "id": 96,
      "name": "test/simple-harness-test-with-plan.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# simple-harness-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# trivial success\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "it works",
      "id": 97,
      "name": "it works",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# two tests\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "math should work",
      "id": 98,
      "name": "math should work",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false should not be ok",
      "id": 99,
      "name": "false should not be ok",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/simple-harness-test.js",
      "id": 100,
      "name": "test/simple-harness-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-assert-todo-skip.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# not much\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "always passes",
      "id": 101,
      "name": "always passes",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": "skip it good",
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false",
      "id": 102,
      "name": "false",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": "always fails",
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "bonus",
      "id": 103,
      "name": "bonus",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": "remove todo directive",
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "expected",
      "id": 104,
      "name": "expected",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": "implement a thing",
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "always passes without explanation",
      "id": 105,
      "name": "always passes without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": true,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false without explanation",
      "id": 106,
      "name": "false without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": true,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "bonus without explanation",
      "id": 107,
      "name": "bonus without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": true,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "expected without explanation",
      "id": 108,
      "name": "expected without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": true,
    },
  ],
  Array [
    "comment",
    "# tests 8\\n",
  ],
  Array [
    "comment",
    "# skip  4\\n",
  ],
  Array [
    "comment",
    "# todo  4\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/test-assert-todo-skip.js",
      "id": 109,
      "name": "test/test-assert-todo-skip.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-descriptions.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# captures test descriptions\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "exit cleanly",
      "id": 110,
      "name": "exit cleanly",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures SKIP description",
      "id": 111,
      "name": "captures SKIP description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary is not from file",
      "id": 112,
      "name": "skip summary is not from file",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not from file",
      "id": 113,
      "name": "todo summary is not from file",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "column": 7,
        "file": "child_process.js",
        "line": 707,
        "stack": Array [
          "getCaller (/some/path/tap-assert.js:439:17)\\n",
          "Function.assert (/some/path/tap-assert.js:21:16)\\n",
          "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
          "/some/path/test/test-descriptions.js:32:7\\n",
          "ChildProcess.exithandler (child_process.js:707:7)\\n",
          "emitTwo (events.js:87:13)\\n",
          "ChildProcess.emit (events.js:169:7)\\n",
          "maybeClose (child_process.js:984:16)\\n",
          "Socket.<anonymous> (child_process.js:1153:11)\\n",
          "emitOne (events.js:77:13)\\n",
        ],
      },
      "fullname": "captures TODO description",
      "id": 114,
      "name": "captures TODO description",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "exit cleanly",
      "id": 115,
      "name": "exit cleanly",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures SKIP description",
      "id": 116,
      "name": "captures SKIP description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary is not in TAP output",
      "id": 117,
      "name": "skip summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not in TAP output",
      "id": 118,
      "name": "todo summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "column": 7,
        "file": "child_process.js",
        "line": 707,
        "stack": Array [
          "getCaller (/some/path/tap-assert.js:439:17)\\n",
          "Function.assert (/some/path/tap-assert.js:21:16)\\n",
          "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
          "/some/path/test/test-descriptions.js:18:7\\n",
          "ChildProcess.exithandler (child_process.js:707:7)\\n",
          "emitTwo (events.js:87:13)\\n",
          "ChildProcess.emit (events.js:169:7)\\n",
          "maybeClose (child_process.js:984:16)\\n",
          "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
        ],
      },
      "fullname": "captures TODO description",
      "id": 119,
      "name": "captures TODO description",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "exit cleanly",
      "id": 120,
      "name": "exit cleanly",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "column": 7,
        "file": "child_process.js",
        "line": 707,
        "stack": Array [
          "getCaller (/some/path/tap-assert.js:439:17)\\n",
          "Function.assert (/some/path/tap-assert.js:21:16)\\n",
          "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
          "/some/path/test/test-descriptions.js:23:7\\n",
          "ChildProcess.exithandler (child_process.js:707:7)\\n",
          "emitTwo (events.js:87:13)\\n",
          "ChildProcess.emit (events.js:169:7)\\n",
          "maybeClose (child_process.js:984:16)\\n",
          "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
        ],
      },
      "fullname": "summarizes skipped count",
      "id": 121,
      "name": "summarizes skipped count",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "column": 7,
        "file": "child_process.js",
        "line": 707,
        "stack": Array [
          "getCaller (/some/path/tap-assert.js:439:17)\\n",
          "Function.assert (/some/path/tap-assert.js:21:16)\\n",
          "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
          "/some/path/test/test-descriptions.js:24:7\\n",
          "ChildProcess.exithandler (child_process.js:707:7)\\n",
          "emitTwo (events.js:87:13)\\n",
          "ChildProcess.emit (events.js:169:7)\\n",
          "maybeClose (child_process.js:984:16)\\n",
          "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
        ],
      },
      "fullname": "summarizes todo count",
      "id": 122,
      "name": "summarizes todo count",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 13\\n",
  ],
  Array [
    "comment",
    "# pass  9\\n",
  ],
  Array [
    "comment",
    "# fail  4\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "command": "/usr/local/bin/iojs test-descriptions.js",
        "exit": 1,
      },
      "fullname": "test/test-descriptions.js",
      "id": 123,
      "name": "test/test-descriptions.js",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-directives.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# captures test descriptions\\n",
  ],
  Array [
    "comment",
    "# raw TAP > TAP consumer > TAP producer\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "overall result is PASS",
      "id": 124,
      "name": "overall result is PASS",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures ok SKIP",
      "id": 125,
      "name": "captures ok SKIP",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures not ok SKIP",
      "id": 126,
      "name": "captures not ok SKIP",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary not in TAP output",
      "id": 127,
      "name": "skip summary not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures ok TODO",
      "id": 128,
      "name": "captures ok TODO",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures not ok TODO",
      "id": 129,
      "name": "captures not ok TODO",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not in TAP output",
      "id": 130,
      "name": "todo summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "no ugly \\"undefined\\" in output",
      "id": 131,
      "name": "no ugly \\"undefined\\" in output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# raw TAP > TAP consumer > summary\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "overall result is PASS",
      "id": 132,
      "name": "overall result is PASS",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "no SKIP in summary",
      "id": 133,
      "name": "no SKIP in summary",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary is not in TAP output",
      "id": 134,
      "name": "skip summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "no TODO in summary",
      "id": 135,
      "name": "no TODO in summary",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not in TAP output",
      "id": 136,
      "name": "todo summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "no ugly \\"undefined\\" in output",
      "id": 137,
      "name": "no ugly \\"undefined\\" in output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# TAP producer via require(\\"tap\\")\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "overall result is PASS",
      "id": 138,
      "name": "overall result is PASS",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures ok SKIP",
      "id": 139,
      "name": "captures ok SKIP",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures not ok SKIP",
      "id": 140,
      "name": "captures not ok SKIP",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary not in TAP output",
      "id": 141,
      "name": "skip summary not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures ok TODO",
      "id": 142,
      "name": "captures ok TODO",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures not ok TODO",
      "id": 143,
      "name": "captures not ok TODO",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not in TAP output",
      "id": 144,
      "name": "todo summary is not in TAP output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "no ugly \\"undefined\\" in output",
      "id": 145,
      "name": "no ugly \\"undefined\\" in output",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 22\\n",
  ],
  Array [
    "comment",
    "# pass  22\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/test-directives.js",
      "id": 146,
      "name": "test/test-directives.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-skip.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "does not count as failure",
      "id": 147,
      "name": "does not count as failure",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": true,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 1\\n",
  ],
  Array [
    "comment",
    "# skip  1\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/test-skip.js",
      "id": 148,
      "name": "test/test-skip.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# testing the test object\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test object should be instanceof Test",
      "id": 149,
      "name": "test object should be instanceof Test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test object should be instanceof Harness",
      "id": 150,
      "name": "test object should be instanceof Harness",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test._Test should be the Test class",
      "id": 151,
      "name": "test._Test should be the Test class",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotDeepEqual method",
      "id": 152,
      "name": "should have isNotDeepEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotDeepEqual method should be a function",
      "id": 153,
      "name": "isNotDeepEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have equals method",
      "id": 154,
      "name": "should have equals method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "equals method should be a function",
      "id": 155,
      "name": "equals method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have inequivalent method",
      "id": 156,
      "name": "should have inequivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "inequivalent method should be a function",
      "id": 157,
      "name": "inequivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have threw method",
      "id": 158,
      "name": "should have threw method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "threw method should be a function",
      "id": 159,
      "name": "threw method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have strictEqual method",
      "id": 160,
      "name": "should have strictEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "strictEqual method should be a function",
      "id": 161,
      "name": "strictEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have emit method",
      "id": 162,
      "name": "should have emit method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "emit method should be a function",
      "id": 163,
      "name": "emit method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have fail method",
      "id": 164,
      "name": "should have fail method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "fail method should be a function",
      "id": 165,
      "name": "fail method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have strictEquals method",
      "id": 166,
      "name": "should have strictEquals method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "strictEquals method should be a function",
      "id": 167,
      "name": "strictEquals method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notLike method",
      "id": 168,
      "name": "should have notLike method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notLike method should be a function",
      "id": 169,
      "name": "notLike method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have dissimilar method",
      "id": 170,
      "name": "should have dissimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "dissimilar method should be a function",
      "id": 171,
      "name": "dissimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have true method",
      "id": 172,
      "name": "should have true method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "true method should be a function",
      "id": 173,
      "name": "true method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have assert method",
      "id": 174,
      "name": "should have assert method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "assert method should be a function",
      "id": 175,
      "name": "assert method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have is method",
      "id": 176,
      "name": "should have is method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "is method should be a function",
      "id": 177,
      "name": "is method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have ok method",
      "id": 178,
      "name": "should have ok method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ok method should be a function",
      "id": 179,
      "name": "ok method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isEqual method",
      "id": 180,
      "name": "should have isEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isEqual method should be a function",
      "id": 181,
      "name": "isEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isDeeply method",
      "id": 182,
      "name": "should have isDeeply method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isDeeply method should be a function",
      "id": 183,
      "name": "isDeeply method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have deepEqual method",
      "id": 184,
      "name": "should have deepEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "deepEqual method should be a function",
      "id": 185,
      "name": "deepEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have deepEquals method",
      "id": 186,
      "name": "should have deepEquals method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "deepEquals method should be a function",
      "id": 187,
      "name": "deepEquals method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have pass method",
      "id": 188,
      "name": "should have pass method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "pass method should be a function",
      "id": 189,
      "name": "pass method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have length method",
      "id": 190,
      "name": "should have length method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "length method should be a function",
      "id": 191,
      "name": "length method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have skip method",
      "id": 192,
      "name": "should have skip method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip method should be a function",
      "id": 193,
      "name": "skip method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotEqual method",
      "id": 194,
      "name": "should have isNotEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotEqual method should be a function",
      "id": 195,
      "name": "isNotEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have looseEquals method",
      "id": 196,
      "name": "should have looseEquals method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "looseEquals method should be a function",
      "id": 197,
      "name": "looseEquals method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have false method",
      "id": 198,
      "name": "should have false method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false method should be a function",
      "id": 199,
      "name": "false method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notDeeply method",
      "id": 200,
      "name": "should have notDeeply method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notDeeply method should be a function",
      "id": 201,
      "name": "notDeeply method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have ifErr method",
      "id": 202,
      "name": "should have ifErr method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ifErr method should be a function",
      "id": 203,
      "name": "ifErr method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have hasFields method",
      "id": 204,
      "name": "should have hasFields method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "hasFields method should be a function",
      "id": 205,
      "name": "hasFields method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotDeeply method",
      "id": 206,
      "name": "should have isNotDeeply method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotDeeply method should be a function",
      "id": 207,
      "name": "isNotDeeply method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have like method",
      "id": 208,
      "name": "should have like method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "like method should be a function",
      "id": 209,
      "name": "like method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have similar method",
      "id": 210,
      "name": "should have similar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "similar method should be a function",
      "id": 211,
      "name": "similar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notOk method",
      "id": 212,
      "name": "should have notOk method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notOk method should be a function",
      "id": 213,
      "name": "notOk method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isDissimilar method",
      "id": 214,
      "name": "should have isDissimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isDissimilar method should be a function",
      "id": 215,
      "name": "isDissimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isEquivalent method",
      "id": 216,
      "name": "should have isEquivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isEquivalent method should be a function",
      "id": 217,
      "name": "isEquivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have doesNotEqual method",
      "id": 218,
      "name": "should have doesNotEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "doesNotEqual method should be a function",
      "id": 219,
      "name": "doesNotEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isSimilar method",
      "id": 220,
      "name": "should have isSimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isSimilar method should be a function",
      "id": 221,
      "name": "isSimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notDeepEqual method",
      "id": 222,
      "name": "should have notDeepEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notDeepEqual method should be a function",
      "id": 223,
      "name": "notDeepEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have type method",
      "id": 224,
      "name": "should have type method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "type method should be a function",
      "id": 225,
      "name": "type method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notok method",
      "id": 226,
      "name": "should have notok method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notok method should be a function",
      "id": 227,
      "name": "notok method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isInequivalent method",
      "id": 228,
      "name": "should have isInequivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isInequivalent method should be a function",
      "id": 229,
      "name": "isInequivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNot method",
      "id": 230,
      "name": "should have isNot method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNot method should be a function",
      "id": 231,
      "name": "isNot method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have same method",
      "id": 232,
      "name": "should have same method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "same method should be a function",
      "id": 233,
      "name": "same method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isInequal method",
      "id": 234,
      "name": "should have isInequal method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isInequal method should be a function",
      "id": 235,
      "name": "isInequal method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have _endNice method",
      "id": 236,
      "name": "should have _endNice method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "_endNice method should be a function",
      "id": 237,
      "name": "_endNice method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have ifError method",
      "id": 238,
      "name": "should have ifError method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ifError method should be a function",
      "id": 239,
      "name": "ifError method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have iferror method",
      "id": 240,
      "name": "should have iferror method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "iferror method should be a function",
      "id": 241,
      "name": "iferror method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have clear method",
      "id": 242,
      "name": "should have clear method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "clear method should be a function",
      "id": 243,
      "name": "clear method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have has method",
      "id": 244,
      "name": "should have has method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "has method should be a function",
      "id": 245,
      "name": "has method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have not method",
      "id": 246,
      "name": "should have not method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "not method should be a function",
      "id": 247,
      "name": "not method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have timeout method",
      "id": 248,
      "name": "should have timeout method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "timeout method should be a function",
      "id": 249,
      "name": "timeout method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notSimilar method",
      "id": 250,
      "name": "should have notSimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notSimilar method should be a function",
      "id": 251,
      "name": "notSimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isUnlike method",
      "id": 252,
      "name": "should have isUnlike method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isUnlike method should be a function",
      "id": 253,
      "name": "isUnlike method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notEquals method",
      "id": 254,
      "name": "should have notEquals method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notEquals method should be a function",
      "id": 255,
      "name": "notEquals method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have unsimilar method",
      "id": 256,
      "name": "should have unsimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "unsimilar method should be a function",
      "id": 257,
      "name": "unsimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have result method",
      "id": 258,
      "name": "should have result method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "result method should be a function",
      "id": 259,
      "name": "result method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have doesNotThrow method",
      "id": 260,
      "name": "should have doesNotThrow method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "doesNotThrow method should be a function",
      "id": 261,
      "name": "doesNotThrow method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have error method",
      "id": 262,
      "name": "should have error method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "error method should be a function",
      "id": 263,
      "name": "error method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have constructor method",
      "id": 264,
      "name": "should have constructor method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "constructor method should be a function",
      "id": 265,
      "name": "constructor method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notEqual method",
      "id": 266,
      "name": "should have notEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notEqual method should be a function",
      "id": 267,
      "name": "notEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have throws method",
      "id": 268,
      "name": "should have throws method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "throws method should be a function",
      "id": 269,
      "name": "throws method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isLike method",
      "id": 270,
      "name": "should have isLike method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isLike method should be a function",
      "id": 271,
      "name": "isLike method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotSimilar method",
      "id": 272,
      "name": "should have isNotSimilar method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotSimilar method should be a function",
      "id": 273,
      "name": "isNotSimilar method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotEquivalent method",
      "id": 274,
      "name": "should have isNotEquivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotEquivalent method should be a function",
      "id": 275,
      "name": "isNotEquivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have inequal method",
      "id": 276,
      "name": "should have inequal method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "inequal method should be a function",
      "id": 277,
      "name": "inequal method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have notEquivalent method",
      "id": 278,
      "name": "should have notEquivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "notEquivalent method should be a function",
      "id": 279,
      "name": "notEquivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isNotLike method",
      "id": 280,
      "name": "should have isNotLike method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isNotLike method should be a function",
      "id": 281,
      "name": "isNotLike method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have equivalent method",
      "id": 282,
      "name": "should have equivalent method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "equivalent method should be a function",
      "id": 283,
      "name": "equivalent method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have looseEqual method",
      "id": 284,
      "name": "should have looseEqual method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "looseEqual method should be a function",
      "id": 285,
      "name": "looseEqual method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have equal method",
      "id": 286,
      "name": "should have equal method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "equal method should be a function",
      "id": 287,
      "name": "equal method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have unlike method",
      "id": 288,
      "name": "should have unlike method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "unlike method should be a function",
      "id": 289,
      "name": "unlike method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have doesNotHave method",
      "id": 290,
      "name": "should have doesNotHave method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "doesNotHave method should be a function",
      "id": 291,
      "name": "doesNotHave method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have comment method",
      "id": 292,
      "name": "should have comment method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "comment method should be a function",
      "id": 293,
      "name": "comment method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should have isa method",
      "id": 294,
      "name": "should have isa method",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "isa method should be a function",
      "id": 295,
      "name": "isa method should be a function",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 147\\n",
  ],
  Array [
    "comment",
    "# pass  147\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/test-test.js",
      "id": 296,
      "name": "test/test-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# timeout.js\\n",
  ],
  Array [
    "extra",
    "timeout test\\n",
  ],
  Array [
    "extra",
    "t.plan=2\\n",
  ],
  Array [
    "extra",
    "a assert\\n",
  ],
  Array [
    "extra",
    "b assert\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# timeout test with plan only\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "a",
      "id": 297,
      "name": "a",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "extra",
    "timeout test\\n",
  ],
  Array [
    "extra",
    "t.plan=2\\n",
  ],
  Array [
    "extra",
    "a assert\\n",
  ],
  Array [
    "extra",
    "b assert\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "b",
      "id": 298,
      "name": "b",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# timeout test with plan and end\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "a",
      "id": 299,
      "name": "a",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "b",
      "id": 300,
      "name": "b",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 4\\n",
  ],
  Array [
    "comment",
    "# pass  4\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/timeout.js",
      "id": 301,
      "name": "test/timeout.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# trivial-success.js\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/trivial-success.js",
      "id": 302,
      "name": "test/trivial-success.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# undefined_indented.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# consume yaml\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 303,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 1\\n",
  ],
  Array [
    "comment",
    "# pass  1\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/undefined_indented.js",
      "id": 304,
      "name": "test/undefined_indented.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# valid-command.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# valid command\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 305,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 306,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 307,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 308,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 309,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 310,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 6\\n",
  ],
  Array [
    "comment",
    "# pass  6\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/valid-command.js",
      "id": 311,
      "name": "test/valid-command.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "plan",
    Plan {
      "comment": "",
      "end": 311,
      "start": 1,
    },
  ],
  Array [
    "comment",
    "# tests 311\\n",
  ],
  Array [
    "comment",
    "# pass  297\\n",
  ],
  Array [
    "comment",
    "# fail  5\\n",
  ],
  Array [
    "comment",
    "# skip  5\\n",
  ],
  Array [
    "comment",
    "# todo  4\\n",
  ],
  Array [
    "complete",
    FinalResults {
      "bailout": false,
      "count": 311,
      "fail": 27,
      "failures": Array [
        Object {
          "data": "debug test\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "t.plan=1\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "'Debugger listening on port 5858\\\\n'\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "gc test\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "t.plan=1\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "assert gc does not exist\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "gc test\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "t.plan=2\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "gc test using --gc\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "t.plan=1\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "assert gc exists\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "gc test using --expose-gc\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "t.plan=1\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "assert gc exists\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "column": 7,
            "file": "child_process.js",
            "line": 707,
            "stack": Array [
              "getCaller (/some/path/tap-assert.js:439:17)\\n",
              "Function.assert (/some/path/tap-assert.js:21:16)\\n",
              "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
              "/some/path/test/test-descriptions.js:32:7\\n",
              "ChildProcess.exithandler (child_process.js:707:7)\\n",
              "emitTwo (events.js:87:13)\\n",
              "ChildProcess.emit (events.js:169:7)\\n",
              "maybeClose (child_process.js:984:16)\\n",
              "Socket.<anonymous> (child_process.js:1153:11)\\n",
              "emitOne (events.js:77:13)\\n",
            ],
          },
          "fullname": "captures TODO description",
          "id": 114,
          "name": "captures TODO description",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "column": 7,
            "file": "child_process.js",
            "line": 707,
            "stack": Array [
              "getCaller (/some/path/tap-assert.js:439:17)\\n",
              "Function.assert (/some/path/tap-assert.js:21:16)\\n",
              "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
              "/some/path/test/test-descriptions.js:18:7\\n",
              "ChildProcess.exithandler (child_process.js:707:7)\\n",
              "emitTwo (events.js:87:13)\\n",
              "ChildProcess.emit (events.js:169:7)\\n",
              "maybeClose (child_process.js:984:16)\\n",
              "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
            ],
          },
          "fullname": "captures TODO description",
          "id": 119,
          "name": "captures TODO description",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "column": 7,
            "file": "child_process.js",
            "line": 707,
            "stack": Array [
              "getCaller (/some/path/tap-assert.js:439:17)\\n",
              "Function.assert (/some/path/tap-assert.js:21:16)\\n",
              "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
              "/some/path/test/test-descriptions.js:23:7\\n",
              "ChildProcess.exithandler (child_process.js:707:7)\\n",
              "emitTwo (events.js:87:13)\\n",
              "ChildProcess.emit (events.js:169:7)\\n",
              "maybeClose (child_process.js:984:16)\\n",
              "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
            ],
          },
          "fullname": "summarizes skipped count",
          "id": 121,
          "name": "summarizes skipped count",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "column": 7,
            "file": "child_process.js",
            "line": 707,
            "stack": Array [
              "getCaller (/some/path/tap-assert.js:439:17)\\n",
              "Function.assert (/some/path/tap-assert.js:21:16)\\n",
              "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
              "/some/path/test/test-descriptions.js:24:7\\n",
              "ChildProcess.exithandler (child_process.js:707:7)\\n",
              "emitTwo (events.js:87:13)\\n",
              "ChildProcess.emit (events.js:169:7)\\n",
              "maybeClose (child_process.js:984:16)\\n",
              "Process.ChildProcess._handle.onexit (child_process.js:1057:5)\\n",
            ],
          },
          "fullname": "summarizes todo count",
          "id": 122,
          "name": "summarizes todo count",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "command": "/usr/local/bin/iojs test-descriptions.js",
            "exit": 1,
          },
          "fullname": "test/test-descriptions.js",
          "id": 123,
          "name": "test/test-descriptions.js",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "data": "timeout test\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "t.plan=2\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "a assert\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "b assert\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "timeout test\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "t.plan=2\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "a assert\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "b assert\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
      ],
      "ok": false,
      "pass": 306,
      "passes": undefined,
      "plan": FinalPlan {
        "comment": "",
        "end": 311,
        "skipAll": false,
        "skipReason": "",
        "start": 1,
      },
      "skip": 5,
      "skips": Array [
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "always passes",
          "id": 101,
          "name": "always passes",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": "skip it good",
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "false",
          "id": 102,
          "name": "false",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": "always fails",
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "always passes without explanation",
          "id": 105,
          "name": "always passes without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": true,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "false without explanation",
          "id": 106,
          "name": "false without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": true,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "does not count as failure",
          "id": 147,
          "name": "does not count as failure",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": true,
          "tapError": null,
          "time": null,
          "todo": false,
        },
      ],
      "time": null,
      "todo": 4,
      "todos": Array [
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "bonus",
          "id": 103,
          "name": "bonus",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": "remove todo directive",
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "expected",
          "id": 104,
          "name": "expected",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": "implement a thing",
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "bonus without explanation",
          "id": 107,
          "name": "bonus without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": true,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "expected without explanation",
          "id": 108,
          "name": "expected without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": true,
        },
      ],
    },
  ],
  Array [
    "finish",
  ],
  Array [
    "close",
  ],
]
`

exports[`test/parse-stringify.ts > TAP > tap-tests.tap > strict > stringified 1`] = `
TAP version 13
# buffer_compare.js
# TAP version 13
# same buffers
ok 1 - should be equivalent
# not same buffers
ok 2 - should not be equivalent
# tests 2
# pass  2
# ok
ok 3 - test/buffer_compare.js
# common.js
ok 4 - just setup, nothing relevant
ok 5 - test/common.js
# consumer.js
# TAP version 13
# basic.tap
ok 6 - should be equivalent
# indent.tap
ok 7 - should be equivalent
# missing.tap
ok 8 - should be equivalent
# skip-all.tap
ok 9 - should be equivalent
# yamlish.tap
ok 10 - should be equivalent
# tests 5
# pass  5
# ok
ok 11 - test/consumer.js
# debug-test.js
debug test
t.plan=1
'Debugger listening on port 5858\\n'
# TAP version 13
# debug test
ok 12 - Should output debugger message
# tests 1
# pass  1
# ok
ok 13 - test/debug-test.js
# deep-strict.js
# TAP version 13
# strictDeepEquals shouldn't care about key order
ok 14 - should be strictly equal
# strictDeepEquals shouldn't care about key order recursively
ok 15 - should be strictly equal
# strictDeepEquals shoudn't care about key order (but still might)
ok 16 - should be strictly equal
# tests 3
# pass  3
# ok
ok 17 - test/deep-strict.js
# deep.js
# TAP version 13
# deepEquals shouldn't care about key order and types
ok 18 - should be equivalent
# deepEquals shouldn't care about key order recursively and types
ok 19 - should be equivalent
# deepEquals shoudn't care about key order (but still might) and types
ok 20 - should be equivalent
# tests 3
# pass  3
# ok
ok 21 - test/deep.js
# executed.sh
ok 22 - File with executable bit should be executed
ok 23 - test/executed.sh
# exit-code.js
# TAP version 13
# exit code 1 when tap results show failure
# test exits 0, has failures
ok 24 - should be equal
ok 25 - should be equal
# test exits 1, has failures
ok 26 - should be equal
ok 27 - should be equal
# test exits 1, has no failures
ok 28 - should be equal
ok 29 - should be equal
# successes exit 0
# test that does nothing, but exits 0
ok 30 - should be equal
ok 31 - should be equal
# test that succeeds, and exits 0
ok 32 - should be equal
ok 33 - should be equal
# tests 10
# pass  10
# ok
ok 34 - test/exit-code.js
# expose-gc-test.js
gc test
t.plan=1
assert gc does not exist
# TAP version 13
# gc test when the gc isn't there
gc test
t.plan=2
gc test using --gc
t.plan=1
assert gc exists
gc test using --expose-gc
t.plan=1
assert gc exists
ok 35 - should be equal
# gc test when the gc should be there
# test for gc using --gc
ok 36 - should be equal
# test for gc using --expose-gc
ok 37 - should be equal
# cleanup
# tests 3
# pass  3
# ok
ok 38 - test/expose-gc-test.js
# global-harness-async.js
# TAP version 13
# outer
# inner 1
ok 39 - 1-1
# inner 2
ok 40 - 2-1
# inner 3
ok 41 - 3-1
ok 42 - test/global-harness-async.js
# independent-timeouts.js
# TAP version 13
# finishes in time
# finishes in time too
# tests 0
# ok
ok 43 - test/independent-timeouts.js
# isolated-conf-test.js
# TAP version 13
# one
ok 44 - should be equal
ok 45 - should be equal
# two
ok 46 - should be equal
ok 47 - should be equal
# tests 4
# pass  4
# ok
ok 48 - test/isolated-conf-test.js
# meta-test.js
# TAP version 13
# meta test
ok 49 - sanity check
ok 50 - not ok
ok 51 - total test count
ok 52 - tests passed
ok 53 - tests failed
ok 54 - ok is boolean
ok 55 - skip is number
ok 56 - results isa Results
ok 57 - test isa Test
ok 58 - test isa Harness
# tests 10
# pass  10
# ok
ok 59 - test/meta-test.js
# nested-async.js
# TAP version 13
# Harness async test support
ok 60 - sync child A
# sync child B
# async grandchild A
ok 61 - (unnamed assert)
# async grandchild B
ok 62 - (unnamed assert)
# async child
ok 63 - sync grandchild in async child A
# sync grandchild in async child B
ok 64 - (unnamed assert)
# tests 5
# pass  5
# ok
ok 65 - test/nested-async.js
# nested-test.js
# TAP version 13
# parent
ok 66 - p test
# subtest
ok 67 - ch test
# nested subtest
ok 68 - grch test
# another subtest
ok 69 - ch test 2
# tests 4
# pass  4
# ok
ok 70 - test/nested-test.js
# non-tap-output.js
# everything is fine
# there are no errors
# this output is not haiku.
# is 8 ok?
ok 71 - , 8 can stay.
# but: nevertheless, here we are
# this: is indented
# and: it
# might: ~
# be: yaml?
ok 72 - might be confusing
ok 73 - done now, exiting
ok 74 - test/non-tap-output.js
# not-executed.sh
# output-childtest-description.js
# TAP version 13
# nested tests, parent and child pass
ok 75 - outputs parent description
ok 76 - outputs child description
ok 77 - outputs parent description before parent result
ok 78 - outputs parent result before child description
ok 79 - outputs child description before child result
# tests 5
# pass  5
# ok
ok 80 - test/output-childtest-description.js
# result-trap.js
# TAP version 13
# trap result
ok 81 - should be equal
ok 82 - should be equal
# tests 2
# pass  2
# ok
ok 83 - test/result-trap.js
# segv.js
# TAP version 13
# setup
ok 84 - compiled seg faulter
# segv
ok 85 - should be equivalent
ok 86 - should be equivalent
ok 87 - should be equivalent
ok 88 - should be equivalent
ok 89 - should be equivalent
ok 90 - should be equal
# cleanup
ok 91 - cleaned up
# tests 8
# pass  8
# ok
ok 92 - test/segv.js
# simple-harness-test-with-plan.js
# TAP version 13
# trivial success
ok 93 - it works
# two tests
ok 94 - math should work
ok 95 - false should not be ok
# tests 3
# pass  3
# ok
ok 96 - test/simple-harness-test-with-plan.js
# simple-harness-test.js
# TAP version 13
# trivial success
ok 97 - it works
# two tests
ok 98 - math should work
ok 99 - false should not be ok
# tests 3
# pass  3
# ok
ok 100 - test/simple-harness-test.js
# test-assert-todo-skip.js
# TAP version 13
# not much
ok 101 - always passes # SKIP skip it good
ok 102 - false # SKIP always fails
ok 103 - bonus # TODO remove todo directive
ok 104 - expected # TODO implement a thing
ok 105 - always passes without explanation # SKIP
ok 106 - false without explanation # SKIP
ok 107 - bonus without explanation # TODO
ok 108 - expected without explanation # TODO
# tests 8
# skip  4
# todo  4
ok 109 - test/test-assert-todo-skip.js
# test-descriptions.js
# TAP version 13
# captures test descriptions
ok 110 - exit cleanly
ok 111 - captures SKIP description
ok 112 - skip summary is not from file
ok 113 - todo summary is not from file
not ok 114 - captures TODO description
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:32:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Socket.<anonymous> (child_process.js:1153:11)
    - |
      emitOne (events.js:77:13)
  ...
ok 115 - exit cleanly
ok 116 - captures SKIP description
ok 117 - skip summary is not in TAP output
ok 118 - todo summary is not in TAP output
not ok 119 - captures TODO description
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:18:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Process.ChildProcess._handle.onexit (child_process.js:1057:5)
  ...
ok 120 - exit cleanly
not ok 121 - summarizes skipped count
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:23:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Process.ChildProcess._handle.onexit (child_process.js:1057:5)
  ...
not ok 122 - summarizes todo count
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:24:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Process.ChildProcess._handle.onexit (child_process.js:1057:5)
  ...
# tests 13
# pass  9
# fail  4
not ok 123 - test/test-descriptions.js
  ---
  exit: 1
  command: /usr/local/bin/iojs test-descriptions.js
  ...
# test-directives.js
# TAP version 13
# captures test descriptions
# raw TAP > TAP consumer > TAP producer
ok 124 - overall result is PASS
ok 125 - captures ok SKIP
ok 126 - captures not ok SKIP
ok 127 - skip summary not in TAP output
ok 128 - captures ok TODO
ok 129 - captures not ok TODO
ok 130 - todo summary is not in TAP output
ok 131 - no ugly "undefined" in output
# raw TAP > TAP consumer > summary
ok 132 - overall result is PASS
ok 133 - no SKIP in summary
ok 134 - skip summary is not in TAP output
ok 135 - no TODO in summary
ok 136 - todo summary is not in TAP output
ok 137 - no ugly "undefined" in output
# TAP producer via require("tap")
ok 138 - overall result is PASS
ok 139 - captures ok SKIP
ok 140 - captures not ok SKIP
ok 141 - skip summary not in TAP output
ok 142 - captures ok TODO
ok 143 - captures not ok TODO
ok 144 - todo summary is not in TAP output
ok 145 - no ugly "undefined" in output
# tests 22
# pass  22
# ok
ok 146 - test/test-directives.js
# test-skip.js
# TAP version 13
ok 147 - does not count as failure # SKIP
# tests 1
# skip  1
ok 148 - test/test-skip.js
# test-test.js
# TAP version 13
# testing the test object
ok 149 - test object should be instanceof Test
ok 150 - test object should be instanceof Harness
ok 151 - test._Test should be the Test class
ok 152 - should have isNotDeepEqual method
ok 153 - isNotDeepEqual method should be a function
ok 154 - should have equals method
ok 155 - equals method should be a function
ok 156 - should have inequivalent method
ok 157 - inequivalent method should be a function
ok 158 - should have threw method
ok 159 - threw method should be a function
ok 160 - should have strictEqual method
ok 161 - strictEqual method should be a function
ok 162 - should have emit method
ok 163 - emit method should be a function
ok 164 - should have fail method
ok 165 - fail method should be a function
ok 166 - should have strictEquals method
ok 167 - strictEquals method should be a function
ok 168 - should have notLike method
ok 169 - notLike method should be a function
ok 170 - should have dissimilar method
ok 171 - dissimilar method should be a function
ok 172 - should have true method
ok 173 - true method should be a function
ok 174 - should have assert method
ok 175 - assert method should be a function
ok 176 - should have is method
ok 177 - is method should be a function
ok 178 - should have ok method
ok 179 - ok method should be a function
ok 180 - should have isEqual method
ok 181 - isEqual method should be a function
ok 182 - should have isDeeply method
ok 183 - isDeeply method should be a function
ok 184 - should have deepEqual method
ok 185 - deepEqual method should be a function
ok 186 - should have deepEquals method
ok 187 - deepEquals method should be a function
ok 188 - should have pass method
ok 189 - pass method should be a function
ok 190 - should have length method
ok 191 - length method should be a function
ok 192 - should have skip method
ok 193 - skip method should be a function
ok 194 - should have isNotEqual method
ok 195 - isNotEqual method should be a function
ok 196 - should have looseEquals method
ok 197 - looseEquals method should be a function
ok 198 - should have false method
ok 199 - false method should be a function
ok 200 - should have notDeeply method
ok 201 - notDeeply method should be a function
ok 202 - should have ifErr method
ok 203 - ifErr method should be a function
ok 204 - should have hasFields method
ok 205 - hasFields method should be a function
ok 206 - should have isNotDeeply method
ok 207 - isNotDeeply method should be a function
ok 208 - should have like method
ok 209 - like method should be a function
ok 210 - should have similar method
ok 211 - similar method should be a function
ok 212 - should have notOk method
ok 213 - notOk method should be a function
ok 214 - should have isDissimilar method
ok 215 - isDissimilar method should be a function
ok 216 - should have isEquivalent method
ok 217 - isEquivalent method should be a function
ok 218 - should have doesNotEqual method
ok 219 - doesNotEqual method should be a function
ok 220 - should have isSimilar method
ok 221 - isSimilar method should be a function
ok 222 - should have notDeepEqual method
ok 223 - notDeepEqual method should be a function
ok 224 - should have type method
ok 225 - type method should be a function
ok 226 - should have notok method
ok 227 - notok method should be a function
ok 228 - should have isInequivalent method
ok 229 - isInequivalent method should be a function
ok 230 - should have isNot method
ok 231 - isNot method should be a function
ok 232 - should have same method
ok 233 - same method should be a function
ok 234 - should have isInequal method
ok 235 - isInequal method should be a function
ok 236 - should have _endNice method
ok 237 - _endNice method should be a function
ok 238 - should have ifError method
ok 239 - ifError method should be a function
ok 240 - should have iferror method
ok 241 - iferror method should be a function
ok 242 - should have clear method
ok 243 - clear method should be a function
ok 244 - should have has method
ok 245 - has method should be a function
ok 246 - should have not method
ok 247 - not method should be a function
ok 248 - should have timeout method
ok 249 - timeout method should be a function
ok 250 - should have notSimilar method
ok 251 - notSimilar method should be a function
ok 252 - should have isUnlike method
ok 253 - isUnlike method should be a function
ok 254 - should have notEquals method
ok 255 - notEquals method should be a function
ok 256 - should have unsimilar method
ok 257 - unsimilar method should be a function
ok 258 - should have result method
ok 259 - result method should be a function
ok 260 - should have doesNotThrow method
ok 261 - doesNotThrow method should be a function
ok 262 - should have error method
ok 263 - error method should be a function
ok 264 - should have constructor method
ok 265 - constructor method should be a function
ok 266 - should have notEqual method
ok 267 - notEqual method should be a function
ok 268 - should have throws method
ok 269 - throws method should be a function
ok 270 - should have isLike method
ok 271 - isLike method should be a function
ok 272 - should have isNotSimilar method
ok 273 - isNotSimilar method should be a function
ok 274 - should have isNotEquivalent method
ok 275 - isNotEquivalent method should be a function
ok 276 - should have inequal method
ok 277 - inequal method should be a function
ok 278 - should have notEquivalent method
ok 279 - notEquivalent method should be a function
ok 280 - should have isNotLike method
ok 281 - isNotLike method should be a function
ok 282 - should have equivalent method
ok 283 - equivalent method should be a function
ok 284 - should have looseEqual method
ok 285 - looseEqual method should be a function
ok 286 - should have equal method
ok 287 - equal method should be a function
ok 288 - should have unlike method
ok 289 - unlike method should be a function
ok 290 - should have doesNotHave method
ok 291 - doesNotHave method should be a function
ok 292 - should have comment method
ok 293 - comment method should be a function
ok 294 - should have isa method
ok 295 - isa method should be a function
# tests 147
# pass  147
# ok
ok 296 - test/test-test.js
# timeout.js
timeout test
t.plan=2
a assert
b assert
# TAP version 13
# timeout test with plan only
ok 297 - a
timeout test
t.plan=2
a assert
b assert
ok 298 - b
# timeout test with plan and end
ok 299 - a
ok 300 - b
# tests 4
# pass  4
# ok
ok 301 - test/timeout.js
# trivial-success.js
ok 302 - test/trivial-success.js
# undefined_indented.js
# TAP version 13
# consume yaml
ok 303 - should be equivalent
# tests 1
# pass  1
# ok
ok 304 - test/undefined_indented.js
# valid-command.js
# TAP version 13
# valid command
ok 305 - should be equivalent
ok 306 - should be equivalent
ok 307 - should be equivalent
ok 308 - should be equivalent
ok 309 - should be equivalent
ok 310 - should be equal
# tests 6
# pass  6
# ok
ok 311 - test/valid-command.js
1..311
# tests 311
# pass  297
# fail  5
# skip  5
# todo  4

`

exports[`test/parse-stringify.ts > TAP > tap-tests.tap > strict > stringified flat 1`] = `
TAP version 13
# buffer_compare.js
# TAP version 13
# same buffers
ok 1 - should be equivalent
# not same buffers
ok 2 - should not be equivalent
# tests 2
# pass  2
# ok
ok 3 - test/buffer_compare.js
# common.js
ok 4 - just setup, nothing relevant
ok 5 - test/common.js
# consumer.js
# TAP version 13
# basic.tap
ok 6 - should be equivalent
# indent.tap
ok 7 - should be equivalent
# missing.tap
ok 8 - should be equivalent
# skip-all.tap
ok 9 - should be equivalent
# yamlish.tap
ok 10 - should be equivalent
# tests 5
# pass  5
# ok
ok 11 - test/consumer.js
# debug-test.js
debug test
t.plan=1
'Debugger listening on port 5858\\n'
# TAP version 13
# debug test
ok 12 - Should output debugger message
# tests 1
# pass  1
# ok
ok 13 - test/debug-test.js
# deep-strict.js
# TAP version 13
# strictDeepEquals shouldn't care about key order
ok 14 - should be strictly equal
# strictDeepEquals shouldn't care about key order recursively
ok 15 - should be strictly equal
# strictDeepEquals shoudn't care about key order (but still might)
ok 16 - should be strictly equal
# tests 3
# pass  3
# ok
ok 17 - test/deep-strict.js
# deep.js
# TAP version 13
# deepEquals shouldn't care about key order and types
ok 18 - should be equivalent
# deepEquals shouldn't care about key order recursively and types
ok 19 - should be equivalent
# deepEquals shoudn't care about key order (but still might) and types
ok 20 - should be equivalent
# tests 3
# pass  3
# ok
ok 21 - test/deep.js
# executed.sh
ok 22 - File with executable bit should be executed
ok 23 - test/executed.sh
# exit-code.js
# TAP version 13
# exit code 1 when tap results show failure
# test exits 0, has failures
ok 24 - should be equal
ok 25 - should be equal
# test exits 1, has failures
ok 26 - should be equal
ok 27 - should be equal
# test exits 1, has no failures
ok 28 - should be equal
ok 29 - should be equal
# successes exit 0
# test that does nothing, but exits 0
ok 30 - should be equal
ok 31 - should be equal
# test that succeeds, and exits 0
ok 32 - should be equal
ok 33 - should be equal
# tests 10
# pass  10
# ok
ok 34 - test/exit-code.js
# expose-gc-test.js
gc test
t.plan=1
assert gc does not exist
# TAP version 13
# gc test when the gc isn't there
gc test
t.plan=2
gc test using --gc
t.plan=1
assert gc exists
gc test using --expose-gc
t.plan=1
assert gc exists
ok 35 - should be equal
# gc test when the gc should be there
# test for gc using --gc
ok 36 - should be equal
# test for gc using --expose-gc
ok 37 - should be equal
# cleanup
# tests 3
# pass  3
# ok
ok 38 - test/expose-gc-test.js
# global-harness-async.js
# TAP version 13
# outer
# inner 1
ok 39 - 1-1
# inner 2
ok 40 - 2-1
# inner 3
ok 41 - 3-1
ok 42 - test/global-harness-async.js
# independent-timeouts.js
# TAP version 13
# finishes in time
# finishes in time too
# tests 0
# ok
ok 43 - test/independent-timeouts.js
# isolated-conf-test.js
# TAP version 13
# one
ok 44 - should be equal
ok 45 - should be equal
# two
ok 46 - should be equal
ok 47 - should be equal
# tests 4
# pass  4
# ok
ok 48 - test/isolated-conf-test.js
# meta-test.js
# TAP version 13
# meta test
ok 49 - sanity check
ok 50 - not ok
ok 51 - total test count
ok 52 - tests passed
ok 53 - tests failed
ok 54 - ok is boolean
ok 55 - skip is number
ok 56 - results isa Results
ok 57 - test isa Test
ok 58 - test isa Harness
# tests 10
# pass  10
# ok
ok 59 - test/meta-test.js
# nested-async.js
# TAP version 13
# Harness async test support
ok 60 - sync child A
# sync child B
# async grandchild A
ok 61 - (unnamed assert)
# async grandchild B
ok 62 - (unnamed assert)
# async child
ok 63 - sync grandchild in async child A
# sync grandchild in async child B
ok 64 - (unnamed assert)
# tests 5
# pass  5
# ok
ok 65 - test/nested-async.js
# nested-test.js
# TAP version 13
# parent
ok 66 - p test
# subtest
ok 67 - ch test
# nested subtest
ok 68 - grch test
# another subtest
ok 69 - ch test 2
# tests 4
# pass  4
# ok
ok 70 - test/nested-test.js
# non-tap-output.js
# everything is fine
# there are no errors
# this output is not haiku.
# is 8 ok?
ok 71 - , 8 can stay.
# but: nevertheless, here we are
# this: is indented
# and: it
# might: ~
# be: yaml?
ok 72 - might be confusing
ok 73 - done now, exiting
ok 74 - test/non-tap-output.js
# not-executed.sh
# output-childtest-description.js
# TAP version 13
# nested tests, parent and child pass
ok 75 - outputs parent description
ok 76 - outputs child description
ok 77 - outputs parent description before parent result
ok 78 - outputs parent result before child description
ok 79 - outputs child description before child result
# tests 5
# pass  5
# ok
ok 80 - test/output-childtest-description.js
# result-trap.js
# TAP version 13
# trap result
ok 81 - should be equal
ok 82 - should be equal
# tests 2
# pass  2
# ok
ok 83 - test/result-trap.js
# segv.js
# TAP version 13
# setup
ok 84 - compiled seg faulter
# segv
ok 85 - should be equivalent
ok 86 - should be equivalent
ok 87 - should be equivalent
ok 88 - should be equivalent
ok 89 - should be equivalent
ok 90 - should be equal
# cleanup
ok 91 - cleaned up
# tests 8
# pass  8
# ok
ok 92 - test/segv.js
# simple-harness-test-with-plan.js
# TAP version 13
# trivial success
ok 93 - it works
# two tests
ok 94 - math should work
ok 95 - false should not be ok
# tests 3
# pass  3
# ok
ok 96 - test/simple-harness-test-with-plan.js
# simple-harness-test.js
# TAP version 13
# trivial success
ok 97 - it works
# two tests
ok 98 - math should work
ok 99 - false should not be ok
# tests 3
# pass  3
# ok
ok 100 - test/simple-harness-test.js
# test-assert-todo-skip.js
# TAP version 13
# not much
ok 101 - always passes # SKIP skip it good
ok 102 - false # SKIP always fails
ok 103 - bonus # TODO remove todo directive
ok 104 - expected # TODO implement a thing
ok 105 - always passes without explanation # SKIP
ok 106 - false without explanation # SKIP
ok 107 - bonus without explanation # TODO
ok 108 - expected without explanation # TODO
# tests 8
# skip  4
# todo  4
ok 109 - test/test-assert-todo-skip.js
# test-descriptions.js
# TAP version 13
# captures test descriptions
ok 110 - exit cleanly
ok 111 - captures SKIP description
ok 112 - skip summary is not from file
ok 113 - todo summary is not from file
not ok 114 - captures TODO description
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:32:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Socket.<anonymous> (child_process.js:1153:11)
    - |
      emitOne (events.js:77:13)
  ...
ok 115 - exit cleanly
ok 116 - captures SKIP description
ok 117 - skip summary is not in TAP output
ok 118 - todo summary is not in TAP output
not ok 119 - captures TODO description
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:18:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Process.ChildProcess._handle.onexit (child_process.js:1057:5)
  ...
ok 120 - exit cleanly
not ok 121 - summarizes skipped count
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:23:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Process.ChildProcess._handle.onexit (child_process.js:1057:5)
  ...
not ok 122 - summarizes todo count
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:24:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Process.ChildProcess._handle.onexit (child_process.js:1057:5)
  ...
# tests 13
# pass  9
# fail  4
not ok 123 - test/test-descriptions.js
  ---
  exit: 1
  command: /usr/local/bin/iojs test-descriptions.js
  ...
# test-directives.js
# TAP version 13
# captures test descriptions
# raw TAP > TAP consumer > TAP producer
ok 124 - overall result is PASS
ok 125 - captures ok SKIP
ok 126 - captures not ok SKIP
ok 127 - skip summary not in TAP output
ok 128 - captures ok TODO
ok 129 - captures not ok TODO
ok 130 - todo summary is not in TAP output
ok 131 - no ugly "undefined" in output
# raw TAP > TAP consumer > summary
ok 132 - overall result is PASS
ok 133 - no SKIP in summary
ok 134 - skip summary is not in TAP output
ok 135 - no TODO in summary
ok 136 - todo summary is not in TAP output
ok 137 - no ugly "undefined" in output
# TAP producer via require("tap")
ok 138 - overall result is PASS
ok 139 - captures ok SKIP
ok 140 - captures not ok SKIP
ok 141 - skip summary not in TAP output
ok 142 - captures ok TODO
ok 143 - captures not ok TODO
ok 144 - todo summary is not in TAP output
ok 145 - no ugly "undefined" in output
# tests 22
# pass  22
# ok
ok 146 - test/test-directives.js
# test-skip.js
# TAP version 13
ok 147 - does not count as failure # SKIP
# tests 1
# skip  1
ok 148 - test/test-skip.js
# test-test.js
# TAP version 13
# testing the test object
ok 149 - test object should be instanceof Test
ok 150 - test object should be instanceof Harness
ok 151 - test._Test should be the Test class
ok 152 - should have isNotDeepEqual method
ok 153 - isNotDeepEqual method should be a function
ok 154 - should have equals method
ok 155 - equals method should be a function
ok 156 - should have inequivalent method
ok 157 - inequivalent method should be a function
ok 158 - should have threw method
ok 159 - threw method should be a function
ok 160 - should have strictEqual method
ok 161 - strictEqual method should be a function
ok 162 - should have emit method
ok 163 - emit method should be a function
ok 164 - should have fail method
ok 165 - fail method should be a function
ok 166 - should have strictEquals method
ok 167 - strictEquals method should be a function
ok 168 - should have notLike method
ok 169 - notLike method should be a function
ok 170 - should have dissimilar method
ok 171 - dissimilar method should be a function
ok 172 - should have true method
ok 173 - true method should be a function
ok 174 - should have assert method
ok 175 - assert method should be a function
ok 176 - should have is method
ok 177 - is method should be a function
ok 178 - should have ok method
ok 179 - ok method should be a function
ok 180 - should have isEqual method
ok 181 - isEqual method should be a function
ok 182 - should have isDeeply method
ok 183 - isDeeply method should be a function
ok 184 - should have deepEqual method
ok 185 - deepEqual method should be a function
ok 186 - should have deepEquals method
ok 187 - deepEquals method should be a function
ok 188 - should have pass method
ok 189 - pass method should be a function
ok 190 - should have length method
ok 191 - length method should be a function
ok 192 - should have skip method
ok 193 - skip method should be a function
ok 194 - should have isNotEqual method
ok 195 - isNotEqual method should be a function
ok 196 - should have looseEquals method
ok 197 - looseEquals method should be a function
ok 198 - should have false method
ok 199 - false method should be a function
ok 200 - should have notDeeply method
ok 201 - notDeeply method should be a function
ok 202 - should have ifErr method
ok 203 - ifErr method should be a function
ok 204 - should have hasFields method
ok 205 - hasFields method should be a function
ok 206 - should have isNotDeeply method
ok 207 - isNotDeeply method should be a function
ok 208 - should have like method
ok 209 - like method should be a function
ok 210 - should have similar method
ok 211 - similar method should be a function
ok 212 - should have notOk method
ok 213 - notOk method should be a function
ok 214 - should have isDissimilar method
ok 215 - isDissimilar method should be a function
ok 216 - should have isEquivalent method
ok 217 - isEquivalent method should be a function
ok 218 - should have doesNotEqual method
ok 219 - doesNotEqual method should be a function
ok 220 - should have isSimilar method
ok 221 - isSimilar method should be a function
ok 222 - should have notDeepEqual method
ok 223 - notDeepEqual method should be a function
ok 224 - should have type method
ok 225 - type method should be a function
ok 226 - should have notok method
ok 227 - notok method should be a function
ok 228 - should have isInequivalent method
ok 229 - isInequivalent method should be a function
ok 230 - should have isNot method
ok 231 - isNot method should be a function
ok 232 - should have same method
ok 233 - same method should be a function
ok 234 - should have isInequal method
ok 235 - isInequal method should be a function
ok 236 - should have _endNice method
ok 237 - _endNice method should be a function
ok 238 - should have ifError method
ok 239 - ifError method should be a function
ok 240 - should have iferror method
ok 241 - iferror method should be a function
ok 242 - should have clear method
ok 243 - clear method should be a function
ok 244 - should have has method
ok 245 - has method should be a function
ok 246 - should have not method
ok 247 - not method should be a function
ok 248 - should have timeout method
ok 249 - timeout method should be a function
ok 250 - should have notSimilar method
ok 251 - notSimilar method should be a function
ok 252 - should have isUnlike method
ok 253 - isUnlike method should be a function
ok 254 - should have notEquals method
ok 255 - notEquals method should be a function
ok 256 - should have unsimilar method
ok 257 - unsimilar method should be a function
ok 258 - should have result method
ok 259 - result method should be a function
ok 260 - should have doesNotThrow method
ok 261 - doesNotThrow method should be a function
ok 262 - should have error method
ok 263 - error method should be a function
ok 264 - should have constructor method
ok 265 - constructor method should be a function
ok 266 - should have notEqual method
ok 267 - notEqual method should be a function
ok 268 - should have throws method
ok 269 - throws method should be a function
ok 270 - should have isLike method
ok 271 - isLike method should be a function
ok 272 - should have isNotSimilar method
ok 273 - isNotSimilar method should be a function
ok 274 - should have isNotEquivalent method
ok 275 - isNotEquivalent method should be a function
ok 276 - should have inequal method
ok 277 - inequal method should be a function
ok 278 - should have notEquivalent method
ok 279 - notEquivalent method should be a function
ok 280 - should have isNotLike method
ok 281 - isNotLike method should be a function
ok 282 - should have equivalent method
ok 283 - equivalent method should be a function
ok 284 - should have looseEqual method
ok 285 - looseEqual method should be a function
ok 286 - should have equal method
ok 287 - equal method should be a function
ok 288 - should have unlike method
ok 289 - unlike method should be a function
ok 290 - should have doesNotHave method
ok 291 - doesNotHave method should be a function
ok 292 - should have comment method
ok 293 - comment method should be a function
ok 294 - should have isa method
ok 295 - isa method should be a function
# tests 147
# pass  147
# ok
ok 296 - test/test-test.js
# timeout.js
timeout test
t.plan=2
a assert
b assert
# TAP version 13
# timeout test with plan only
ok 297 - a
timeout test
t.plan=2
a assert
b assert
ok 298 - b
# timeout test with plan and end
ok 299 - a
ok 300 - b
# tests 4
# pass  4
# ok
ok 301 - test/timeout.js
# trivial-success.js
ok 302 - test/trivial-success.js
# undefined_indented.js
# TAP version 13
# consume yaml
ok 303 - should be equivalent
# tests 1
# pass  1
# ok
ok 304 - test/undefined_indented.js
# valid-command.js
# TAP version 13
# valid command
ok 305 - should be equivalent
ok 306 - should be equivalent
ok 307 - should be equivalent
ok 308 - should be equivalent
ok 309 - should be equivalent
ok 310 - should be equal
# tests 6
# pass  6
# ok
ok 311 - test/valid-command.js
1..311
# tests 311
# pass  297
# fail  5
# skip  5
# todo  4

`

exports[`test/parse-stringify.ts > TAP > tap-tests.tap > strictBail > parsed 1`] = `
Array [
  Array [
    "version",
    13,
  ],
  Array [
    "comment",
    "# buffer_compare.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# same buffers\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 1,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# not same buffers\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should not be equivalent",
      "id": 2,
      "name": "should not be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 2\\n",
  ],
  Array [
    "comment",
    "# pass  2\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/buffer_compare.js",
      "id": 3,
      "name": "test/buffer_compare.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# common.js\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "just setup, nothing relevant",
      "id": 4,
      "name": "just setup, nothing relevant",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/common.js",
      "id": 5,
      "name": "test/common.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# consumer.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# basic.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 6,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# indent.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 7,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# missing.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 8,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# skip-all.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 9,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# yamlish.tap\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 10,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 5\\n",
  ],
  Array [
    "comment",
    "# pass  5\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/consumer.js",
      "id": 11,
      "name": "test/consumer.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# debug-test.js\\n",
  ],
  Array [
    "extra",
    "debug test\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "'Debugger listening on port 5858\\\\n'\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# debug test\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "Should output debugger message",
      "id": 12,
      "name": "Should output debugger message",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 1\\n",
  ],
  Array [
    "comment",
    "# pass  1\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/debug-test.js",
      "id": 13,
      "name": "test/debug-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deep-strict.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# strictDeepEquals shouldn't care about key order\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be strictly equal",
      "id": 14,
      "name": "should be strictly equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# strictDeepEquals shouldn't care about key order recursively\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be strictly equal",
      "id": 15,
      "name": "should be strictly equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# strictDeepEquals shoudn't care about key order (but still might)\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be strictly equal",
      "id": 16,
      "name": "should be strictly equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/deep-strict.js",
      "id": 17,
      "name": "test/deep-strict.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deep.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# deepEquals shouldn't care about key order and types\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 18,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deepEquals shouldn't care about key order recursively and types\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 19,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# deepEquals shoudn't care about key order (but still might) and types\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 20,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/deep.js",
      "id": 21,
      "name": "test/deep.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# executed.sh\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "File with executable bit should be executed",
      "id": 22,
      "name": "File with executable bit should be executed",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/executed.sh",
      "id": 23,
      "name": "test/executed.sh",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# exit-code.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# exit code 1 when tap results show failure\\n",
  ],
  Array [
    "comment",
    "# test exits 0, has failures\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 24,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 25,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test exits 1, has failures\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 26,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 27,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test exits 1, has no failures\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 28,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 29,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# successes exit 0\\n",
  ],
  Array [
    "comment",
    "# test that does nothing, but exits 0\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 30,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 31,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test that succeeds, and exits 0\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 32,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 33,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 10\\n",
  ],
  Array [
    "comment",
    "# pass  10\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/exit-code.js",
      "id": 34,
      "name": "test/exit-code.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# expose-gc-test.js\\n",
  ],
  Array [
    "extra",
    "gc test\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "assert gc does not exist\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# gc test when the gc isn't there\\n",
  ],
  Array [
    "extra",
    "gc test\\n",
  ],
  Array [
    "extra",
    "t.plan=2\\n",
  ],
  Array [
    "extra",
    "gc test using --gc\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "assert gc exists\\n",
  ],
  Array [
    "extra",
    "gc test using --expose-gc\\n",
  ],
  Array [
    "extra",
    "t.plan=1\\n",
  ],
  Array [
    "extra",
    "assert gc exists\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 35,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# gc test when the gc should be there\\n",
  ],
  Array [
    "comment",
    "# test for gc using --gc\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 36,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test for gc using --expose-gc\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 37,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# cleanup\\n",
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/expose-gc-test.js",
      "id": 38,
      "name": "test/expose-gc-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# global-harness-async.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# outer\\n",
  ],
  Array [
    "comment",
    "# inner 1\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "1-1",
      "id": 39,
      "name": "1-1",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# inner 2\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "2-1",
      "id": 40,
      "name": "2-1",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# inner 3\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "3-1",
      "id": 41,
      "name": "3-1",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/global-harness-async.js",
      "id": 42,
      "name": "test/global-harness-async.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# independent-timeouts.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# finishes in time\\n",
  ],
  Array [
    "comment",
    "# finishes in time too\\n",
  ],
  Array [
    "comment",
    "# tests 0\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/independent-timeouts.js",
      "id": 43,
      "name": "test/independent-timeouts.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# isolated-conf-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# one\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 44,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 45,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# two\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 46,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 47,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 4\\n",
  ],
  Array [
    "comment",
    "# pass  4\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/isolated-conf-test.js",
      "id": 48,
      "name": "test/isolated-conf-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# meta-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# meta test\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "sanity check",
      "id": 49,
      "name": "sanity check",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "not ok",
      "id": 50,
      "name": "not ok",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "total test count",
      "id": 51,
      "name": "total test count",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "tests passed",
      "id": 52,
      "name": "tests passed",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "tests failed",
      "id": 53,
      "name": "tests failed",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ok is boolean",
      "id": 54,
      "name": "ok is boolean",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip is number",
      "id": 55,
      "name": "skip is number",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "results isa Results",
      "id": 56,
      "name": "results isa Results",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test isa Test",
      "id": 57,
      "name": "test isa Test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test isa Harness",
      "id": 58,
      "name": "test isa Harness",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 10\\n",
  ],
  Array [
    "comment",
    "# pass  10\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/meta-test.js",
      "id": 59,
      "name": "test/meta-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# nested-async.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# Harness async test support\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "sync child A",
      "id": 60,
      "name": "sync child A",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# sync child B\\n",
  ],
  Array [
    "comment",
    "# async grandchild A\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "(unnamed assert)",
      "id": 61,
      "name": "(unnamed assert)",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# async grandchild B\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "(unnamed assert)",
      "id": 62,
      "name": "(unnamed assert)",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# async child\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "sync grandchild in async child A",
      "id": 63,
      "name": "sync grandchild in async child A",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# sync grandchild in async child B\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "(unnamed assert)",
      "id": 64,
      "name": "(unnamed assert)",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 5\\n",
  ],
  Array [
    "comment",
    "# pass  5\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/nested-async.js",
      "id": 65,
      "name": "test/nested-async.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# nested-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# parent\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "p test",
      "id": 66,
      "name": "p test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# subtest\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ch test",
      "id": 67,
      "name": "ch test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# nested subtest\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "grch test",
      "id": 68,
      "name": "grch test",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# another subtest\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "ch test 2",
      "id": 69,
      "name": "ch test 2",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 4\\n",
  ],
  Array [
    "comment",
    "# pass  4\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/nested-test.js",
      "id": 70,
      "name": "test/nested-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# non-tap-output.js\\n",
  ],
  Array [
    "comment",
    "# everything is fine\\n",
  ],
  Array [
    "comment",
    "# there are no errors\\n",
  ],
  Array [
    "comment",
    "# this output is not haiku.\\n",
  ],
  Array [
    "comment",
    "# is 8 ok?\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": ", 8 can stay.",
      "id": 71,
      "name": ", 8 can stay.",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# but: nevertheless, here we are\\n",
  ],
  Array [
    "comment",
    "# this: is indented\\n",
  ],
  Array [
    "comment",
    "# and: it\\n",
  ],
  Array [
    "comment",
    "# might: ~\\n",
  ],
  Array [
    "comment",
    "# be: yaml?\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "might be confusing",
      "id": 72,
      "name": "might be confusing",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "done now, exiting",
      "id": 73,
      "name": "done now, exiting",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/non-tap-output.js",
      "id": 74,
      "name": "test/non-tap-output.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# not-executed.sh\\n",
  ],
  Array [
    "comment",
    "# output-childtest-description.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# nested tests, parent and child pass\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs parent description",
      "id": 75,
      "name": "outputs parent description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs child description",
      "id": 76,
      "name": "outputs child description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs parent description before parent result",
      "id": 77,
      "name": "outputs parent description before parent result",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs parent result before child description",
      "id": 78,
      "name": "outputs parent result before child description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "outputs child description before child result",
      "id": 79,
      "name": "outputs child description before child result",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 5\\n",
  ],
  Array [
    "comment",
    "# pass  5\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/output-childtest-description.js",
      "id": 80,
      "name": "test/output-childtest-description.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# result-trap.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# trap result\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 81,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 82,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 2\\n",
  ],
  Array [
    "comment",
    "# pass  2\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/result-trap.js",
      "id": 83,
      "name": "test/result-trap.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# segv.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# setup\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "compiled seg faulter",
      "id": 84,
      "name": "compiled seg faulter",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# segv\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 85,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 86,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 87,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 88,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equivalent",
      "id": 89,
      "name": "should be equivalent",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "should be equal",
      "id": 90,
      "name": "should be equal",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# cleanup\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "cleaned up",
      "id": 91,
      "name": "cleaned up",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 8\\n",
  ],
  Array [
    "comment",
    "# pass  8\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/segv.js",
      "id": 92,
      "name": "test/segv.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# simple-harness-test-with-plan.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# trivial success\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "it works",
      "id": 93,
      "name": "it works",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# two tests\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "math should work",
      "id": 94,
      "name": "math should work",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false should not be ok",
      "id": 95,
      "name": "false should not be ok",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/simple-harness-test-with-plan.js",
      "id": 96,
      "name": "test/simple-harness-test-with-plan.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# simple-harness-test.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# trivial success\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "it works",
      "id": 97,
      "name": "it works",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# two tests\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "math should work",
      "id": 98,
      "name": "math should work",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false should not be ok",
      "id": 99,
      "name": "false should not be ok",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# tests 3\\n",
  ],
  Array [
    "comment",
    "# pass  3\\n",
  ],
  Array [
    "comment",
    "# ok\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/simple-harness-test.js",
      "id": 100,
      "name": "test/simple-harness-test.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-assert-todo-skip.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# not much\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "always passes",
      "id": 101,
      "name": "always passes",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": "skip it good",
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false",
      "id": 102,
      "name": "false",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": "always fails",
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "bonus",
      "id": 103,
      "name": "bonus",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": "remove todo directive",
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "expected",
      "id": 104,
      "name": "expected",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": "implement a thing",
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "always passes without explanation",
      "id": 105,
      "name": "always passes without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": true,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "false without explanation",
      "id": 106,
      "name": "false without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": true,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "bonus without explanation",
      "id": 107,
      "name": "bonus without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": true,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "expected without explanation",
      "id": 108,
      "name": "expected without explanation",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": true,
    },
  ],
  Array [
    "comment",
    "# tests 8\\n",
  ],
  Array [
    "comment",
    "# skip  4\\n",
  ],
  Array [
    "comment",
    "# todo  4\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "test/test-assert-todo-skip.js",
      "id": 109,
      "name": "test/test-assert-todo-skip.js",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "comment",
    "# test-descriptions.js\\n",
  ],
  Array [
    "comment",
    "# TAP version 13\\n",
  ],
  Array [
    "comment",
    "# captures test descriptions\\n",
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "exit cleanly",
      "id": 110,
      "name": "exit cleanly",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "captures SKIP description",
      "id": 111,
      "name": "captures SKIP description",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "skip summary is not from file",
      "id": 112,
      "name": "skip summary is not from file",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": null,
      "fullname": "todo summary is not from file",
      "id": 113,
      "name": "todo summary is not from file",
      "ok": true,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "assert",
    Result {
      "buffered": false,
      "closingTestPoint": false,
      "diag": Object {
        "column": 7,
        "file": "child_process.js",
        "line": 707,
        "stack": Array [
          "getCaller (/some/path/tap-assert.js:439:17)\\n",
          "Function.assert (/some/path/tap-assert.js:21:16)\\n",
          "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
          "/some/path/test/test-descriptions.js:32:7\\n",
          "ChildProcess.exithandler (child_process.js:707:7)\\n",
          "emitTwo (events.js:87:13)\\n",
          "ChildProcess.emit (events.js:169:7)\\n",
          "maybeClose (child_process.js:984:16)\\n",
          "Socket.<anonymous> (child_process.js:1153:11)\\n",
          "emitOne (events.js:77:13)\\n",
        ],
      },
      "fullname": "captures TODO description",
      "id": 114,
      "name": "captures TODO description",
      "ok": false,
      "plan": null,
      "previous": null,
      "skip": false,
      "tapError": null,
      "time": null,
      "todo": false,
    },
  ],
  Array [
    "bailout",
    "captures TODO description",
  ],
  Array [
    "complete",
    FinalResults {
      "bailout": "captures TODO description",
      "count": 114,
      "fail": 15,
      "failures": Array [
        Object {
          "data": "debug test\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "t.plan=1\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "'Debugger listening on port 5858\\\\n'\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "gc test\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "t.plan=1\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "assert gc does not exist\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "gc test\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "t.plan=2\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "gc test using --gc\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "t.plan=1\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "assert gc exists\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "gc test using --expose-gc\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "t.plan=1\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Object {
          "data": "assert gc exists\\n",
          "tapError": "Non-TAP data encountered in strict mode",
        },
        Result {
          "buffered": false,
          "closingTestPoint": false,
          "diag": Object {
            "column": 7,
            "file": "child_process.js",
            "line": 707,
            "stack": Array [
              "getCaller (/some/path/tap-assert.js:439:17)\\n",
              "Function.assert (/some/path/tap-assert.js:21:16)\\n",
              "Test._testAssert (/some/path/tap-test.js:87:16)\\n",
              "/some/path/test/test-descriptions.js:32:7\\n",
              "ChildProcess.exithandler (child_process.js:707:7)\\n",
              "emitTwo (events.js:87:13)\\n",
              "ChildProcess.emit (events.js:169:7)\\n",
              "maybeClose (child_process.js:984:16)\\n",
              "Socket.<anonymous> (child_process.js:1153:11)\\n",
              "emitOne (events.js:77:13)\\n",
            ],
          },
          "fullname": "captures TODO description",
          "id": 114,
          "name": "captures TODO description",
          "ok": false,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": false,
        },
      ],
      "ok": false,
      "pass": 113,
      "passes": undefined,
      "plan": FinalPlan {
        "comment": "",
        "end": null,
        "skipAll": false,
        "skipReason": "",
        "start": null,
      },
      "skip": 4,
      "skips": Array [
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "always passes",
          "id": 101,
          "name": "always passes",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": "skip it good",
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "false",
          "id": 102,
          "name": "false",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": "always fails",
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "always passes without explanation",
          "id": 105,
          "name": "always passes without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": true,
          "tapError": null,
          "time": null,
          "todo": false,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "false without explanation",
          "id": 106,
          "name": "false without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": true,
          "tapError": null,
          "time": null,
          "todo": false,
        },
      ],
      "time": null,
      "todo": 4,
      "todos": Array [
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "bonus",
          "id": 103,
          "name": "bonus",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": "remove todo directive",
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "expected",
          "id": 104,
          "name": "expected",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": "implement a thing",
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "bonus without explanation",
          "id": 107,
          "name": "bonus without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": true,
        },
        Object {
          "buffered": false,
          "closingTestPoint": false,
          "diag": null,
          "fullname": "expected without explanation",
          "id": 108,
          "name": "expected without explanation",
          "ok": true,
          "plan": null,
          "previous": null,
          "skip": false,
          "tapError": null,
          "time": null,
          "todo": true,
        },
      ],
    },
  ],
  Array [
    "finish",
  ],
  Array [
    "close",
  ],
]
`

exports[`test/parse-stringify.ts > TAP > tap-tests.tap > strictBail > stringified 1`] = `
TAP version 13
# buffer_compare.js
# TAP version 13
# same buffers
ok 1 - should be equivalent
# not same buffers
ok 2 - should not be equivalent
# tests 2
# pass  2
# ok
ok 3 - test/buffer_compare.js
# common.js
ok 4 - just setup, nothing relevant
ok 5 - test/common.js
# consumer.js
# TAP version 13
# basic.tap
ok 6 - should be equivalent
# indent.tap
ok 7 - should be equivalent
# missing.tap
ok 8 - should be equivalent
# skip-all.tap
ok 9 - should be equivalent
# yamlish.tap
ok 10 - should be equivalent
# tests 5
# pass  5
# ok
ok 11 - test/consumer.js
# debug-test.js
debug test
t.plan=1
'Debugger listening on port 5858\\n'
# TAP version 13
# debug test
ok 12 - Should output debugger message
# tests 1
# pass  1
# ok
ok 13 - test/debug-test.js
# deep-strict.js
# TAP version 13
# strictDeepEquals shouldn't care about key order
ok 14 - should be strictly equal
# strictDeepEquals shouldn't care about key order recursively
ok 15 - should be strictly equal
# strictDeepEquals shoudn't care about key order (but still might)
ok 16 - should be strictly equal
# tests 3
# pass  3
# ok
ok 17 - test/deep-strict.js
# deep.js
# TAP version 13
# deepEquals shouldn't care about key order and types
ok 18 - should be equivalent
# deepEquals shouldn't care about key order recursively and types
ok 19 - should be equivalent
# deepEquals shoudn't care about key order (but still might) and types
ok 20 - should be equivalent
# tests 3
# pass  3
# ok
ok 21 - test/deep.js
# executed.sh
ok 22 - File with executable bit should be executed
ok 23 - test/executed.sh
# exit-code.js
# TAP version 13
# exit code 1 when tap results show failure
# test exits 0, has failures
ok 24 - should be equal
ok 25 - should be equal
# test exits 1, has failures
ok 26 - should be equal
ok 27 - should be equal
# test exits 1, has no failures
ok 28 - should be equal
ok 29 - should be equal
# successes exit 0
# test that does nothing, but exits 0
ok 30 - should be equal
ok 31 - should be equal
# test that succeeds, and exits 0
ok 32 - should be equal
ok 33 - should be equal
# tests 10
# pass  10
# ok
ok 34 - test/exit-code.js
# expose-gc-test.js
gc test
t.plan=1
assert gc does not exist
# TAP version 13
# gc test when the gc isn't there
gc test
t.plan=2
gc test using --gc
t.plan=1
assert gc exists
gc test using --expose-gc
t.plan=1
assert gc exists
ok 35 - should be equal
# gc test when the gc should be there
# test for gc using --gc
ok 36 - should be equal
# test for gc using --expose-gc
ok 37 - should be equal
# cleanup
# tests 3
# pass  3
# ok
ok 38 - test/expose-gc-test.js
# global-harness-async.js
# TAP version 13
# outer
# inner 1
ok 39 - 1-1
# inner 2
ok 40 - 2-1
# inner 3
ok 41 - 3-1
ok 42 - test/global-harness-async.js
# independent-timeouts.js
# TAP version 13
# finishes in time
# finishes in time too
# tests 0
# ok
ok 43 - test/independent-timeouts.js
# isolated-conf-test.js
# TAP version 13
# one
ok 44 - should be equal
ok 45 - should be equal
# two
ok 46 - should be equal
ok 47 - should be equal
# tests 4
# pass  4
# ok
ok 48 - test/isolated-conf-test.js
# meta-test.js
# TAP version 13
# meta test
ok 49 - sanity check
ok 50 - not ok
ok 51 - total test count
ok 52 - tests passed
ok 53 - tests failed
ok 54 - ok is boolean
ok 55 - skip is number
ok 56 - results isa Results
ok 57 - test isa Test
ok 58 - test isa Harness
# tests 10
# pass  10
# ok
ok 59 - test/meta-test.js
# nested-async.js
# TAP version 13
# Harness async test support
ok 60 - sync child A
# sync child B
# async grandchild A
ok 61 - (unnamed assert)
# async grandchild B
ok 62 - (unnamed assert)
# async child
ok 63 - sync grandchild in async child A
# sync grandchild in async child B
ok 64 - (unnamed assert)
# tests 5
# pass  5
# ok
ok 65 - test/nested-async.js
# nested-test.js
# TAP version 13
# parent
ok 66 - p test
# subtest
ok 67 - ch test
# nested subtest
ok 68 - grch test
# another subtest
ok 69 - ch test 2
# tests 4
# pass  4
# ok
ok 70 - test/nested-test.js
# non-tap-output.js
# everything is fine
# there are no errors
# this output is not haiku.
# is 8 ok?
ok 71 - , 8 can stay.
# but: nevertheless, here we are
# this: is indented
# and: it
# might: ~
# be: yaml?
ok 72 - might be confusing
ok 73 - done now, exiting
ok 74 - test/non-tap-output.js
# not-executed.sh
# output-childtest-description.js
# TAP version 13
# nested tests, parent and child pass
ok 75 - outputs parent description
ok 76 - outputs child description
ok 77 - outputs parent description before parent result
ok 78 - outputs parent result before child description
ok 79 - outputs child description before child result
# tests 5
# pass  5
# ok
ok 80 - test/output-childtest-description.js
# result-trap.js
# TAP version 13
# trap result
ok 81 - should be equal
ok 82 - should be equal
# tests 2
# pass  2
# ok
ok 83 - test/result-trap.js
# segv.js
# TAP version 13
# setup
ok 84 - compiled seg faulter
# segv
ok 85 - should be equivalent
ok 86 - should be equivalent
ok 87 - should be equivalent
ok 88 - should be equivalent
ok 89 - should be equivalent
ok 90 - should be equal
# cleanup
ok 91 - cleaned up
# tests 8
# pass  8
# ok
ok 92 - test/segv.js
# simple-harness-test-with-plan.js
# TAP version 13
# trivial success
ok 93 - it works
# two tests
ok 94 - math should work
ok 95 - false should not be ok
# tests 3
# pass  3
# ok
ok 96 - test/simple-harness-test-with-plan.js
# simple-harness-test.js
# TAP version 13
# trivial success
ok 97 - it works
# two tests
ok 98 - math should work
ok 99 - false should not be ok
# tests 3
# pass  3
# ok
ok 100 - test/simple-harness-test.js
# test-assert-todo-skip.js
# TAP version 13
# not much
ok 101 - always passes # SKIP skip it good
ok 102 - false # SKIP always fails
ok 103 - bonus # TODO remove todo directive
ok 104 - expected # TODO implement a thing
ok 105 - always passes without explanation # SKIP
ok 106 - false without explanation # SKIP
ok 107 - bonus without explanation # TODO
ok 108 - expected without explanation # TODO
# tests 8
# skip  4
# todo  4
ok 109 - test/test-assert-todo-skip.js
# test-descriptions.js
# TAP version 13
# captures test descriptions
ok 110 - exit cleanly
ok 111 - captures SKIP description
ok 112 - skip summary is not from file
ok 113 - todo summary is not from file
not ok 114 - captures TODO description
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:32:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Socket.<anonymous> (child_process.js:1153:11)
    - |
      emitOne (events.js:77:13)
  ...
Bail out! captures TODO description

`

exports[`test/parse-stringify.ts > TAP > tap-tests.tap > strictBail > stringified flat 1`] = `
TAP version 13
# buffer_compare.js
# TAP version 13
# same buffers
ok 1 - should be equivalent
# not same buffers
ok 2 - should not be equivalent
# tests 2
# pass  2
# ok
ok 3 - test/buffer_compare.js
# common.js
ok 4 - just setup, nothing relevant
ok 5 - test/common.js
# consumer.js
# TAP version 13
# basic.tap
ok 6 - should be equivalent
# indent.tap
ok 7 - should be equivalent
# missing.tap
ok 8 - should be equivalent
# skip-all.tap
ok 9 - should be equivalent
# yamlish.tap
ok 10 - should be equivalent
# tests 5
# pass  5
# ok
ok 11 - test/consumer.js
# debug-test.js
debug test
t.plan=1
'Debugger listening on port 5858\\n'
# TAP version 13
# debug test
ok 12 - Should output debugger message
# tests 1
# pass  1
# ok
ok 13 - test/debug-test.js
# deep-strict.js
# TAP version 13
# strictDeepEquals shouldn't care about key order
ok 14 - should be strictly equal
# strictDeepEquals shouldn't care about key order recursively
ok 15 - should be strictly equal
# strictDeepEquals shoudn't care about key order (but still might)
ok 16 - should be strictly equal
# tests 3
# pass  3
# ok
ok 17 - test/deep-strict.js
# deep.js
# TAP version 13
# deepEquals shouldn't care about key order and types
ok 18 - should be equivalent
# deepEquals shouldn't care about key order recursively and types
ok 19 - should be equivalent
# deepEquals shoudn't care about key order (but still might) and types
ok 20 - should be equivalent
# tests 3
# pass  3
# ok
ok 21 - test/deep.js
# executed.sh
ok 22 - File with executable bit should be executed
ok 23 - test/executed.sh
# exit-code.js
# TAP version 13
# exit code 1 when tap results show failure
# test exits 0, has failures
ok 24 - should be equal
ok 25 - should be equal
# test exits 1, has failures
ok 26 - should be equal
ok 27 - should be equal
# test exits 1, has no failures
ok 28 - should be equal
ok 29 - should be equal
# successes exit 0
# test that does nothing, but exits 0
ok 30 - should be equal
ok 31 - should be equal
# test that succeeds, and exits 0
ok 32 - should be equal
ok 33 - should be equal
# tests 10
# pass  10
# ok
ok 34 - test/exit-code.js
# expose-gc-test.js
gc test
t.plan=1
assert gc does not exist
# TAP version 13
# gc test when the gc isn't there
gc test
t.plan=2
gc test using --gc
t.plan=1
assert gc exists
gc test using --expose-gc
t.plan=1
assert gc exists
ok 35 - should be equal
# gc test when the gc should be there
# test for gc using --gc
ok 36 - should be equal
# test for gc using --expose-gc
ok 37 - should be equal
# cleanup
# tests 3
# pass  3
# ok
ok 38 - test/expose-gc-test.js
# global-harness-async.js
# TAP version 13
# outer
# inner 1
ok 39 - 1-1
# inner 2
ok 40 - 2-1
# inner 3
ok 41 - 3-1
ok 42 - test/global-harness-async.js
# independent-timeouts.js
# TAP version 13
# finishes in time
# finishes in time too
# tests 0
# ok
ok 43 - test/independent-timeouts.js
# isolated-conf-test.js
# TAP version 13
# one
ok 44 - should be equal
ok 45 - should be equal
# two
ok 46 - should be equal
ok 47 - should be equal
# tests 4
# pass  4
# ok
ok 48 - test/isolated-conf-test.js
# meta-test.js
# TAP version 13
# meta test
ok 49 - sanity check
ok 50 - not ok
ok 51 - total test count
ok 52 - tests passed
ok 53 - tests failed
ok 54 - ok is boolean
ok 55 - skip is number
ok 56 - results isa Results
ok 57 - test isa Test
ok 58 - test isa Harness
# tests 10
# pass  10
# ok
ok 59 - test/meta-test.js
# nested-async.js
# TAP version 13
# Harness async test support
ok 60 - sync child A
# sync child B
# async grandchild A
ok 61 - (unnamed assert)
# async grandchild B
ok 62 - (unnamed assert)
# async child
ok 63 - sync grandchild in async child A
# sync grandchild in async child B
ok 64 - (unnamed assert)
# tests 5
# pass  5
# ok
ok 65 - test/nested-async.js
# nested-test.js
# TAP version 13
# parent
ok 66 - p test
# subtest
ok 67 - ch test
# nested subtest
ok 68 - grch test
# another subtest
ok 69 - ch test 2
# tests 4
# pass  4
# ok
ok 70 - test/nested-test.js
# non-tap-output.js
# everything is fine
# there are no errors
# this output is not haiku.
# is 8 ok?
ok 71 - , 8 can stay.
# but: nevertheless, here we are
# this: is indented
# and: it
# might: ~
# be: yaml?
ok 72 - might be confusing
ok 73 - done now, exiting
ok 74 - test/non-tap-output.js
# not-executed.sh
# output-childtest-description.js
# TAP version 13
# nested tests, parent and child pass
ok 75 - outputs parent description
ok 76 - outputs child description
ok 77 - outputs parent description before parent result
ok 78 - outputs parent result before child description
ok 79 - outputs child description before child result
# tests 5
# pass  5
# ok
ok 80 - test/output-childtest-description.js
# result-trap.js
# TAP version 13
# trap result
ok 81 - should be equal
ok 82 - should be equal
# tests 2
# pass  2
# ok
ok 83 - test/result-trap.js
# segv.js
# TAP version 13
# setup
ok 84 - compiled seg faulter
# segv
ok 85 - should be equivalent
ok 86 - should be equivalent
ok 87 - should be equivalent
ok 88 - should be equivalent
ok 89 - should be equivalent
ok 90 - should be equal
# cleanup
ok 91 - cleaned up
# tests 8
# pass  8
# ok
ok 92 - test/segv.js
# simple-harness-test-with-plan.js
# TAP version 13
# trivial success
ok 93 - it works
# two tests
ok 94 - math should work
ok 95 - false should not be ok
# tests 3
# pass  3
# ok
ok 96 - test/simple-harness-test-with-plan.js
# simple-harness-test.js
# TAP version 13
# trivial success
ok 97 - it works
# two tests
ok 98 - math should work
ok 99 - false should not be ok
# tests 3
# pass  3
# ok
ok 100 - test/simple-harness-test.js
# test-assert-todo-skip.js
# TAP version 13
# not much
ok 101 - always passes # SKIP skip it good
ok 102 - false # SKIP always fails
ok 103 - bonus # TODO remove todo directive
ok 104 - expected # TODO implement a thing
ok 105 - always passes without explanation # SKIP
ok 106 - false without explanation # SKIP
ok 107 - bonus without explanation # TODO
ok 108 - expected without explanation # TODO
# tests 8
# skip  4
# todo  4
ok 109 - test/test-assert-todo-skip.js
# test-descriptions.js
# TAP version 13
# captures test descriptions
ok 110 - exit cleanly
ok 111 - captures SKIP description
ok 112 - skip summary is not from file
ok 113 - todo summary is not from file
not ok 114 - captures TODO description
  ---
  file: child_process.js
  line: 707
  column: 7
  stack:
    - |
      getCaller (/some/path/tap-assert.js:439:17)
    - |
      Function.assert (/some/path/tap-assert.js:21:16)
    - |
      Test._testAssert (/some/path/tap-test.js:87:16)
    - |
      /some/path/test/test-descriptions.js:32:7
    - |
      ChildProcess.exithandler (child_process.js:707:7)
    - |
      emitTwo (events.js:87:13)
    - |
      ChildProcess.emit (events.js:169:7)
    - |
      maybeClose (child_process.js:984:16)
    - |
      Socket.<anonymous> (child_process.js:1153:11)
    - |
      emitOne (events.js:77:13)
  ...
Bail out! captures TODO description

`
